@@ -3,19 +3,37 @@
 #       Buddy pouncing...
 #       This is just a first sketch.
 #       TODO: all message translations.
+#
+#  Copyright (c) 2007-2008 Mats Bengtsson
+#  
+#   This program is free software: you can redistribute it and/or modify
+#   it under the terms of the GNU General Public License as published by
+#   the Free Software Foundation, either version 3 of the License, or
+#   (at your option) any later version.
+#   
+#   This program is distributed in the hope that it will be useful,
+#   but WITHOUT ANY WARRANTY; without even the implied warranty of
+#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#   GNU General Public License for more details.
+#   
+#   You should have received a copy of the GNU General Public License
+#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #       
-# $Id: BuddyPounce.tcl,v 1.7 2004/11/27 14:52:52 matben Exp $
+# $Id: BuddyPounce.tcl,v 1.33 2008-07-22 07:09:49 matben Exp $
 
 # Key phrases are: 
 #     event:    something happens, presence change, incoming message etc.
 #     target:   is either a jid, a roster group, or 'any'  
 #     action:   how to respond, popup, sound, reply etc.
 
-namespace eval ::BuddyPounce:: {
+package require jlib
+
+namespace eval ::BuddyPounce {
     
+    component::define BuddyPounce "Set actions for contact events"
 }
 
-proc ::BuddyPounce::Init { } {
+proc ::BuddyPounce::Init {} {
     global  this
     
     ::Debug 2 "::BuddyPounce::Init"
@@ -26,57 +44,28 @@ proc ::BuddyPounce::Init { } {
     ::hooks::register newChatMessageHook       ::BuddyPounce::NewChatMsgHook
     ::hooks::register presenceHook             ::BuddyPounce::PresenceHook    
     ::hooks::register prefsInitHook            ::BuddyPounce::InitPrefsHook
-    ::hooks::register closeWindowHook          ::BuddyPounce::CloseHook
     
     # Register popmenu entry.
-    set popMenuSpec \
-      [list {Buddy Pouncing} any {::BuddyPounce::Build $typesel $jid $group}]
-    ::Jabber::UI::RegisterPopupEntry roster $popMenuSpec
+    # TRANSLATORS: actions to be execute on an event related to a contact (buddy pounces); see right-mouse click menu in the main window
+    set menuDef {
+	command mContactActions... {[mc "&Contact Actions"]...} {::BuddyPounce::Build $clicked $jidL $group}
+    }
+    set menuType {
+	mContactActions... {group user}
+    }
+    set menuType {
+	mContactActions... {}
+    }
+    ::Roster::RegisterPopupEntry $menuDef $menuType
     
-    component::register BuddyPounce  \
-      "Buddy pouncing enables you to make various things happen when\
-      a particular user becomes online, offline etc."
+    component::register BuddyPounce
         
-    # Audio formats.
-    variable audioSuffixes {}
-    
-    switch -- $this(platform) {
-	macosx - macintosh {
-	    if {![catch {package require QuickTimeTcl}]} {
-		lappend audioSuffixes .aif .aiff .wav .mp3
-	    }
-	}
-	windows {
-	    if {![catch {package require QuickTimeTcl}]} {
-		lappend audioSuffixes .aif .aiff .wav .mp3
-	    }
-	    if {![catch {package require snack}]} {
-		lappend audioSuffixes .aif .aiff .wav .mp3
-	    }	    
-	}
-	default {
-	    if {![catch {package require snack}]} {
-		lappend audioSuffixes .aif .aiff .wav .mp3
-	    }	    
-	}
-    }
-    # This doesn't work due to the order of which things are loaded.
-    if {0} {
-	foreach subtype {aiff aiff wav mpeg} suff {.aif .aiff .wav .mp3} {
-	    if {[::Plugins::HaveImporterForMime audio/$subtype]} {
-		lappend audioSuffixes $suff
-	    }
-	}
-    }
-    
-    # Unique id needed to create instance tokens.
-    variable uid 0
     variable wdlg .budpounce
     
     # These define which the events are.
     variable events
-    set events(keys) {available unavailable msg                chat}
-    set events(str)  {Online    Offline     {Incoming Message} {New Chat}}
+    set events(keys) {available unavailable msg     chat}
+    set events(str)  {Online    Offline     Message Chat}
     
     variable actionlist
     set actionlist(keys) {msgbox sound chat msg}
@@ -91,36 +80,28 @@ proc ::BuddyPounce::Init { } {
     
     # And for 'any' which is not an array.
     variable budprefsany {}
-    
-    variable alertStr
-    array set alertStr {
-	available   {The user "%s" just went online!}
-	unavailable {The user "%s" just went offline!}
-	msg         {The user "%s" just sent you a message!}
-	chat        {The user "%s" just started a chat!}
-    }
-    
+        
     variable alertTitle 
     array set alertTitle {
-	available   {Online!}
-	unavailable {Offline!}
-	msg         {New message!}
-	chat        {New chat!}
+	available   Online
+	unavailable Offline
+	msg         Message
+	chat        Chat
     }
 }
 
-proc ::BuddyPounce::InitPrefsHook { } {
+proc ::BuddyPounce::InitPrefsHook {} {
     
     variable budprefsany
         
-    ::PreferencesUtils::Add [list  \
+    ::PrefUtils::Add [list  \
       [list ::BuddyPounce::budprefs      budprefs_array      [GetJidPrefsArr]] \
       [list ::BuddyPounce::budprefsgroup budprefsgroup_array [GetGroupPrefsArr]]\
       [list ::BuddyPounce::budprefsany   budprefsany         $budprefsany]   \
       ]    
 }
 
-proc ::BuddyPounce::GetJidPrefsArr { } {    
+proc ::BuddyPounce::GetJidPrefsArr {} {    
     variable budprefs
     return [array get budprefs]
 }
@@ -136,53 +117,56 @@ proc ::BuddyPounce::GetGroupPrefsArr { } {
 #       
 #       typeselected:   user, wb, group, ""
 
-proc ::BuddyPounce::Build {typeselected item group} {    
+proc ::BuddyPounce::Build {typeselected item groupL} {    
     global  this prefs
     
-    variable uid
     variable wdlg
     variable events
-    variable audioSuffixes
     
-    ::Debug 2 "::BuddyPounce::Build typeselected=$typeselected, item=$item"
+    ::Debug 2 "::BuddyPounce::Build typeselected=$typeselected, item=$item, groupL=$groupL"
 
     # Initialize the state variable, an array, that keeps is the storage.
-    
-    set token [namespace current]::[incr uid]
+    set uid [join [split [jlib::barejid $item] "@."] ""]
+    set token [namespace current]::$uid
     variable $token
     upvar 0 $token state
     
-    set w ${wdlg}${uid}
+    set w $wdlg$uid
     set state(w) $w
     
-    switch -- $typeselected {
-	user - wb {    
+    if {[lsearch $typeselected group] >= 0} {
+	set clicked group
+    } elseif {[lsearch $typeselected user] >= 0} {
+	set clicked user
+    } else {
+	set clicked $typeselected
+    }
+    
+    switch -- $clicked {
+	user {    
 	    set jid [jlib::jidmap $item]
 	    jlib::splitjid $jid jid2 res
 	    set state(jid)  $jid
 	    set state(jid2) $jid2
 	    set state(type) jid
-	    set msg "Set a specific action when something happens with \"$jid\",\
-	      which can be a changed status, or if you receive a message etc.\
-	      Pick events using the tabs below."
-	    set title "[mc {Buddy Pouncing}]: $jid"
+	    set msg [mc "Set a specific action for events related to %s. Select events using the tabs below." $jid]
+	    set title [mc "Contact Actions"]
+	    append title ": $jid"
 	}
 	group {
+	    set group [lindex $groupL 0]
 	    set state(group) $group
 	    set state(type)  group
-	    set msg "Set a specific action when something happens with\
-	      any contact belonging to the group \"$group\",\
-	      which can be a changed status, or if you receive a message etc.\
-	      Pick events using the tabs below."
-	    set title "[mc {Buddy Pouncing}]: $group"
+	    set msg [mc "Set a specific action for events related to any contact belonging to the group %s. Select events using the tabs below." $group]
+	    set title [mc "Contact Actions"]
+	    append title ": $group"
 	}
 	"" {
 	    set state(type) any
-	    set msg "Set a specific action when something happens with\
-	      any contact in your contact list,\
-	      which can be a changed status, or if you receive a message etc.\
-	      Pick events using the tabs below."
-	    set title "[mc {Buddy Pouncing}]: Any"
+	    set msg [mc "Set a specific action for events related to any contact in your contact list. Select events using the tabs below."]
+	    set title [mc "Contact Actions"]
+	    append title ": "
+	    append title [mc "Any"]
 	}
 	default {
 	    unset state
@@ -191,116 +175,139 @@ proc ::BuddyPounce::Build {typeselected item group} {
     }
     
     # Get all sounds.
-    set allSounds [GetAllSounds]
-    set fontS      [option get . fontSmall {}]
-    set contrastBg [option get . backgroundLightContrast {}]
-    set maxlen 0
-    set maxstr ""
-    foreach f $allSounds {
-	set len [string length $f]
-	if {$len > $maxlen} {
-	    set maxlen $len
-	    set maxstr $f
+    if {[component::exists Sounds]} {
+	set menuDef [list]
+	set allSounds [GetAllSounds]
+	foreach s $allSounds {
+	    lappend menuDef [list [::Sounds::GetTextForName $s] -value $s]
 	}
+	set soundfileDef [lindex $allSounds 0]
+    } else {
+	set menuDef [list [list [mc "None"]]]
+	set soundfileDef [mc "None"]
     }
     
+    
     # Toplevel with class BuddyPounce.
-    ::UI::Toplevel $w -class BuddyPounce -usemacmainmenu 1 -macstyle documentProc
-    wm title $w $title
-
-    # Global frame.
-    frame $w.frall -borderwidth 1 -relief raised
-    pack  $w.frall -fill both -expand 1 -ipadx 4
-
-    ::headlabel::headlabel $w.frall.head -text [mc {Buddy Pouncing}]
-    pack $w.frall.head -side top -fill both -expand 1
-    label $w.frall.msg -wraplength 300 -justify left -padx 10 -pady 2 \
-      -text $msg
-    pack $w.frall.msg -side top -anchor w
-    
-    frame $w.frall.fr -bg $contrastBg -bd 0
-    pack  $w.frall.fr -padx 6 -pady 4
-    
-    set wnb $w.frall.fr.nb
-    ::mactabnotebook::mactabnotebook $wnb
-    pack $wnb -padx 1 -pady 1
-        
-    # Fake menubutton to compute max width.
-    set wtmp $w.frall._tmp
-    menubutton $wtmp -text $maxstr
-    set soundMaxWidth [winfo reqwidth $wtmp]
-    destroy $wtmp
-    
-    set i 0
-    foreach eventStr $events(str) ekey $events(keys) {
-
-	set wpage [$wnb newpage $eventStr -text [mc $eventStr]]	
-		
-	# Action
-	set wact $wpage.f$ekey
-	frame $wact
-	pack  $wact -padx 6 -pady 2
-	checkbutton $wact.alrt -text " [mc {Show Popup}]" \
-	  -variable $token\($ekey,msgbox)
-	
-	checkbutton $wact.lsound -text " [mc {Play Sound}]:" \
-	  -variable $token\($ekey,sound)
-	set wmenu [eval {
-	    tk_optionMenu $wact.msound $token\($ekey,soundfile)
-	} $allSounds]
-	$wmenu configure -font $fontS
-
-	set wpad $wact.pad[incr i]
-	frame $wpad -width [expr $soundMaxWidth + 40] -height 1
-
-	checkbutton $wact.chat -text " [mc {Start Chat}]" \
-	  -variable $token\($ekey,chat)
-	set wmsg $wact.fmsg
-	frame $wmsg
-	
-	frame $wmsg.f1
-	pack  $wmsg.f1 -side top -anchor w
-	checkbutton $wmsg.f1.c -text " [mc {Send Message with subject}]:" \
-	  -variable $token\($ekey,msg)
-	entry $wmsg.f1.e -width 12 -textvariable $token\($ekey,msg,subject)
-	pack  $wmsg.f1.c $wmsg.f1.e -side left
-	
-	frame $wmsg.f2
-	pack  $wmsg.f2 -side top -anchor w -fill x
-	label $wmsg.f2.l -text "[mc Message]:"
-	text  $wmsg.f2.t -height 2 -width 24 -wrap word
-	pack  $wmsg.f2.l -side left -anchor n
-	pack  $wmsg.f2.t -side top -fill x
-	
-	set state($ekey,msg,wtext) $wmsg.f2.t
-	
-	grid x          x            $wpad
-	grid $wact.alrt $wact.lsound $wact.msound -sticky w  -padx 4 -pady 1
-	grid $wact.chat $wact.fmsg   -            -sticky nw -padx 4 -pady 1
-		
-	if {([llength audioSuffixes] == 0) || ![component::exists Sounds]} {
-	    $wact.lsound configure -state disabled
-	    $wact.msound configure -state disabled
-	}
+    if ([winfo exists $w]) {
+        raise $w
+        focus $w
+        return
     }
-    
-    # Button part.
-    set frbot [frame $w.frall.frbot -borderwidth 0]
-    pack [button $frbot.btok -text [mc OK] \
-      -default active -command [list [namespace current]::OK $token]] \
-      -side right -padx 5 -pady 5
-    pack [button $frbot.btcancel -text [mc Cancel]  \
-      -command [list [namespace current]::Cancel $token]]  \
-      -side right -padx 5 -pady 5
-    pack [button $frbot.btoff -text [mc {All Off}]  \
-      -command [list [namespace current]::AllOff $token]]  \
-      -side left -padx 5 -pady 5
-    pack $frbot -side top -fill both -expand 1 -padx 8 -pady 6
+    ::UI::Toplevel $w -class BuddyPounce \
+      -usemacmainmenu 1 -macstyle documentProc -command ::BuddyPounce::CloseHook
+    wm title $w $title
 
     set nwin [llength [::UI::GetPrefixedToplevels $wdlg]]
     if {$nwin == 1} {
 	::UI::SetWindowPosition $w $wdlg
     }
+
+    # Global frame.
+    ttk::frame $w.frall
+    pack $w.frall -fill both -expand 1
+
+    ttk::label $w.frall.head -style Headlabel \
+      -text [mc "Contact Actions"] -compound left
+    pack $w.frall.head -side top -fill both -expand 1
+
+    ttk::separator $w.frall.s -orient horizontal
+    pack $w.frall.s -side top -fill x
+
+    set wbox $w.frall.f
+    ttk::frame $wbox -padding [option get . dialogPadding {}]
+    pack $wbox -fill both -expand 1
+
+    ttk::label $wbox.msg -style Small.TLabel \
+      -padding {0 0 0 6} -wraplength 300 -justify left -text $msg
+    pack $wbox.msg -side top -anchor w
+        
+    set wnb $wbox.nb
+    ttk::notebook $wnb -padding {4}
+    pack $wnb
+        
+    set i 0
+    foreach estr $events(str) ekey $events(keys) {
+
+	$wnb add [ttk::frame $wnb.$ekey] -text [mc $estr] -sticky news
+		
+	# Action
+	set wact $wnb.$ekey.f
+	ttk::frame $wact -padding [option get . notebookPagePadding {}]
+	pack  $wact  -side top -anchor [option get . dialogAnchor {}]
+	
+	ttk::checkbutton $wact.alrt -text [mc "Show popup"] \
+	  -variable $token\($ekey,msgbox)
+	
+	ttk::checkbutton $wact.lsound -text [mc "Play sound"]: \
+	  -variable $token\($ekey,sound)
+	ui::combobutton $wact.msound -variable $token\($ekey,soundfile) \
+	  -menulist $menuDef
+	
+	ttk::checkbutton $wact.chat -text [mc "Start chat"] \
+	  -variable $token\($ekey,chat)
+
+	set wmsg $wact.fmsg
+	ttk::frame $wmsg
+	
+	ttk::frame $wmsg.f1
+	pack  $wmsg.f1 -side top -anchor w
+	ttk::checkbutton $wmsg.f1.c -text [mc "Send message with subject"]: \
+	  -variable $token\($ekey,msg)
+	ttk::entry $wmsg.f1.e -textvariable $token\($ekey,msg,subject)
+	pack  $wmsg.f1.c -side left
+	pack  $wmsg.f1.e -side top -fill x
+	
+	ttk::frame $wmsg.f2 -padding {0 2}
+	pack  $wmsg.f2 -side top -anchor w -fill x
+	ttk::label $wmsg.f2.l -text [mc "Message"]:
+	text  $wmsg.f2.t -height 2 -width 24 -wrap word -bd 1 -relief sunken
+	pack  $wmsg.f2.l -side left -anchor n
+	pack  $wmsg.f2.t -side top -fill x
+	
+	set state($ekey,msg,wtext) $wmsg.f2.t
+	
+	grid  $wact.alrt  $wact.lsound  $wact.msound  -sticky w  -padx 4 -pady 1
+	grid  $wact.chat  $wact.fmsg    -             -sticky nw -padx 4 -pady 1
+	
+	set maxw [$wact.msound maxwidth]
+	grid columnconfigure $wact 2 -minsize [expr {$maxw + 2*4}]
+	grid $wact.msound $wact.fmsg -sticky ew
+	
+	if {![component::exists Sounds]} {
+	    $wact.lsound state disabled
+	    $wact.msound state disabled
+	}
+	
+	# Set defaults.
+	set state($ekey,msgbox) 0
+	set state($ekey,sound)  0
+	set state($ekey,chat)   0
+	set state($ekey,msg)    0
+	set state($ekey,soundfile) $soundfileDef
+    }
+    
+    # Button part.
+    set frbot $wbox.b
+    ttk::frame $frbot -padding [option get . okcancelTopPadding {}]
+    ttk::button $frbot.btok -text [mc "OK"] -default active \
+      -command [list [namespace current]::OK $token]
+    ttk::button $frbot.btcancel -text [mc "Cancel"]  \
+      -command [list [namespace current]::Cancel $token]
+    ttk::button $frbot.btoff -text [mc "Disable All"]  \
+      -command [list [namespace current]::AllOff $token]
+    set padx [option get . buttonPadX {}]
+    if {[option get . okcancelButtonOrder {}] eq "cancelok"} {
+	pack $frbot.btok -side right
+	pack $frbot.btcancel -side right -padx $padx
+	pack $frbot.btoff -side right
+    } else {
+	pack $frbot.btoff -side right
+	pack $frbot.btcancel -side right -padx $padx
+	pack $frbot.btok -side right
+    }
+    pack $frbot -side bottom -fill x
+
     wm resizable $w 0 0
     
     AllOff $token
@@ -310,7 +317,7 @@ proc ::BuddyPounce::Build {typeselected item group} {
     set script [format {
 	update idletasks
 	%s configure -wraplength [expr [winfo reqwidth %s] - 12]
-    } $w.frall.msg $w]    
+    } $wbox.msg $w]    
     after idle $script
 
     return $token
@@ -328,7 +335,7 @@ proc ::BuddyPounce::PrefsToState {token} {
     variable budprefsgroup
     variable budprefsany
     
-    set eventActions {}
+    set eventActions [list]
     
     switch -- $state(type) {
 	jid {
@@ -353,8 +360,10 @@ proc ::BuddyPounce::PrefsToState {token} {
 	    switch -glob -- $akey {
 		soundfile:* {
 		    # The sound file is treated specially.
-		    set state($ekey,soundfile)  \
-		      [string map {soundfile: ""} $akey]
+		    if {[component::exists Sounds]} {
+			set state($ekey,soundfile)  \
+			  [string map {soundfile: ""} $akey]
+		    }    
 		}
 		subject:* {
 		    set state($ekey,msg,subject) \
@@ -365,6 +374,13 @@ proc ::BuddyPounce::PrefsToState {token} {
 		    set body [subst -nocommands -novariables $body]
 		    $state($ekey,msg,wtext) insert end $body
 		}
+		sound {
+		    if {[component::exists Sounds]} {
+			set state($ekey,$akey) 1
+		    } else {
+			set state($ekey,$akey) 0
+		    }
+		}
 		default {
 		    set state($ekey,$akey) 1
 		}
@@ -386,11 +402,11 @@ proc ::BuddyPounce::StateToPrefs {token} {
     variable events
     variable actionlist
 
-    set eventActions {}
+    set eventActions [list]
     
     # Build event-action list from state.
     foreach ekey $events(keys) {
-	set actlist {}
+	set actlist [list]
 	foreach akey $actionlist(keys) {
 	    if {$state($ekey,$akey) == 1} {
 		lappend actlist $akey
@@ -398,15 +414,17 @@ proc ::BuddyPounce::StateToPrefs {token} {
 		switch -- $akey {
 		    sound {
 			# If sound we also need the soundfile.
-			lappend actlist soundfile:$state($ekey,soundfile)
+			if {[component::exists Sounds]} {
+			    lappend actlist soundfile:$state($ekey,soundfile)
+			}
 		    }
 		    msg {
-			if {$state($ekey,msg,subject) != ""} {
+			if {$state($ekey,msg,subject) ne ""} {
 			    lappend actlist subject:$state($ekey,msg,subject)
 			}
 			set body [$state($ekey,msg,wtext) get 1.0 "end -1 char"]
 			regsub -all "\n" $body {\\n} body
-			if {$body != ""} {
+			if {$body ne ""} {
 			    lappend actlist body:$body
 			}
 		    }
@@ -503,15 +521,15 @@ proc ::BuddyPounce::Event {from eventkey args} {
     variable budprefs
     variable budprefsgroup
     variable budprefsany
-    variable alertStr
     variable alertTitle
     
     ::Debug 4 "::BuddyPounce::Event from = $from, eventkey=$eventkey"
-    array set argsArr $args
+    array set argsA $args
+    set xmldata $argsA(-xmldata)
     
     # We must check 'jid', 'group' and 'any' in that order.
     # A list of actions to perform if any.
-    set actions {}
+    set actions [list]
     
     # First this specific JID.
     set jid [jlib::jidmap $from]
@@ -545,44 +563,51 @@ proc ::BuddyPounce::Event {from eventkey args} {
 	}
     }
     
+    set budpounce [dict create]
+    dict set budpounce available [mc "%s just went online!" $from]
+    dict set budpounce unavailable [mc "%s just went offline!" $from]
+    dict set budpounce msg [mc "%s just sent you a message!" $from]
+    dict set budpounce chat [mc "%s just started a chat!" $from]
+
     foreach action $actions {
 	
 	switch -- $action {
 	    msgbox {
-		::UI::AlertBox [format $alertStr($eventkey) $from] \
-		  -title $alertTitle($eventkey)
+		ui::dialog -message [dict get $budpounce $eventkey] \
+		  -title [mc $alertTitle($eventkey)]
 	    }
 	    sound {
 		set soundfile [lsearch -inline -glob $actions soundfile:*]
-		set tail [string map {soundfile: ""} $soundfile]
-		if {$tail != ""} {
-		    PlaySound $tail
+		set name [string map {soundfile: ""} $soundfile]
+		if {$name ne ""} {
+		    PlaySound $name
 		}
 	    }
 	    chat {			
 		# If already have chat.
-		set w [::Chat::HaveChat $jid]
-		if {$w != ""} {
+		set w [::Chat::GetWindow $jid]
+		if {$w ne ""} {
 		    raise $w
 		} else {
 		    ::Chat::StartThread $from
 		}
 	    }
 	    msg {
-		set subject [mc {Auto Reply}]
+		set subject [mc "Auto Reply"]
 		set body "Insert your message!"
 		set subjectopt [lsearch -inline -glob $actions subject:*]
-		if {$subjectopt != ""} {
+		if {$subjectopt ne ""} {
 		    set subject [string map {subject: ""} $subjectopt]
 		}
 		set bodyopt [lsearch -inline -glob $actions body:*]
-		if {$bodyopt != ""} {
+		if {$bodyopt ne ""} {
 		    set body [string map {body: ""} $bodyopt]
 		    set body [subst -nocommands -novariables $body]
 		}
-		set opts {}
-		if {[info exists argsArr(-body)]} {
-		    lappend opts -quotemessage $argsArr(-body)
+		set opts [list]
+		set msgbody [wrapper::getcdata [wrapper::getfirstchildwithtag $xmldata body]]
+		if {$msgbody ne ""} {
+		    lappend opts -quotemessage $msgbody
 		}
 		eval {::NewMsg::Build -to $from  \
 		  -subject $subject -message $body} $opts
@@ -592,33 +617,24 @@ proc ::BuddyPounce::Event {from eventkey args} {
 }
 
 proc ::BuddyPounce::GetAllSounds {} {
-    global  this
-    variable audioSuffixes
     
-    set all {}
-    foreach f [glob -nocomplain -directory $this(soundsPath) *] {
-	if {[lsearch $audioSuffixes [file extension $f]] >= 0} {
-	    lappend all [file tail $f]
-	}
+    set all [list]
+    if {[component::exists Sounds]} {
+	set all [::Sounds::GetAllSoundsPresentSet]
     }
     if {[llength $all] == 0} {
-	set all [msgcat::mc None]
+	set all [msgcat::mc "None"]
     }
     return $all
 }
 
-proc ::BuddyPounce::PlaySound {tail} {
-    global  this
-    
+proc ::BuddyPounce::PlaySound {name} {    
     if {[component::exists Sounds]} {
-	
-	# We have no good way of avoiding playing sounds simultaneously!
-	after 1200 [list ::Sounds::PlaySoundTmp \
-	  [file join $this(soundsPath) $tail]]
+	::Sounds::DoPlayWhenIdle $name
     }
 }
 
-proc ::BuddyPounce::QuitHook { } {
+proc ::BuddyPounce::QuitHook {} {
     variable wdlg
     
     ::UI::SaveWinPrefixGeom $wdlg
@@ -626,29 +642,23 @@ proc ::BuddyPounce::QuitHook { } {
 
 proc ::BuddyPounce::CloseHook {wclose} {
     variable wdlg
-    
-    if {[string match $wdlg* $wclose]} {
-	::UI::SaveWinGeom $wdlg $wclose
-    }   
+
+    ::UI::SaveWinGeom $wdlg $wclose
     return ""
 }
 
-proc ::BuddyPounce::NewChatMsgHook {body args} {
+proc ::BuddyPounce::NewChatMsgHook {xmldata} {
 
-    array set argsArr $args
-    if {[info exists argsArr(-from)]} {
-	jlib::splitjid $argsArr(-from) jid2 res
-	eval {Event $jid2 chat} $args
-    }
+    set from [wrapper::getattribute $xmldata from]
+    set jid2 [jlib::barejid $from]
+    Event $jid2 chat -xmldata $xmldata
 }
 
-proc ::BuddyPounce::NewMsgHook {body args} {
+proc ::BuddyPounce::NewMsgHook {xmldata uuid} {
     
-    array set argsArr $args
-    if {[info exists argsArr(-from)]} {
-	jlib::splitjid $argsArr(-from) jid2 res
-	eval {Event $jid2 msg} $args
-    }
+    set from [wrapper::getattribute $xmldata from]
+    set jid2 [jlib::barejid $from]
+    Event $jid2 msg -xmldata $xmldata
 }
 
 proc ::BuddyPounce::PresenceHook {jid type args} {
@@ -660,19 +670,18 @@ proc ::BuddyPounce::PresenceHook {jid type args} {
     switch -- $type {
 	available {
 	    if {![::Jabber::RosterCmd wasavailable $jid]} {
-		Event $jid $type
+		eval {Event $jid $type} $args
 	    }
 	}
 	unavailable {
 	    if {[::Jabber::RosterCmd wasavailable $jid]} {
-		Event $jid $type
+		eval {Event $jid $type} $args
 	    }
 	}
     }
 }
 
-proc ::BuddyPounce::PresenceUnavailableHook  {jid type args} {
-    
+proc ::BuddyPounce::PresenceUnavailableHook  {jid type args} {   
     Event $jid unavailable
 }
 
