@@ -2,11 +2,11 @@
 #  
 #      This file provides translation from canvas commands to XML/SVG format.
 #      
-#  Copyright (c) 2002-2004  Mats Bengtsson
+#  Copyright (c) 2002-2007  Mats Bengtsson
 #  
-#  This particular package is BSD licensed. 
+#  This file is distributed under BSD style license.
 #
-# $Id: can2svg.tcl,v 1.18 2004/12/02 15:22:07 matben Exp $
+# $Id: can2svg.tcl,v 1.26 2008-01-27 08:19:36 matben Exp $
 # 
 # ########################### USAGE ############################################
 #
@@ -22,13 +22,14 @@
 #                      -width
 #      
 #      can2svg::can2svg canvasCmd ?options?
-#           options:    -httpbasedir  path
-#                       -ovalasellipse 0|1
-#                       -reusedefs     0|1
-#                       -uritype    file|http
-#                       -usestyleattribute 0|1
-#                       -usetags    0|all|first|last
-#                       -windowitemhandler
+#           options:    -httpbasedir        path
+#                       -imagehandler       tclProc
+#                       -ovalasellipse      0|1
+#                       -reusedefs          0|1
+#                       -uritype            file|http
+#                       -usestyleattribute  0|1
+#                       -usetags            0|all|first|last
+#                       -windowitemhandler  tclProc
 #                       
 #      can2svg::config ?options?
 #           options:	-allownewlines      0
@@ -39,7 +40,7 @@
 #	                -uritype            file
 #	                -usetags            all
 #	                -usestyleattribute  1
-#                       -windowitemhandler
+#                       -windowitemhandler  tclProc
 #
 # ########################### CHANGES ##########################################
 #
@@ -102,13 +103,15 @@ namespace eval can2svg {
     variable stippleDataArr
     
     set stippleDataArr(gray75)  \
-      {M 0 0 h3 M 0 1 h1 m 1 0 h2 M 0 2 h2 m 1 0 h1 M 0 3 h3}
+      {M 0 0 h3  M 0 1 h1 M 2 1 h2
+       M 0 2 h3  M 0 3 h1 M 2 3 h1}
     set stippleDataArr(gray50)  \
-      {M 0 0 h1 m 1 0 h1 M 1 1 h1 m 1 0 h1 \
-      M 0 2 h1 m 1 0 h1 M 1 3 h1 m 1 0 h1}
+      {M 0 0 h1 M 2 0 h1  M 1 1 h1 M 3 1 h1
+       M 0 2 h1 M 2 2 h1  M 1 3 h1 M 3 3 h1}
     set stippleDataArr(gray25)  \
-      {M 0 0 h1 M 2 1 h1 M 1 2 h1 M 3 3 h1}
-    set stippleDataArr(gray12) {M 0 0 h1 M 2 2 h1}
+      {M 3 0 h1 M 1 1 h1 M 3 2 h1 M 1 3 h1}
+    set stippleDataArr(gray12)  \
+      {M 1 1 h1 M 3 3 h1}
     
 }
 
@@ -150,12 +153,13 @@ proc can2svg::config {args} {
 #       
 # Arguments:
 #       cmd         canvas create commands without prepending widget path.
-#       args    -httpbasedir  path
-#               -ovalasellipse 0|1
-#               -reusedefs     0|1
-#               -uritype    file|http
-#               -usestyleattribute 0|1
-#               -usetags    0|all|first|last
+#       args    -httpbasedir        path
+#               -imagehandler       tclProc
+#               -ovalasellipse      0|1
+#               -reusedefs          0|1
+#               -uritype            file|http
+#               -usestyleattribute  0|1
+#               -usetags            0|all|first|last
 #       
 # Results:
 #   xml data
@@ -175,10 +179,13 @@ proc can2svg::can2svg {cmd args} {
 #       
 # Arguments:
 #       cmd         canvas create command without prepending widget path.
-#       args    -httpbasedir  path
-#               -uritype    file|http
-#               -usestyleattribute 0|1
-#               -usetags    0|all|first|last
+#       args    -httpbasedir        path
+#               -imagehandler       tclProc
+#               -ovalasellipse      0|1
+#               -reusedefs          0|1
+#               -uritype            file|http
+#               -usestyleattribute  0|1
+#               -usetags            0|all|first|last
 #       
 # Results:
 #       a list of xmllist = {tag attrlist isempty cdata {child1 child2 ...}}
@@ -190,17 +197,17 @@ proc can2svg::svgasxmllist {cmd args} {
     variable defsStipplePatternArr
     variable defaultFont
     variable grayStipples
-    
+        
     set nonum_ {[^0-9]}
     set wsp_ {[ ]+}
-    set xmlListList {}
+    set xmlLL [list]
     
-    array set argsArr [array get confopts]
-    array set argsArr $args
-    set args [array get argsArr]
+    array set argsA [array get confopts]
+    array set argsA $args
+    set args [array get argsA]
     
     if {![string equal [lindex $cmd 0] "create"]} {
-	return {}
+	return
     }
     
     set type [lindex $cmd 1]
@@ -210,7 +217,7 @@ proc can2svg::svgasxmllist {cmd args} {
     set indopt [lsearch -regexp $rest "-${nonum_}"]
     if {$indopt < 0} {
 	set ind end
-	set opts {}
+	set opts [list]
     } else {
 	set ind [expr $indopt - 1]
 	set opts [lrange $rest $indopt end]
@@ -221,40 +228,45 @@ proc can2svg::svgasxmllist {cmd args} {
     if {[llength $coo] == 1} {
 	set coo [lindex $coo 0]
     }
-    array set optArr $opts
+    array set optA $opts
+    
+    # Is the item in normal state? If not, return.
+    if {[info exists optA(-state)] && $optA(-state) != "normal"} {
+	return
+    }
     
     # Figure out if we've got a spline.
     set haveSpline 0
-    if {[info exists optArr(-smooth)] && ($optArr(-smooth) != "0") &&  \
-      [info exists optArr(-splinesteps)] && ($optArr(-splinesteps) > 2)} {
+    if {[info exists optA(-smooth)] && ($optA(-smooth) != "0") &&  \
+      [info exists optA(-splinesteps)] && ($optA(-splinesteps) > 2)} {
 	set haveSpline 1
     }
-    if {[info exists optArr(-fill)]} {
-	set fillValue $optArr(-fill)
+    if {[info exists optA(-fill)]} {
+	set fillValue $optA(-fill)
 	if {![regexp {#[0-9]+} $fillValue]} {
 	    set fillValue [FormatColorName $fillValue]
 	}
     } else {
 	set fillValue black
     }
-    if {[string length $argsArr(-filtertags)] && [info exists optArr(-tags)]} {
-	set tag [uplevel #0 $argsArr(-filtertags) [list $optArr(-tags)]]
+    if {[string length $argsA(-filtertags)] && [info exists optA(-tags)]} {
+	set tag [uplevel #0 $argsA(-filtertags) [list $optA(-tags)]]
 	set idAttr [list id $tag]
-    } elseif {($argsArr(-usetags) != "0") && [info exists optArr(-tags)]} {
+    } elseif {($argsA(-usetags) != "0") && [info exists optA(-tags)]} {
 	
 	# Remove any 'current' tag.
-	set optArr(-tags) \
-	  [lsearch -all -not -inline $optArr(-tags) current]
+	set optA(-tags) \
+	  [lsearch -all -not -inline $optA(-tags) current]
 	
-	switch -- $argsArr(-usetags) {
+	switch -- $argsA(-usetags) {
 	    all {			
-		set idAttr [list id $optArr(-tags)]
+		set idAttr [list id $optA(-tags)]
 	    }
 	    first {
-		set idAttr [list id [lindex $optArr(-tags) 0]]
+		set idAttr [list id [lindex $optA(-tags) 0]]
 	    }
 	    last {
-		set idAttr [list id [lindex $optArr(-tags) end]]
+		set idAttr [list id [lindex $optA(-tags) end]]
 	    }
 	}
     } else {
@@ -262,39 +274,40 @@ proc can2svg::svgasxmllist {cmd args} {
     }
     
     # If we need a marker (arrow head) need to make that first.
-    if {[info exists optArr(-arrow)] && ![string equal $optArr(-arrow) "none"]} {
-	if {[info exists optArr(-arrowshape)]} {
+    if {[info exists optA(-arrow)] && ![string equal $optA(-arrow) "none"]} {
+	if {[info exists optA(-arrowshape)]} {
 	    
 	    # Make a key of the arrowshape list into the array.
-	    regsub -all -- $wsp_ $optArr(-arrowshape) _ shapeKey
+	    regsub -all -- $wsp_ $optA(-arrowshape) _ shapeKey
 	    set arrowKey ${fillValue}_${shapeKey}
-	    set arrowShape $optArr(-arrowshape)
+	    set arrowShape $optA(-arrowshape)
 	} else {
 	    set arrowKey ${fillValue}
 	    set arrowShape {8 10 3}
 	}
-	if {!$argsArr(-reusedefs) || \
+	if {!$argsA(-reusedefs) || \
 	  ![info exists defsArrowMarkerArr($arrowKey)]} {
 	    set defsArrowMarkerArr($arrowKey)  \
 	      [eval {MakeArrowMarker} $arrowShape {$fillValue}]
-	    set xmlListList \
-	      [concat $xmlListList $defsArrowMarkerArr($arrowKey)]
+	    set xmlLL \
+	      [concat $xmlLL $defsArrowMarkerArr($arrowKey)]
 	}
     }
     
     # If we need a stipple bitmap, need to make that first. Limited!!!
     # Only: gray12, gray25, gray50, gray75
     foreach key {-stipple -outlinestipple} {
-	if {[info exists optArr($key)] &&  \
-	  ([lsearch $grayStipples $optArr($key)] >= 0)} {
-	    set stipple $optArr($key)
+	if {[info exists optA($key)] &&  \
+	  ([lsearch $grayStipples $optA($key)] >= 0)} {
+	    set stipple $optA($key)
 	    if {![info exists defsStipplePatternArr($stipple)]} {
 		set defsStipplePatternArr($stipple)  \
 		  [MakeGrayStippleDef $stipple]
 	    }
-	    lappend xmlListList $defsStipplePatternArr($stipple)
+	    lappend xmlLL $defsStipplePatternArr($stipple)
 	}
     }
+    puts "can2svg::svgasxmllist cmd=$cmd, args=$args"
     
     switch -- $type {
 	
@@ -307,16 +320,25 @@ proc can2svg::svgasxmllist {cmd args} {
 		set attr [concat $attr $idAttr]
 	    }
 	    set attr [concat $attr [MakeAttrList \
-	      $type $opts $argsArr(-usestyleattribute)]]
-	    lappend xmlListList [MakeXMLList $elem -attrlist $attr]
+	      $type $opts $argsA(-usestyleattribute)]]
+	    lappend xmlLL [MakeXMLList $elem -attrlist $attr]
 	}
 	bitmap - image {
-	    set elem "image"
-	    set attr [eval {MakeImageAttr $coo $opts} $args]
-	    if {[string length $idAttr] > 0} {
-		set attr [concat $attr $idAttr]
+            if {[info exists optA(-image)]} {
+	        set elem "image"
+	        set attr [eval {MakeImageAttr $coo $opts} $args]
+	        if {[string length $idAttr] > 0} {
+	            set attr [concat $attr $idAttr]
+	        }
+		set subEs [list]
+		if {[info exists argsA(-imagehandler)]} {
+		    set subE [uplevel #0 $argsA(-imagehandler) [list $cmd] $args]
+		    if {[llength $subE]} {
+			set subEs [list $subE]
+		    }
+		}
+		lappend xmlLL [MakeXMLList $elem -attrlist $attr -subtags $subEs]
 	    }
-	    lappend xmlListList [MakeXMLList $elem -attrlist $attr]
 	}
 	line {
 	    set attr [CoordsToAttr $type $coo $opts elem]	    
@@ -324,14 +346,15 @@ proc can2svg::svgasxmllist {cmd args} {
 		set attr [concat $attr $idAttr]
 	    }		    
 	    set attr [concat $attr [MakeAttrList \
-	      $type $opts $argsArr(-usestyleattribute)]]
-	    lappend xmlListList [MakeXMLList $elem -attrlist $attr]
+	      $type $opts $argsA(-usestyleattribute)]]
+	    lappend xmlLL [MakeXMLList $elem -attrlist $attr]
 	}
 	oval {
 	    set attr [CoordsToAttr $type $coo $opts elem]	    
 	    foreach {x y w h} [NormalizeRectCoords $coo] break
-	    if {[expr $w == $h] && !$argsArr(-ovalasellipse)} {
-		set elem "circle"
+	    if {[expr $w == $h] && !$argsA(-ovalasellipse)} {
+		# set elem "circle";# circle needs an r: not an rx & ry
+		set elem "ellipse"
 	    } else {
 		set elem "ellipse"
 	    }
@@ -339,8 +362,8 @@ proc can2svg::svgasxmllist {cmd args} {
 		set attr [concat $attr $idAttr]
 	    }
 	    set attr [concat $attr [MakeAttrList \
-	      $type $opts $argsArr(-usestyleattribute)]]
-	    lappend xmlListList [MakeXMLList $elem -attrlist $attr]
+	      $type $opts $argsA(-usestyleattribute)]]
+	    lappend xmlLL [MakeXMLList $elem -attrlist $attr]
 	}
 	polygon {
 	    set attr [CoordsToAttr $type $coo $opts elem]	    
@@ -348,8 +371,8 @@ proc can2svg::svgasxmllist {cmd args} {
 		set attr [concat $attr $idAttr]
 	    }
 	    set attr [concat $attr [MakeAttrList \
-	      $type $opts $argsArr(-usestyleattribute)]]
-	    lappend xmlListList [MakeXMLList $elem -attrlist $attr]
+	      $type $opts $argsA(-usestyleattribute)]]
+	    lappend xmlLL [MakeXMLList $elem -attrlist $attr]
 	}
 	rectangle {
 	    set attr [CoordsToAttr $type $coo $opts elem]	    
@@ -357,47 +380,53 @@ proc can2svg::svgasxmllist {cmd args} {
 		set attr [concat $attr $idAttr]
 	    }
 	    set attr [concat $attr [MakeAttrList \
-	      $type $opts $argsArr(-usestyleattribute)]]
-	    lappend xmlListList [MakeXMLList $elem -attrlist $attr]
+	      $type $opts $argsA(-usestyleattribute)]]
+	    lappend xmlLL [MakeXMLList $elem -attrlist $attr]
 	}
 	text {
 	    set elem "text"
+	    set chdata ""
 	    set nlines 1
-	    if {[info exists optArr(-font)]} {
-		set theFont $optArr(-font)
+	    if {[info exists optA(-font)]} {
+		set theFont $optA(-font)
 	    } else {
 		set theFont $defaultFont
 	    }
 	    set ascent [font metrics $theFont -ascent]
 	    set lineSpace [font metrics $theFont -linespace]
-	    if {[info exists optArr(-text)]} {
-		set chdata $optArr(-text)
+	    if {[info exists optA(-text)]} {
+		set chdata $optA(-text)
 		
-		# MICK O'DONNELL: if the text is wrapped in the wgt, we need
-		# to simulate linebreaks
-		# 
-		# If the item has got -width != 0 then we must wrap it ourselves
-		# using newlines since the -text does not have extra newlines
-		# at these linebreaks.
-		set lines [split $chdata \n]
-		set newlines {}
-		foreach line $lines {
-		    set lines2 [SplitWrappedLines $line $theFont $optArr(-width)]
-		    set newlines [concat $newlines $lines2]
+		if {[info exists optA(-width)]} {
+		    
+		    # MICK O'DONNELL: if the text is wrapped in the wgt, we need
+		    # to simulate linebreaks
+		    # 
+		    # If the item has got -width != 0 then we must wrap it ourselves
+		    # using newlines since the -text does not have extra newlines
+		    # at these linebreaks.
+		    set lines [split $chdata \n]
+		    set newlines {}
+		    foreach line $lines {
+			set lines2 [SplitWrappedLines $line $theFont $optA(-width)]
+			set newlines [concat $newlines $lines2]
+		    }
+		    set chdata [join $newlines \n]
+		    if {!$argsA(-allownewlines) || \
+		      ([llength $newlines] > [llength $lines])} {
+			set nlines [expr [regexp -all "\n" $chdata] + 1]
+		    }
+		} else {
+		    if {!$argsA(-allownewlines)} {
+			set nlines [expr [regexp -all "\n" $chdata] + 1]
+		    }
 		}
-		set chdata [join $newlines \n]
-		if {!$argsArr(-allownewlines) || \
-		  ([llength $newlines] > [llength $lines])} {
-		    set nlines [expr [regexp -all "\n" $chdata] + 1]
-		}
-	    } else {
-		set chdata ""
 	    }
 	    
 	    # Figure out the coords of the first baseline.
 	    set anchor center
-	    if {[info exists optArr(-anchor)]} {
-		set anchor $optArr(-anchor)
+	    if {[info exists optA(-anchor)]} {
+		set anchor $optA(-anchor)
 	    }		    		    
 	    
 	    foreach {xbase ybase}  \
@@ -408,7 +437,7 @@ proc can2svg::svgasxmllist {cmd args} {
 		set attr [concat $attr $idAttr]
 	    }
 	    set attr [concat $attr [MakeAttrList \
-	      $type $opts $argsArr(-usestyleattribute)]]
+	      $type $opts $argsA(-usestyleattribute)]]
 	    set dy 0
 	    if {$nlines > 1} {
 		
@@ -419,10 +448,10 @@ proc can2svg::svgasxmllist {cmd args} {
 		      -attrlist [list "x" $xbase "dy" $dy] -chdata $line]
 		    set dy $lineSpace
 		}
-		lappend xmlListList [MakeXMLList $elem -attrlist $attr \
+		lappend xmlLL [MakeXMLList $elem -attrlist $attr \
 		  -subtags $subList]
 	    } else {
-		lappend xmlListList [MakeXMLList $elem -attrlist $attr \
+		lappend xmlLL [MakeXMLList $elem -attrlist $attr \
 		  -chdata $chdata]
 	    }
 	}
@@ -430,21 +459,21 @@ proc can2svg::svgasxmllist {cmd args} {
 	    
 	    # There is no svg for this; must be handled by application layer.	    
 	    #puts "window: $cmd"
-	    if {[string length $argsArr(-windowitemhandler)]} {
+	    if {[string length $argsA(-windowitemhandler)]} {
 		set xmllist \
-		  [uplevel #0 $argsArr(-windowitemhandler) [list $cmd] $args]
+		  [uplevel #0 $argsA(-windowitemhandler) [list $cmd] $args]
 		if {[llength $xmllist]} {
-		    lappend xmlListList $xmllist
+		    lappend xmlLL $xmllist
 		}
 	    }
 	}
     }
-    return $xmlListList
+    return $xmlLL
 }
 
 # can2svg::CoordsToAttr --
 #
-#       Makes a list of attrbutes corresponding to type and coords.
+#       Makes a list of attributes corresponding to type and coords.
 #       
 # Arguments:
 #
@@ -455,12 +484,12 @@ proc can2svg::svgasxmllist {cmd args} {
 proc can2svg::CoordsToAttr {type coo opts svgElementVar} {
     upvar $svgElementVar elem 
 
-    array set optArr $opts
+    array set optA $opts
     
     # Figure out if we've got a spline.
     set haveSpline 0
-    if {[info exists optArr(-smooth)] && ($optArr(-smooth) != "0") &&  \
-      [info exists optArr(-splinesteps)] && ($optArr(-splinesteps) > 2)} {
+    if {[info exists optA(-smooth)] && ($optA(-smooth) != "0") &&  \
+      [info exists optA(-splinesteps)] && ($optA(-splinesteps) > 2)} {
 	set haveSpline 1
     }
     set attr {}
@@ -472,8 +501,11 @@ proc can2svg::CoordsToAttr {type coo opts svgElementVar} {
 	    set attr [list "d" $data]
 	}
 	bitmap - image {
-	    set elem "image"
-	    set attr [ImageCoordsToAttr $coo $opts]
+	    array set __optA $opts
+            if {[info exists __optA(-image)]} {
+	        set elem "image"
+	        set attr [ImageCoordsToAttr $coo $opts]
+	    }
 	}
 	line {
 	    if {$haveSpline} {
@@ -529,12 +561,12 @@ proc can2svg::MakeArcPath {coo opts} {
     variable pi
 
     # Canvas defaults.
-    array set optArr {
+    array set optA {
 	-extent 90
 	-start  0
 	-style  pieslice
     }
-    array set optArr $opts
+    array set optA $opts
 
     # Extract center and radius from bounding box.
     foreach {x1 y1 x2 y2} $coo break
@@ -543,8 +575,8 @@ proc can2svg::MakeArcPath {coo opts} {
     set rx [expr abs($x1 - $x2)/2.0]
     set ry [expr abs($y1 - $y2)/2.0]
 
-    set start  [expr $anglesToRadians * $optArr(-start)]
-    set extent [expr $anglesToRadians * $optArr(-extent)]
+    set start  [expr $anglesToRadians * $optA(-start)]
+    set extent [expr $anglesToRadians * $optA(-extent)]
 
     # NOTE: direction of angles are opposite for Tk and SVG!    
     set theta1 [expr -1*$start]
@@ -569,7 +601,7 @@ proc can2svg::MakeArcPath {coo opts} {
     append data [format " %.1f %.1f %.1f %1d %1d %.1f %.1f"  \
       $rx $ry $phi $fa $fs $x2 $y2]
     
-    switch -- $optArr(-style) {
+    switch -- $optA(-style) {
 	arc {
 	    # empty.
 	}
@@ -591,7 +623,7 @@ proc can2svg::MakeArcPathNonA {coo opts} {
 
     variable anglesToRadians
 
-    array set optArr $opts
+    array set optA $opts
     
     foreach {x1 y1 x2 y2} $coo break
     set cx [expr ($x1 + $x2)/2.0]
@@ -602,8 +634,8 @@ proc can2svg::MakeArcPathNonA {coo opts} {
     
     # This approximation gives a maximum half pixel error.
     set deltaPhi [expr 2.0/sqrt($rmin)]
-    set extent   [expr $anglesToRadians * $optArr(-extent)]
-    set start    [expr $anglesToRadians * $optArr(-start)]
+    set extent   [expr $anglesToRadians * $optA(-extent)]
+    set start    [expr $anglesToRadians * $optA(-start)]
     set nsteps   [expr int(abs($extent)/$deltaPhi) + 2]
     set delta    [expr $extent/$nsteps]
     set data [format "M %.1f %.1f L"  \
@@ -613,9 +645,9 @@ proc can2svg::MakeArcPathNonA {coo opts} {
 	append data [format " %.1f %.1f"  \
 	  [expr $cx + $rx*cos($phi)] [expr $cy - $ry*sin($phi)]]
     }
-    if {[info exists optArr(-style)]} {
+    if {[info exists optA(-style)]} {
 	
-	switch -- $optArr(-style) {
+	switch -- $optA(-style) {
 	    chord {
 		append data " Z"
 	    }
@@ -667,13 +699,13 @@ proc can2svg::MakeStyleAttr {type opts} {
 
 proc can2svg::MakeStyleList {type opts args} {
     
-    array set argsArr {
+    array set argsA {
 	-setdefaults 1 
     }
-    array set argsArr $args
+    array set argsA $args
     
     # Defaults for everything except text.
-    if {$argsArr(-setdefaults) && ![string equal $type "text"]} {
+    if {$argsA(-setdefaults) && ![string equal $type "text"]} {
 	array set styleArr {fill none stroke black}
     }
     set fillCol black
@@ -772,19 +804,19 @@ proc can2svg::MakeStyleList {type opts args} {
     if {[info exists stippleValue]} {
 	
 	# Overwrite any existing.
-	set styleArr(fill) "url(#tile$stippleValue)"
+	set styleArr(fill) "url(#tile[string trimleft $stippleValue @])"
     }
     if {[info exists outlineStippleValue]} {
 	
 	# Overwrite any existing.
-	set styleArr(stroke) "url(#tile$stippleValue)"
+	set styleArr(stroke) "url(#tile[string trimleft $stippleValue @])"
     }
     
     # Transform dash value.
     if {[info exists dashValue]} {
 		
 	# Two different syntax here.		
-	if {[regexp {[\.,\-_ ]} $dashValue]} {
+	if {[regexp {[\.,\-_]} $dashValue]} {
 	    
 	    # .=2 ,=4 -=6 space=4    times stroke width.
 	    # A space enlarges the... space.
@@ -809,13 +841,15 @@ proc can2svg::MakeStyleList {type opts args} {
 	    }
 	    set styleArr(stroke-dasharray) [string trim $dash]
 	} else {
-	    set styleArr(stroke-dasharray) $value
+	    set dashValue [string trim $dashValue]
+	    if {$dashValue ne ""} {
+		set styleArr(stroke-dasharray) $dashValue
+	    }
 	}
     }
     if {[string equal $type "polygon"]} {
 	set styleArr(fill-rule) "evenodd"
-    }
-        
+    }        
     return [array get styleArr]
 }
 
@@ -973,20 +1007,20 @@ proc can2svg::BreakChar {char} {
 proc can2svg::MakeImageAttr {coo opts args} {
     variable confopts
     
-    array set optArr {-anchor nw}
-    array set optArr $opts
-    array set argsArr $args
+    array set optA {-anchor nw}
+    array set optA $opts
+    array set argsA $args
     
     set attrList [ImageCoordsToAttr $coo $opts]
 
     # We should make this an URI.
-    set theImage $optArr(-image)
-    set theFile [$theImage cget -file]
-    if {$theFile != ""} {
-	if {[string equal $argsArr(-uritype) "file"]} {
-	    set uri [FileUriFromLocalFile $theFile]
-	} elseif {[string equal $argsArr(-uritype) "http"]} {
-	    set uri [HttpFromLocalFile $theFile]
+    set image $optA(-image)
+    set fileName [$image cget -file]
+    if {$fileName ne ""} {
+	if {[string equal $argsA(-uritype) "file"]} {
+	    set uri [FileUriFromLocalFile $fileName]
+	} elseif {[string equal $argsA(-uritype) "http"]} {
+	    set uri [HttpFromLocalFile $fileName]
 	}
 	lappend attrList "xlink:href" $uri
     } else {
@@ -995,12 +1029,55 @@ proc can2svg::MakeImageAttr {coo opts args} {
     return $attrList
 }
 
-proc can2svg::ImageCoordsToAttr {coo opts} {
+# Function  : can2svg::ImageCoordsToAttr 
+# ------------------------------ ------------------------------ --------- 
+# Returns   : list of x y width and height including description 
+# Parameters : coo - coordinates of the image 
+#        opts - argument list -anchor nw ... 
+#
+# Description : 
+# fixme (Roger) 01/25/2008 :Why not using the bounding box? 
+#
+# Written   : 2002-2007, Mats 
+# Rewritten  : 01/25/2008, Roger 
+# ------------------------------ ------------------------------ --------- 
+
+proc can2svg::ImageCoordsToAttr {coo opts} { 
     
     array set optArr {-anchor nw}
     array set optArr $opts
-    if {[info exists optArr(-image)]} {
-	set theImage $optArr(-image)
+    
+    if {![info exists optArr(-image)]} { 
+	return -code error "Missing -image option; can't parse that" 
+    } 
+    set theImage $optArr(-image) 
+
+    lassign $coo x0 y0
+    set w [image width $theImage]
+    set h [image height $theImage]
+    
+    set x [expr {$x0 - $w/2.0}] 
+    set y [expr {$y0 - $h/2.0}] 
+
+    if { "center" ne $optArr(-anchor) } { 
+	foreach orientation [split $optArr(-anchor) {}] { 
+	    switch $orientation { 
+		n { set y $y0 } 
+		s { set y [expr {$y0 - $w}] } 
+		e { set x [expr {$x0 - $h}] } 
+		w { set x $x0 } 
+		default {} 
+	    } 
+	} 
+    } 
+    return [list "x" $x "y" $y "width" $w "height" $h] 
+}
+
+proc can2svg::ImageCoordsToAttrBU {coo opts} {
+    array set optA {-anchor nw}
+    array set optA $opts
+    if {[info exists optA(-image)]} {
+	set theImage $optA(-image)
 	set w [image width $theImage]
 	set h [image height $theImage]
     } else {
@@ -1008,7 +1085,7 @@ proc can2svg::ImageCoordsToAttr {coo opts} {
     }
     foreach {x0 y0} $coo break
     
-    switch -- $optArr(-anchor) {
+    switch -- $optA(-anchor) {
 	nw {
 	    set x $x0
 	    set y $y0
@@ -1336,10 +1413,9 @@ proc can2svg::NormalizeRectCoords {coo} {
 proc can2svg::makedocument {width height xml} {
     
     set pre "<?xml version='1.0'?>\n\
-      <!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.0//EN\"\
-      \"http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd\">"
-    
-    set svgStart "<svg width='$width' height='$height'>"
+      <!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\"\
+      \"Graphics/SVG/1.1/DTD/svg11.dtd\">"
+    set svgStart "<svg width='$width' height='$height' version='1.1' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'>"
     set svgEnd "</svg>"
     return "${pre}\n${svgStart}\n${xml}${svgEnd}"
 }
@@ -1363,11 +1439,11 @@ proc can2svg::canvas2file {wcan path args} {
     variable defsArrowMarkerArr
     variable defsStipplePatternArr
     
-    array set argsArr [array get confopts]
+    array set argsA [array get confopts]
     foreach {x y width height} [$wcan cget -scrollregion] break
-    array set argsArr [list -width $width -height $height]
-    array set argsArr $args
-    set args [array get argsArr]
+    array set argsA [list -width $width -height $height]
+    array set argsA $args
+    set args [array get argsA]
     
     # Need to make a fresh start for marker def's.
     unset -nocomplain defsArrowMarkerArr defsStipplePatternArr
@@ -1394,7 +1470,7 @@ proc can2svg::canvas2file {wcan path args} {
 	set cmd [concat "create" $type $co $opcmd]
 	append xml "\t[eval {can2svg $cmd} $args]\n"	
     }
-    puts $fd [makedocument $argsArr(-width) $argsArr(-height) $xml]
+    puts $fd [makedocument $argsA(-width) $argsA(-height) $xml]
     close $fd
 }
 
@@ -1469,12 +1545,12 @@ proc can2svg::MakeXMLList {tagname args} {
     if {[llength $args] > 0} {
 	array set xmlarr $args
     }
-    if {!(($xmlarr(-chdata) == "") && ($xmlarr(-subtags) == ""))} {
+    if {!(($xmlarr(-chdata) eq "") && ($xmlarr(-subtags) eq ""))} {
 	set xmlarr(-isempty) 0
     }
     
     # Build sub elements list.
-    set sublist {}
+    set sublist [list]
     foreach child $xmlarr(-subtags) {
 	lappend sublist $child
     }
