@@ -5,10 +5,11 @@
 #      side and server side.
 #      Modelled after the http package with modifications.
 #      
-#  Copyright (c) 2002-2004  Mats Bengtsson only for the new and rewritten parts.
-#  This source file is distributed under the BSD license.
+#  Copyright (c) 2002-2007  Mats Bengtsson only for the new and rewritten parts.
+#  
+#  This file is distributed under BSD style license.
 #
-# $Id: httpex.tcl,v 1.18 2004/11/30 15:11:10 matben Exp $
+# $Id: httpex.tcl,v 1.29 2008-02-23 06:36:17 matben Exp $
 # 
 # USAGE ########################################################################
 #
@@ -50,6 +51,10 @@
 #       The '-progress' argument is a tcl procedure: 
 #       progressProc {token totalsize currentsize}
 #       'totalsize' can be 0 if no Content-Length attribute.
+#       
+# httpex::geturl url ?-channel channame -key value ...?
+#       Simplified httpex::get which only invokes any -command when final.
+#       Can be used in place for http::geturl when getting url.
 #
 # httpex::head url ?-key value ...?
 #       Makes a HEAD request to url. The -channel key is not allowed.
@@ -60,6 +65,10 @@
 #	  -query data
 #	  -querychannel name
 #       
+# httpex::posturl url ?-key value ...?
+#       Simplified httpex::post which only invokes any -command when final.
+#       Can be used in place for http::geturl -query when POST url.
+#       
 # httpex::put url ?-key value ...?
 #       Puts local data to server. The channame is typically a file descriptor
 #       where data shall be read if no -putdata option.
@@ -123,6 +132,9 @@ namespace eval httpex {
     variable debug 0
     variable codeToText
     
+    # Test avoiding fcopy and use fileevents instead.
+    variable useFcopy 0
+    
     # Only for the config command.
     array set opts {
 	-accept        */*
@@ -160,6 +172,12 @@ namespace eval httpex {
     # This can be changed, but iso8859-1 is the RFC standard.
     variable defaultCharset "iso8859-1"
     
+    # @@ Unclear: let certain Content-Type (MIME type) set the charset.
+    variable typeCharsetMap
+    array set typeCharsetMap {
+	application/xml   utf-8
+    }
+    
     array set codeToText {
 	100 Continue
 	101 {Switching Protocols}
@@ -219,7 +237,7 @@ namespace eval httpex {
     }
 
     # Client side options.
-    set locals(opts,get) {-binary -blocksize -channel -command -headers \
+    set locals(opts,get) {-binary -blocksize -channel -command -handler -headers \
       -httpvers -persistent -progress -socket -timeout -type}
     set locals(opts,head) {-binary -command -headers -httpvers -persistent \
       -socket -timeout -type}
@@ -334,6 +352,34 @@ proc httpex::put {url args} {
     return [eval {Request put $url} $args]
 }
 
+# httpex::geturl --
+# 
+#       As httpex::get but -command only invoked when final.
+
+proc httpex::geturl {url args} {
+    set argsA(-command) ""
+    array set argsA $args
+    set argsA(-command) [list [namespace current]::compatcmd $argsA(-command)]
+    eval {get $url} [array get argsA]
+}
+
+# httpex::posturl --
+# 
+#       As httpex::post but -command only invoked when final.
+
+proc httpex::posturl {url args} {
+    set argsA(-command) ""
+    array set argsA $args
+    set argsA(-command) [list [namespace current]::compatcmd $argsA(-command)]
+    eval {post $url} [array get argsA]
+}
+
+proc httpex::compatcmd {cmd token} {
+    if {([state $token] eq "final") && [llength $cmd]} {
+	uplevel #0 $cmd $token
+    }
+}
+
 # httpex::Request --
 #
 #	Initiates a GET, HEAD, POST, or PUT HTTP request.
@@ -553,7 +599,6 @@ proc httpex::Request {method url args} {
 #	Proceeds with the httpex protocol,
 
 proc httpex::Connect {token} {    
-    global  tcl_platform
     variable $token
     variable opts
     variable locals
@@ -565,17 +610,10 @@ proc httpex::Connect {token} {
     fileevent $s writable {}
     catch {after cancel $state(after)}
     
-    if {[string equal $tcl_platform(platform) "macintosh"]} {
-	if {[catch {eof $s} iseof] || $iseof} {
-	    Eof $token $iseof
-	    return
-	}
-    } else {
-	if {[catch {eof $s} iseof] || $iseof ||  \
-	  [string length [fconfigure $s -error]]} {
-	    Eof $token $iseof
-	    return
-	}
+    if {[catch {eof $s} iseof] || $iseof ||  \
+      [string length [fconfigure $s -error]]} {
+	Eof $token $iseof
+	return
     }
     
     # On track :-)
@@ -837,8 +875,9 @@ proc httpex::readrequest {s callback args} {
 
 proc httpex::Event {token} {    
     variable $token
-    variable locals
     upvar 0 $token state
+    variable locals
+    variable useFcopy
     
     Debug 1 "httpex::Event state(state)=$state(state)"
     
@@ -859,6 +898,7 @@ proc httpex::Event {token} {
 	Finish $token $n
     } elseif {$n == 0} {
 	variable encodings
+	variable defaultCharset
 	
 	Debug 2 "\tn=$n"
 	fileevent $s readable {}
@@ -879,8 +919,15 @@ proc httpex::Event {token} {
 	    }
 	}
 	
-	if {$state(-binary) || ![regexp -nocase ^text $state(type)] || \
-	  [regexp gzip|compress $state(coding)]} {
+	# Figure out translation and encoding issues.
+	set binary 0
+	if {$state(-binary) || [regexp gzip|compress $state(coding)]} {
+	    set binary 1
+	} elseif {![regexp -nocase ^text $state(type)] && \
+	  [string equal $state(charset) $defaultCharset]} {
+	    set binary 1
+	}	
+	if {$binary} {
 	    Debug 2 "\tfconfigure $s -translation binary"
 	    
 	    # Turn off conversions for non-text data
@@ -894,10 +941,10 @@ proc httpex::Event {token} {
 	    # encoding correctly.  iso8859-1 is the RFC default, but
 	    # this could be any IANA charset.  However, we only know
 	    # how to convert what we have encodings for.
-	    set idx [lsearch -exact $encodings \
-	      [string tolower $state(charset)]]
+	    set idx [lsearch -exact $encodings [string tolower $state(charset)]]
 	    if {$idx >= 0} {
 		fconfigure $s -encoding [lindex $encodings $idx]
+		# @@@ -channel ?
 		Debug 2 "\tfconfigure -encoding [lindex $encodings $idx]"
 	    }
 	}
@@ -920,21 +967,24 @@ proc httpex::Event {token} {
 	    if {[info exists state(-command)]} {
 		uplevel #0 $state(-command) $token
 	    }
-	    if {[info exists state(-channel)]} {
+	    if {$useFcopy && [info exists state(-channel)]} {
 		
 		# Initiate a sequence of background fcopies
 		CopyStart $s $token
 	    } else {
-		Debug 2 "\tfileevent readable httpex::Read"
-		fileevent $s readable [list httpex::Read $s $token]
+		SetReadable $s $token
 	    }
 	}
     } elseif {$n > 0} {
+	variable typeCharsetMap
 	Debug 2 "\tline=$line"
 	
 	if {[regexp -nocase {^content-type:(.+)$} $line x type]} {
-	    set state(type) [string trim $type]
-	    
+	    set type [string trim $type]
+	    set state(type) $type	    
+	    if {[info exists typeCharsetMap($type)]} {
+		set state(charset) $typeCharsetMap($type)
+	    }
 	    # grab the optional charset information
 	    regexp -nocase {charset\s*=\s*(\S+)} $type x state(charset)
 	} elseif {[regexp -nocase {^content-length:(.+)$} $line x length]} {
@@ -948,7 +998,7 @@ proc httpex::Event {token} {
 	    set state(chunked) 1
 	} elseif {[regexp -nocase {^range: *bytes=(.+)$} $line x byteSet]} {
 	    set state(haveranges) 1
-	    set ranges {}
+	    set ranges [list]
 	    foreach byteSpec [split $byteSet ,] {
 		if {[regexp -- {^([0-9]+)-([0-9]+)$} $byteSpec x lower upper]} {
 		    lappend ranges [list $lower $upper]
@@ -963,8 +1013,8 @@ proc httpex::Event {token} {
 	}
 	if {[regexp -nocase {^([^:]+):(.+)$} $line x key value]} {
 	    lappend state(meta) $key [string trim $value]
-	} elseif {[regexp {^HTTP/([0-9]+\.[0-9]+) +([0-9]{3})} $line  \
-	  match httpvers ncode]} {
+	} elseif {[regexp {^HTTP/([0-9]+\.[0-9]+) +([0-9]{3})} $line \
+	  - httpvers ncode]} {
 	    
 	    # Only clients.
 	    set state(http) $line
@@ -1178,8 +1228,8 @@ proc httpex::CopyStart {s token} {
 	set blocksize [expr {$state(totalsize) - $state(currentsize)}]
     }
     if {[catch {
-	fcopy $s $state(-channel) -size $blocksize -command \
-	    [list httpex::CopyDone $token]
+	fcopy $s $state(-channel) -size $blocksize \
+	  -command [list httpex::CopyDone $token]
     } err]} {
 	Finish $token $err
     }
@@ -1275,7 +1325,7 @@ proc httpex::Eof {token {iseof 0}} {
 
 # httpex::Read
 #
-#	Reads from socket to internal variable.
+#	Reads from socket to internal variable or puts to channel.
 #	Used by methods: get, serverpost, serverput.
 #
 # Arguments
@@ -1291,6 +1341,8 @@ proc httpex::Read {s token} {
     
     Debug 1 "httpex::Read"
     
+    fileevent $s readable {}
+
     if {[catch {eof $s} iseof] || $iseof} {
 	Eof $token
 	return
@@ -1306,10 +1358,14 @@ proc httpex::Read {s token} {
 	if {[info exists state(-handler)]} {
 	    set n [eval $state(-handler) {$s $token}]
 	} else {
-	    set block [read $s $blocksize]
-	    set n [string length $block]
+	    set data [read $s $blocksize]
+	    set n [string length $data]
 	    if {$n >= 0} {
-		append state(body) $block
+		if {[info exists state(-channel)]} {
+		    puts -nonewline $state(-channel) $data
+		} else {
+		    append state(body) $data
+		}
 	    }
 	}
 	if {$n >= 0} {
@@ -1332,6 +1388,21 @@ proc httpex::Read {s token} {
 	} else {
 	    Eof $token
 	}
+    } else {
+	
+	# This is a trick to put this event at the back of the queue to
+	# avoid using any 'update'.
+	after idle [list after 0 [list \
+	  [namespace current]::SetReadable $s $token]]
+    }
+}
+
+proc httpex::SetReadable {s token} {    
+    
+    # We could have been closed since this event comes async.
+    if {[lsearch [file channels] $s] >= 0} {
+	fileevent $s readable  \
+	  [list [namespace current]::Read $s $token]
     }
 }
 
@@ -1514,9 +1585,7 @@ proc httpex::cleanup {token} {
 
     Debug 1 "httpex::cleanup"
     
-    if {[info exist state]} {
-	unset state
-    }
+    unset -nocomplain state
 }
 
 # httpex::Finish --
@@ -1545,6 +1614,10 @@ proc httpex::Finish {token {errormsg ""} {skipCB 0}} {
 
     Debug 1 "httpex::Finish errormsg=$errormsg, skipCB=$skipCB"
     
+    # A -handler could have done cleanup on us.
+    if {![info exists state]} {
+	return
+    }
     set s $state(-socket)
     set doClose 0
     if {[string length $errormsg] != 0} {
@@ -1604,9 +1677,9 @@ proc httpex::Finish {token {errormsg ""} {skipCB 0}} {
 		set state(status) error
 	    }
 	}
-	if {$doClose && [info exist state(-command)]} {
+	if {$doClose} {
 	    # Command callback may already have unset our state
-	    unset state(-command)
+	    unset -nocomplain state(-command)
 	}
     }
 }
@@ -1661,7 +1734,7 @@ proc httpex::ncode {token} {
     if {[info exists state(ncode)]} {
 	return $state(ncode)
     } else {
-	return ""
+	return
     }
 }
 proc httpex::ncodetotext {ncode} {
@@ -1669,7 +1742,7 @@ proc httpex::ncodetotext {ncode} {
     if {[info exists codeToText($ncode)]} {
 	return $codeToText($ncode)
     } else {
-	return ""
+	return
     }
 }
 proc httpex::error {token} {
@@ -1984,7 +2057,7 @@ if {0} {
 		    return [list 500 $headlist]
 		}
 		if {[catch {clock format [file mtime $abspath]  \
-		  -format "%a, %d %b %Y %H:%M:%S GMT" -gmt 1} modTime]} {
+		  -format "%a, %d %b %Y %H:%M:%S GMT" -timezone :UTC} modTime]} {
 		    return [list 404 $headlist]
 		} else {
 		    set headlist [list Last-Modified $modTime]
