@@ -4,14 +4,17 @@
 #      Some code plus idee from Kerem 'Waster_' HADIMLI.
 #      Made from RFC 1928.
 #
-#  (C) 2000 Kerem 'Waster_' HADIMLI (minor parts)
-#  (c) 2003  Mats Bengtsson
+#  (C) 2000  Kerem 'Waster_' HADIMLI (minor parts)
+#  (c) 2003-2007  Mats Bengtsson
+#  
 #  This source file is distributed under the BSD license.
 #  
-# $Id: socks5.tcl,v 1.11 2004/10/12 13:48:56 matben Exp $
+# $Id: socks5.tcl,v 1.20 2007-07-19 06:28:11 matben Exp $
 # 
 # TODO:  GSSAPI authentication which is a MUST is missing.
 #        Only CMD CONNECT implemented.
+#        Do not report English text in callback but rather error keys like
+#        rsp_notallowed etc. Client done, server to go.
 
 package provide socks5 0.1
 
@@ -47,6 +50,21 @@ namespace eval socks5 {
 	rsp_cmdunsupported  \x07
 	rsp_addrunsupported \x08
     }
+
+    # Practical when mapping errors to error codes.
+    variable iconst
+    array set iconst {
+	\x00    rsp_succeeded
+	\x01    rsp_failure
+	\x02    rsp_notallowed
+	\x03    rsp_netunreachable
+	\x04    rsp_hostunreachable
+	\x05    rsp_refused
+	\x06    rsp_expired
+	\x07    rsp_cmdunsupported
+	\x08    rsp_addrunsupported
+    }
+    
     variable ipv4_num_re {([0-9]{1,3}\.){3}[0-9]{1,3}}
     variable ipv6_num_re {([0-9a-fA-F]{4}:){7}[0-9a-fA-F]{4}}
     
@@ -62,7 +80,7 @@ namespace eval socks5 {
 	8 "Address type not supported"
     }
     
-    variable debug 3
+    variable debug 0
     variable uid 0
 }
 
@@ -75,7 +93,7 @@ namespace eval socks5 {
 #       addr:       the peer address, not SOCKS server
 #       port:       the peer's port number
 #       args:   
-#               -command    tclProc {token type args}
+#               -command    tclProc {token status}
 #               -username   username
 #               -password   password
 #               -timeout    millisecs
@@ -88,7 +106,7 @@ proc socks5::init {sock addr port args} {
     variable const
     variable uid
     
-    debug 2 "socks5::init"
+    debug 2 "socks5::init $addr $port $args"
 
     # Initialize the state variable, an array.  We'll return the
     # name of this array as the token for the transaction.
@@ -136,7 +154,7 @@ proc socks5::init {sock addr port args} {
 	puts -nonewline $sock "$const(ver)$nmethods$methods"
 	flush $sock
     } err]} {
-	return -code error $err
+	return -code error network-failure
     }
 
     # Setup timeout timer. !async remains!
@@ -167,7 +185,7 @@ proc socks5::response_method {token} {
     set sock $state(sock)
     
     if {[catch {read $sock 2} data] || [eof $sock]} {
-	finish $token eof
+	finish $token network-failure
 	return
     }    
     set serv_ver ""
@@ -176,7 +194,7 @@ proc socks5::response_method {token} {
     debug 2 "\tserv_ver=$serv_ver, smethod=$smethod"
     
     if {![string equal $serv_ver 5]} {
-	finish $token "Socks server isn't version 5!"
+	finish $token err_version
 	return
     }
     
@@ -188,7 +206,7 @@ proc socks5::response_method {token} {
 	
 	# User/Pass authorization required
 	if {$state(auth) == 0} {
-	    finish $token "User/Pass authorization required by Socks Server!"
+	    finish $token err_authorization_required
 	    return
 	}
     
@@ -202,7 +220,7 @@ proc socks5::response_method {token} {
 	      "$const(auth_userpass)$ulen$state(-username)$plen$state(-password)"
 	    flush $sock
 	} err]} {
-	    finish $token $err
+	    finish $token network-failure
 	    return
 	}
 
@@ -218,7 +236,7 @@ proc socks5::response_method {token} {
 	    return [response_auth $token]
 	}
     } else {
-	finish $token "Method not supported by Socks Server!"
+	finish $token err_unsupported_method
 	return
     }
 }
@@ -232,7 +250,7 @@ proc socks5::response_auth {token} {
     set sock $state(sock)
 
     if {[catch {read $sock 2} data] || [eof $sock]} {
-	finish $token eof
+	finish $token network-failure
 	return
     }    
     set auth_ver ""
@@ -241,11 +259,11 @@ proc socks5::response_auth {token} {
     debug 2 "\tauth_ver=$auth_ver, status=$status"
     
     if {![string equal $auth_ver 1]} {
-	finish $token "Socks Server's authentication isn't supported!"
+	finish $token err_authentication_unsupported
 	return
     }
     if {![string equal $status 0]} {
-	finish $token "Wrong username or password!"
+	finish $token err_authorization
 	return
     }	
     
@@ -296,7 +314,7 @@ proc socks5::request {token} {
 	puts -nonewline $sock "$aconst$atyp_addr_port"
 	flush $sock
     } err]} {
-	finish $token $err
+	finish $token network-failure
 	return
     }
     
@@ -316,8 +334,8 @@ proc socks5::request {token} {
 
 proc socks5::response {token} {
     variable $token
-    variable msg
     upvar 0 $token state    
+    variable iconst
     
     debug 2 "socks5::response"
     
@@ -326,7 +344,7 @@ proc socks5::response {token} {
     
     # Start by reading ver+cmd+rsv.
     if {[catch {read $sock 3} data] || [eof $sock]} {
-	finish $token eof
+	finish $token network-failure
 	return
     }        
     set serv_ver ""
@@ -334,22 +352,17 @@ proc socks5::response {token} {
     binary scan $data ccc serv_ver rep rsv
     
     if {![string equal $serv_ver 5]} {
-	finish $token "Socks server isn't version 5!"
+	finish $token err_version
 	return
     }
-
-    switch -- $rep {
-	0 {
-	    #fconfigure $sock -translation {auto auto}
-	}
-	1 - 2 - 3 - 4 - 5 - 6 - 7 - 8 {
-	    finish $token $msg($rep)
-	    return
-	}
-	default {
-	    finish $token "Socks server responded: Unknown Error"
-	    return
-	}    
+    if {$rep == 0} {
+	# OK
+    } elseif {[info exists iconst($rep)]} {
+	finish $token $iconst($rep)
+	return
+    } else {
+	finish $token err_unknown
+	return
     }
 
     # Now parse the variable length atyp+addr+host.
@@ -379,7 +392,7 @@ proc socks5::parse_atyp_addr {token addrVar portVar} {
 
     # Start by reading atyp.
     if {[catch {read $sock 1} data] || [eof $sock]} {
-	return -code error eof
+	return -code error network-failure
     }        
     set atyp ""
     binary scan $data c atyp
@@ -389,7 +402,7 @@ proc socks5::parse_atyp_addr {token addrVar portVar} {
     switch -- $atyp {
 	1 {
 	    if {[catch {read $sock 6} data] || [eof $sock]} {
-		return -code error eof
+		return -code error network-failure
 	    }        
 	    binary scan $data ccccS i0 i1 i2 i3 port
 	    set addr ""
@@ -405,18 +418,18 @@ proc socks5::parse_atyp_addr {token addrVar portVar} {
 	}
 	3 {
 	    if {[catch {read $sock 1} data] || [eof $sock]} {
-		return -code error eof
+		return -code error network-failure
 	    }        
 	    binary scan $data c len
 	    debug 2 "\tlen=$len"
 	    set len [expr ( $len + 0x100 ) % 0x100]
 	    if {[catch {read $sock $len} data] || [eof $sock]} {
-		return -code error eof
+		return -code error network-failure
 	    }        
 	    set addr $data
 	    debug 2 "\taddr=$addr"
 	    if {[catch {read $sock 2} data] || [eof $sock]} {
-		return -code error eof
+		return -code error network-failure
 	    }        
 	    binary scan $data S port
 	    # Translate to unsigned!
@@ -427,7 +440,7 @@ proc socks5::parse_atyp_addr {token addrVar portVar} {
 	    # todo
 	}
 	default {
-	    return -code error "Unknown address type"
+	    return -code error err_unknown_address_type
 	}
     }
 }
@@ -440,36 +453,39 @@ proc socks5::finish {token {errormsg ""}} {
     debug 2 "socks5::finish errormsg=$errormsg"
     catch {after cancel $state(timeoutid)}
         
+    # In case of error we do the cleanup.
     if {$state(async)} {
 	if {[string length $errormsg]} {
 	    catch {close $state(sock)}
-	    uplevel #0 $state(-command) [list $token error $errormsg]
-	    cleanup $token
+	    uplevel #0 $state(-command) [list $token $errormsg]
+	    free $token
 	} else {
 	    uplevel #0 $state(-command) [list $token ok]
 	}
     } else {
 	if {[string length $errormsg]} {
 	    catch {close $state(sock)}
-	    cleanup $token
+	    free $token
 	    return -code error $errormsg
 	} else {
-	    return ""
+	    return
 	}
     }
 }
 
-proc socks5::timeout {token} {
+proc socks5::getipandport {token} {
     variable $token
-    upvar 0 $token state
-    
+    upvar 0 $token state   
+    return [list $state(bnd_addr) $state(bnd_port)]
+}
+
+proc socks5::timeout {token} {
     finish $token timeout
 }
 
-proc socks5::cleanup {token} {
+proc socks5::free {token} {
     variable $token
     upvar 0 $token state
-    
     unset -nocomplain state
 }
 
@@ -525,7 +541,7 @@ proc socks5::serverinit {sock ip port command args} {
 
     # Start by reading the method stuff.
     if {[catch {read $sock 2} data] || [eof $sock]} {
-	serv_finish $token eof
+	serv_finish $token network-failure
 	return
     }    
     set ver ""
@@ -541,7 +557,7 @@ proc socks5::serverinit {sock ip port command args} {
     }
     for {set i 0} {$i < $nmethods} {incr i} {
 	if {[catch {read $sock 1} data] || [eof $sock]} {
-	    serv_finish $token eof
+	    serv_finish $token network-failure
 	    return
 	}    
 	binary scan $data c method
@@ -598,7 +614,7 @@ proc socks5::serv_auth {token} {
     fileevent $sock readable {}
     
     if {[catch {read $sock 2} data] || [eof $sock]} {
-	serv_finish $token eof
+	serv_finish $token network-failure
 	return
     }    
     set auth_ver ""
@@ -611,19 +627,19 @@ proc socks5::serv_auth {token} {
 	return
     }    
     if {[catch {read $sock $ulen} data] || [eof $sock]} {
-	return -code error eof
+	return -code error network-failure
     }        
     set state(username) $data
     debug 2 "\tusername=$data"
     if {[catch {read $sock 1} data] || [eof $sock]} {
-	serv_finish $token eof
+	serv_finish $token network-failure
 	return
     }        
     binary scan $data c plen
     set plen [expr ( $plen + 0x100 ) % 0x100]
     debug 2 "\tplen=$plen"
     if {[catch {read $sock $plen} data] || [eof $sock]} {
-	serv_finish $token eof
+	serv_finish $token network-failure
 	return
     }        
     set state(password) $data
@@ -665,7 +681,7 @@ proc socks5::serv_request {token} {
 
     # Start by reading ver+cmd+rsv.
     if {[catch {read $sock 3} data] || [eof $sock]} {
-	serv_finish $token eof
+	serv_finish $token network-failure
 	return
     }        
     set ver ""
@@ -696,7 +712,7 @@ proc socks5::serv_request {token} {
     # Init the SOCKS connection to dst if wanted. Else???
     if {$state(-opendstsocket)} {
 	if {[catch {socket -async $addr $port} sock_dst]} {
-	    serv_finish $token eof
+	    serv_finish $token network-failure
 	    return
 	}
 	set state(sock_dst) $sock_dst
@@ -723,7 +739,7 @@ proc socks5::serv_dst_connect {token} {
     
     set sock_dst $state(sock_dst)
     if {[eof $sock_dst]} {
-	serv_finish $token eof
+	serv_finish $token network-failure
 	return
     }
     
@@ -733,7 +749,7 @@ proc socks5::serv_dst_connect {token} {
 	  break
     } err]} {
 	debug 2 "\tfconfigure failed: $err"
-	serv_finish $token eof
+	serv_finish $token network-failure
 	return
     }
     array set state [list bnd_ip $bnd_ip bnd_addr $bnd_addr bnd_port $bnd_port]
@@ -813,7 +829,7 @@ proc socks5::read_stream {token in out} {
     } elseif {[catch {eof $out} iseof] || $iseof} {
 	serv_finish $token
     } elseif {[catch {read $in} data]} {
-	serv_finish $token eof
+	serv_finish $token network-failure
     } else {
 	
 	# We could wait here (in the event loop) for channel to be writable
@@ -825,7 +841,7 @@ proc socks5::read_stream {token in out} {
 	    vwait $token\(writetrigger${primary})
 	}
 	if {[catch {puts -nonewline $out $data; flush $out}]} {
-	    serv_finish $token eof
+	    serv_finish $token network-failure
 	}
     }
 }
@@ -847,7 +863,7 @@ proc socks5::serv_finish {token {errormsg ""}} {
 	catch {close $state(sock_dst)}
     }        
     if {[string length $errormsg]} {
-	uplevel #0 $state(command) [list $token error -error $errormsg]
+	uplevel #0 $state(command) [list $token $errormsg]
     } else {
 	uplevel #0 $state(command) [list $token ok]
     }
@@ -882,12 +898,9 @@ proc socks5::debug {num str} {
 if {0} {
     
     # Server
-    proc serv_cmd {token type args} {
-	puts "server: token=$token, type=$type, args=$args"	
-	switch -- $type {
-	    error {
-		
-	    }
+    proc serv_cmd {token status} {
+	puts "server: token=$token, status=$status"	
+	switch -- $status {
 	    ok {
 		
 	    }
@@ -895,6 +908,9 @@ if {0} {
 		# Here we should check that the username and password is ok.
 		return 1
 	    }
+	    default {
+		puts "error $status"
+	    }
 	}	    
     }
     proc server_connect {sock ip port} {
@@ -904,10 +920,10 @@ if {0} {
     socket -server server_connect 1080
     
     # Client
-    proc cb {token type args} {
+    proc cb {token status} {
 	global s
-	puts "client: token=$token, type=$type, args=$args"
-	if {$type == "ok"} {
+	puts "client: token=$token, status=$status"
+	if {$status eq "ok"} {
 	    fconfigure $s -buffering none
 	}
     }
