@@ -4,7 +4,9 @@
 #      
 #  Copyright (c) 2002-2003  Mats Bengtsson
 #  
-# $Id: tinyfileutils.tcl,v 1.5 2004/12/04 15:01:06 matben Exp $
+#  This file is distributed under BSD style license.
+#  
+# $Id: tinyfileutils.tcl,v 1.11 2008-02-28 07:51:25 matben Exp $
 
 package provide tinyfileutils 1.0
 
@@ -197,7 +199,11 @@ proc ::tfileutils::appendfile {dstFile args} {
 #   returns a file name
 #
 
-proc ::tfileutils::tempfile {tmpdir prefix} {
+proc ::tfileutils::tempfile {tmpdir {prefix {}}} {
+    return [file normalize [TempFile $tmpdir $prefix]]
+}
+
+proc ::tfileutils::TempFile {tmpdir prefix} {
 
     set chars "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
     set nrand_chars 10
@@ -206,12 +212,12 @@ proc ::tfileutils::tempfile {tmpdir prefix} {
     set permission 0600
     set channel ""
     set checked_dir_writable 0
-    set mypid [pid]
+
     for {set i 0} {$i < $maxtries} {incr i} {
 	set newname $prefix
 	for {set j 0} {$j < $nrand_chars} {incr j} {
 	    append newname [string index $chars \
-	      [expr {([clock clicks] ^ $mypid) % 62}]]
+		    [expr {int(rand()*62)}]]
 	}
 	set newname [file join $tmpdir $newname]
 	if {[file exists $newname]} {
@@ -239,23 +245,15 @@ proc ::tfileutils::tempfile {tmpdir prefix} {
     }
 }
 
-# This could be a smarter way.
+# tfileutils::deleteallfiles --
+# 
+#       Deletes all files in a directory but keeps the directory.
 
-proc ::tfileutils::newuid { } {
-    
-    set tm1 [clock format [clock seconds] -format "%m%d%H%M%S"]
-    set hx1 [format %X [string trimleft $tm1 0]]
-    set tm2 [string range [clock clicks -milliseconds] end-2 end]
-    set hx2 [format %X [string trimleft $tm2 0]]
-    if {$hx2 == ""} {
-	set hx2 0
+proc ::tfileutils::deleteallfiles {dir {pattern *}} {
+    set files [eval {glob -nocomplain -directory $dir} $pattern]
+    if {[llength $files]} {
+	eval {file delete -force} $files
     }
-    set ran [string trimleft [string range [expr rand()] end-7 end] 0.]
-    set hx3 [format %X $ran]
-    if {$hx3 == ""} {
-	set hx3 0
-    }
-    return ${hx1}-${hx2}-${hx3}
 }
 
 #------------------------------------------------------------------------------
