@@ -3,12 +3,13 @@
 #       This file is part of The Coccinella application. It implements a tiny
 #       http server.
 #      
-#  Copyright (c) 2002-2004  Mats Bengtsson
-#  This source file is distributed under the BSD license.
+#  Copyright (c) 2002-2006  Mats Bengtsson
+#  
+#  This file is distributed under BSD style license.
 #  
 #  See the README file for license, bugs etc.
 #  
-# $Id: tinyhttpd.tcl,v 1.24 2004/12/02 08:22:33 matben Exp $
+# $Id: tinyhttpd.tcl,v 1.35 2008-02-18 13:54:38 matben Exp $
 
 # ########################### USAGE ############################################
 #
@@ -25,12 +26,14 @@
 #      ::tinyhttpd::avergaebytespersec
 #      ::tinyhttpd::cleanup
 #      ::tinyhttpd::mount absPath name
+#      ::tinyhttpd::mountfile absFilePath
 #      ::tinyhttpd::unmount name
 #      ::tinyhttpd::allmounted
 #      ::tinyhttpd::registercgicommand path command
 #      ::tinyhttpd::unregistercgicommand path
 #      ::tinyhttpd::allcgibins
 #      ::tinyhttpd::putheader token httpcode ?-headers list?
+#      ::tinyhttpd::configure ?-option?
 #      
 # ##############################################################################
 
@@ -39,7 +42,7 @@ package require uriencode
 
 package provide tinyhttpd 1.0
 
-namespace eval ::tinyhttpd:: {
+namespace eval ::tinyhttpd {
         
     # Keep track of useful things.
     variable priv
@@ -50,12 +53,19 @@ namespace eval ::tinyhttpd:: {
     variable this
     variable timing
     variable mounts
+    variable fmounts
     variable cgibin
+    variable langfile
+    variable inited 0
     variable uid 0
-        
+    
+    # Workaround for the "fcopy bug".
+    variable useFcopy 0 
+    
     set this(path)          [file dirname [info script]]
-    set this(httpvers)      1.1
+    set this(httpvers)      1.0
     set this(accept-ranges) 1
+    set langfile [file join [file dirname [info script]] tinyhttpdmsg.tcl]
 
     set this(debug) 0
     
@@ -105,7 +115,7 @@ namespace eval ::tinyhttpd:: {
     # Use variables to store typical html return messages instead of files.
     set html(404) {<HTML><HEAD>
 	<TITLE>File Not Found</TITLE>
-	</HEAD><BODY BGCOLOR="#FFA6FF" TEXT=black>
+	</HEAD><BODY TEXT=black>
 	<FONT SIZE="5" COLOR="#CC0033" FACE="Arial,Helvetica,Verdana,sans-serif">
 	<B> Error 404: The file was not found on the server. </B></FONT><P>
 	<FONT SIZE="2" FACE="Arial,Helvetica,Verdana,sans-serif">
@@ -164,137 +174,109 @@ Server: tinyhttpd/1.0"
     # These shall be used with 'format' to make html dir listings.
     set html(head,plain) {
 <html><head><title> %s </title></head>
-<body bgcolor="white">
+<body>
 <!-- HTML generated by tinyhttpd -->}
 
     set html(updir,css) {}
 
-    set html(tabletop,css) {
-	<table nowrap border=0 cellpadding="2">
+    set html(tabletop,plain) {
+	<table class="center" border=0 cellpadding="2">
 	    <tr>
-	        <td align=center nowrap height=19> %s objects </td>
-		<td align=left nowrap><b>Name</b></td>
-		<td align=right nowrap>Size</td>
-		<td align=left nowrap></td>
-		<td align=left nowrap>Type</td>
-		<td align=left nowrap></td>
-		<td align=left nowrap>Date</td>
+		<td align="left"><b>@Name</b></td>
+		<td align="right">@Size</td>
+		<td align="left"></td>
+		<td align="left">@Type</td>
+		<td align="left"></td>
+		<td align="left">@Date</td>
 	    </tr>
 	    <tr>
-	        <td colspan=7 height=10><hr></td>
+	        <td colspan="7" height="10"><hr></td>
 	    </tr>
     }
 
     set html(updir,plain) {
-	<a href="%s"><img border=0 src="%s">
-	    Parent directory "%s"
+	<a href="%s"><img src="%s" alt=""/>
+	    @Parent (%s)
 	</a>
-	<br>
     }
         
     set html(line,plain) {
     <tr>
-	<td align=center nowrap height=17>
-	    <a href="%s"><img border=0 width=16 height=16 src="%s"></a>
-	</td>
-	<td align=left nowrap><a href="%s"> %s </a></td>
-	<td align=right nowrap> %s </td>
-	<td align=left nowrap width="16">&nbsp;</td>
-	<td align=left nowrap> %s </td>
-	<td align=left nowrap width="16">&nbsp;</td>
-	<td align=left nowrap> %s </td>
+	<td align="left"><a href="%s"><a href="%s"><img width="16" height="16" src="%s" alt=""/> %s </a></td>
+	<td align="right"> %s </td>
+	<td align="left" width="16">&nbsp;</td>
+	<td align="left"> %s </td>
+	<td align="left" width="16">&nbsp;</td>
+	<td align="left"> %s </td>
     </tr>}
     
     set html(dirbottom,plain) {
     </table>
-    <br><br>
+    <br/><br/>
     </body>
     </html>}
     
     # Using CSS.
-    set html(head,css) {
-	<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
-	<html>
+    set html(head,css) {<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
+	<html xmlns='http://www.w3.org/1999/xhtml' xml:lang='en'>
 	<head>
-	    <title>The Coccinella: directory listing</title> 	
-	    <style type="text/css" media="screen">
-	        @import "/httpd/std.css";
-	    </style>
+	    <title>@dirlisting</title>
+	    <meta http-equiv='Content-Type' content='text/html; charset=utf-8'/>
+	    <link href="/httpd/std.css" type='text/css' rel='stylesheet'/>
 	</head>
-	<body bgcolor="white" leftmargin="0" topmargin="0" marginwidth="0" marginheight="0">
+	<body>
 	<!-- HTML generated by tinyhttpd -->
-	<center>
-	<br>
-	<table width="600" border="0" cellspacing="1" cellpadding="4" bgcolor="#A1A5A9">
-	    <tr>
-	        <td bgcolor="#F2F2F2"><b>The Coccinella: directory listing of "%s"</b></td>
-	    </tr>
-	</table>
-	<br>
+	<h2>@contains</h2>
+
     }
 
     set html(updir,css) {
-	<table width="600" border="0" cellspacing="1" cellpadding="4" bgcolor="white">
+	<table class="center" width="600" border="0" cellspacing="1" cellpadding="4">
 	    <tr>
-		<td align=left nowrap bgcolor="white">
-		<a href="%s"><img border=0 src="%s">
-		    Parent directory "%s"
+		<td align="left">
+		<a href="%s"><img src="%s" alt=""/>
+		    @Parent (%s)
 		</a>
 		</td>
 	    </tr>
 	</table>
-	<br>
     }
 
     set html(tabletop,css) {
-	<table width="600" border="0" cellspacing="1" cellpadding="4" bgcolor="#A1A5A9">
-	    <tr valign="middle" bgcolor="#E2EEFF">
-	        <td align=center nowrap><b> %s objects </b></td>
-		<td align=left nowrap><b>Name</b></td>
-		<td align=right nowrap><b>Size</b></td>
-		<td align=left nowrap><b>Type</b></td>
-		<td align=left nowrap><b>Date</b></td>
+	<table class="center" width="600" border="0" cellspacing="1" cellpadding="4">
+	    <tr valign="middle">
+		<td align="left"><b>@Name</b></td>
+		<td align="right"><b>@Size</b></td>
+		<td align="left"><b>@Type</b></td>
+		<td align="left"><b>@Date</b></td>
 	    </tr>
     }
 
     set html(line,css) {
-	<tr bgcolor="%s">
-	    <td align=center nowrap>
-	        <a href="%s"><img border=0 width=16 height=16 src="%s"></a>
-	    </td>
-	    <td align=left nowrap><a href="%s"> %s </a></td>
-	    <td align=right nowrap> %s </td>
-	    <td align=left nowrap> %s </td>
-	    <td align=left nowrap> %s </td>
+	<tr class="%s">
+	    <td align="left"><a href="%s"><img width="16" height="16" src="%s" alt=""/> %s </a></td>
+	    <td align="right"> %s </td>
+	    <td align="left"> %s </td>
+	    <td align="left"> %s </td>
 	</tr>
     }
     
     # Pyjamas colors.
-    set html(bgcolor,css,0) #FFFFFF
-    set html(bgcolor,css,1) #F3F8FF
+    set html(class,css,0) white
+    set html(class,css,1) blue
     
+    # End of table and page.																																								 
     set html(dirbottom,css) {
     </table>
-    <br>
-    <table width="600" border="0" cellspacing="0" cellpadding="4" class="tableborder">
-        <tr>
-	    <td bgcolor="#F2F2F2" align="left" class="G10G"><b>The Coccinella</b></td>
-	    <td bgcolor="#F2F2F2" align="right"><img src="/httpd/images/download.gif" width="11" height="10" border="0" alt="" align="absmiddle"></a>
-	        <a href="http://hem.fyristorg.com/matben/download/">Download</a>
-	    </td>
-	</tr>
-    </table>
-    <br>
-    <table width="600" border="0" cellspacing="0" cellpadding="4">
-        <tr>
-	    <td class="G10G">This application is distributed under the General Public License (GPL). See the <a href="http://www.gnu.org/">Free Software Foundation</a> for further details.</td>
-        </tr>
-    </table>
-    <br> 
-    </center>
+    <div id='clearfooter'></div>
+    <div id='footerouter'>
+    <div id="footer">
+        <a href="http://coccinella.im/">@powered</a>
+    </div>
+    </div>
     </body>
-    </html>}
-
+    </html>
+    }
     
     
     # Default mapping from file suffix to Mime type.
@@ -304,7 +286,8 @@ Server: tinyhttpd/1.0"
       .html     text/html
       .htm      text/html
       .text     text/plain
-      .gif      image/gif
+      .css      text/css
+      .png      image/gif
       .jpeg     image/jpeg
       .jpg      image/jpeg
       .png      image/png
@@ -335,6 +318,13 @@ Server: tinyhttpd/1.0"
     }
 }
 
+proc ::tinyhttpd::InitOnce {} {
+    variable inited
+
+    McLoad
+    set inited 1
+}
+
 # tinyhttpd::start --
 #
 #       Start the http server.
@@ -361,7 +351,12 @@ proc ::tinyhttpd::start {args} {
     variable options
     variable priv
     variable this
-        
+    variable inited
+    
+    if {!$inited} {
+	InitOnce
+    }
+    
     # Any configuration options. Start with defaults, overwrite with args.
     array set options {
 	-cgibinrelativepath   httpd/cgibin
@@ -379,12 +374,12 @@ proc ::tinyhttpd::start {args} {
     }
     set options(-rootdirectory) [file dirname [info script]]
     array set options $args
-    if {[file pathtype $options(-rootdirectory)] != "absolute"} {
+    if {[file pathtype $options(-rootdirectory)] ne "absolute"} {
 	return -code error "the path must be an absolute path"
     }
 
     set servopts ""
-    if {$options(-myaddr) != ""} {
+    if {$options(-myaddr) ne ""} {
 	set servopts [list -myaddr $options(-myaddr)]
     }
     if {[catch {
@@ -424,16 +419,16 @@ proc ::tinyhttpd::start {args} {
     set priv(apath,mnt)   [file join $options(-rootdirectory) $priv(npath,mnt)]
     set priv(apath,cgi)   [file join $options(-rootdirectory) $priv(npath,cgi)]
 
-    set priv(rpath,imgs,folder) $priv(rpath,imgs)/folder.gif
-    set priv(rpath,imgs,file)   $priv(rpath,imgs)/file.gif
-    set priv(rpath,imgs,up)     $priv(rpath,imgs)/up.gif
+    set priv(rpath,imgs,folder) $priv(rpath,imgs)/folder.png
+    set priv(rpath,imgs,file)   $priv(rpath,imgs)/file.png
+    set priv(rpath,imgs,up)     $priv(rpath,imgs)/up.png
     
-    set priv(apath,imgs,folder) [file join $priv(apath,imgs) folder.gif]
-    set priv(apath,imgs,file)   [file join $priv(apath,imgs) file.gif]
-    set priv(apath,imgs,up)     [file join $priv(apath,imgs) up.gif]
+    set priv(apath,imgs,folder) [file join $priv(apath,imgs) folder.png]
+    set priv(apath,imgs,file)   [file join $priv(apath,imgs) file.png]
+    set priv(apath,imgs,up)     [file join $priv(apath,imgs) up.png]
     
     if {![file exists $priv(apath,imgs,folder)]} {
-	return -code error "Failed localizing folder.gif"
+	return -code error "Failed localizing folder.png"
     }
     foreach name {400 404} {
 	set f [file join $priv(apath,httpd) $name.html]
@@ -444,7 +439,28 @@ proc ::tinyhttpd::start {args} {
         
     LogMsg "Tiny Httpd started"
 
-    return ""
+    return
+}
+
+proc ::tinyhttpd::configure {args} {
+    variable options
+    
+    if {$args eq {}} {
+	set opts {}
+	foreach {name value} [array get options] {
+	    lappend opts $name $value
+	}
+	return $opts
+    } elseif {[llength $args] == 1} {
+	set name [lindex $args 0]
+	if {[info exists options($name)]} {
+	    return $options($name)
+	} else {
+	    return -code error "option \"$name\" does not exist"
+	}
+    } else {
+	return -code error "usage: ::tinyhttpd::configure ?-key?"
+    }
 }
 
 # tinyhttpd::NewChannel --
@@ -480,7 +496,8 @@ proc ::tinyhttpd::NewChannel {s ip port} {
     set state(port) $port
     
     # Everything should be done with 'fileevent'.
-    fconfigure $s -blocking 0 -buffering line
+    # Be sure to accept any eol and always write CRLF.
+    fconfigure $s -blocking 0 -buffering line -translation {auto crlf}
     fileevent $s readable [list [namespace current]::HandleRequest $token]
     
     return $token
@@ -505,7 +522,7 @@ proc ::tinyhttpd::HandleRequest {token} {
     
     set s $state(s)
     set state(state) reading
-
+    
     # If client closes socket.
     if {[catch {eof $s} iseof] || $iseof} {
 	Finish $token eof
@@ -543,10 +560,12 @@ proc ::tinyhttpd::HandleRequest {token} {
 
 	    # Make local absolute path.
 	    set relpath [string trimleft $path /]
-	    set relpath [uriencode::decodefile $relpath]
-	    set apath   [file join $options(-rootdirectory) $relpath]
-	    set state(rpath) $relpath
-	    set state(apath) [file nativename [file normalize $apath]]
+	    set rpathun [::uri::urn::unquote $relpath]
+	    set apath [file join $options(-rootdirectory) $rpathun]
+	    set npath [file nativename [file normalize $apath]]
+	    set state(rpath)   $relpath
+	    set state(rpathun) $rpathun
+	    set state(apath)   $npath
 	    set state(targetabspath) $state(apath)
 	    
 	    set state(chunk)     $options(-chunk)
@@ -604,12 +623,21 @@ proc ::tinyhttpd::Event {token} {
     } elseif {$nbytes > 0} {
 	
 	# Keep track of request meta data.
-	if {[regexp -nocase {^([^:]+):(.+)$} $line x key value]} {
+	if {[regexp -nocase {^([^:]+):(.+)$} $line - key value]} {
 	    lappend state(meta) $key [string trim $value]
 	}
-	if {[regexp -nocase {^content-type:(.+)$} $line x type]} {
+	if {[regexp -nocase {^content-type:(.+)$} $line - type]} {
 	    set state(content-type) [string trim $type]
-	} elseif {[regexp -nocase {^range:(.+)$} $line x range]} {
+	} elseif {[regexp -nocase {^accept-language:(.+)$} $line - langs]} {
+	    set langs [string trim $langs]
+	    set idx [string first "," $langs]
+	    if {$idx == -1} {
+		set lang $langs
+	    } else {
+		set lang [string range $langs 0 [expr {$idx-1}]]		
+	    }
+	    set state(lang) [lindex [split $lang "-"] 0]
+	} elseif {[regexp -nocase {^range:(.+)$} $line - range]} {
 	    set range [string trim $range]
 	    set rangelist [ParseRange $range]
 	    set state(range)       $range
@@ -629,9 +657,9 @@ proc ::tinyhttpd::ParseRange {range} {
     set rlist {}
     foreach r [split $range ,] {
 	if {[regexp {([0-9]*)-([0-9]*)} $r match n1 n2]} {
-	    if {$n2 == ""} {
+	    if {$n2 eq ""} {
 		set n2 end
-	    } elseif {$n1 == ""} {
+	    } elseif {$n1 eq ""} {
 		set n1 end-$n2
 		set n2 end
 	    }
@@ -660,18 +688,19 @@ proc ::tinyhttpd::Respond {token} {
     variable options
     variable priv
     variable mounts
+    variable fmounts
     variable cgibin
     
     Debug 2 "Respond:: $token"
             
-    set cmd     $state(cmd)
-    set relpath $state(rpath)    
-    set abspath $state(apath)
+    set cmd   $state(cmd)
+    set rpath $state(rpath)    
+    set apath $state(apath)
 
     set state(state) responding
 
-    Debug 2 "\t relpath=$relpath"
-    Debug 2 "\t abspath=$abspath"
+    Debug 2 "\t rpath=$rpath"
+    Debug 2 "\t apath=$apath"
     
     set httpcode 200
     set ismounted  0
@@ -682,7 +711,7 @@ proc ::tinyhttpd::Respond {token} {
 	    PutIfError $token 416
 	    return
 	}	
-    } elseif {[regexp ^$priv(rpath,cgi)/(.*)$ $relpath match subpath]} {	
+    } elseif {[regexp ^$priv(rpath,cgi)/(.*)$ $rpath m subpath]} {	
 
 	# See if cgi-bin. Handle over control.
 	if {[info exists cgibin($subpath)]} {
@@ -693,7 +722,7 @@ proc ::tinyhttpd::Respond {token} {
 	    Put404 $token
 	}
 	return
-    } elseif {[regexp ^$priv(rpath,mnt)/(.*)$ $relpath match subpath]} {		
+    } elseif {[regexp ^$priv(rpath,mnt)/(.*)$ $rpath m subpath]} {
 	
 	# See first if this is a "mounted" directory.
 	# Find name of mounted from subpath.
@@ -705,25 +734,29 @@ proc ::tinyhttpd::Respond {token} {
 	    set abspath [eval {file join $mounts($mname)} $rest]
 	    set abspath [file normalize $abspath]
 	    set state(targetabspath) $abspath
+	} elseif {[info exists fmounts($subpath)]} {
+	    set abspath $fmounts($subpath)	    
 	} else {
 	    Put404 $token
 	    return
 	}
 	Debug 2 "\t abspath=$abspath"
+    } else {
+        set abspath $apath
     }
     
-    if {[file isdirectory $abspath]} {
+    if {[file isdirectory $apath]} {
 
 	# If directory given then search for the '-defaultindexfile',
 	# or if no one, possibly return directory listing, else 404.
 
-	set indexfile [file join $abspath $options(-defaultindexfile)]
+	set indexfile [file join $apath $options(-defaultindexfile)]
 	if {[file exists $indexfile]} {
 	    set abspath $indexfile
 	} elseif {$options(-directorylisting)} {
 	    
 	    # No default html file exists, return directory listing.
-	    PutHtmlDirList $token $abspath
+	    PutHtmlDirList $token $apath
 	    return
 	} else {
 	    Put404 $token
@@ -754,6 +787,7 @@ proc ::tinyhttpd::PutResponse {token httpcode abspath} {
     variable http
     variable timing
     variable priv
+    variable useFcopy
     
     Debug 2 "::tinyhttpd::PutResponse httpcode=$httpcode, abspath=$abspath"
     
@@ -815,7 +849,7 @@ proc ::tinyhttpd::PutResponse {token httpcode abspath} {
 
     # Prepare the header.
     set modTime [clock format [file mtime $abspath]  \
-      -format "%a, %d %b %Y %H:%M:%S GMT" -gmt 1]
+      -format "%a, %d %b %Y %H:%M:%S GMT" -timezone :UTC]
     set msg $httpMsg($httpcode)
     set header [format $http(header) $httpcode $msg $modTime $mime \
       $state(contentlen)]
@@ -838,10 +872,7 @@ proc ::tinyhttpd::PutResponse {token httpcode abspath} {
 	fconfigure $fd -translation binary
 	fconfigure $s  -translation binary
 	flush $s
-	
-	# Seems necessary (?) to avoid blocking the UI. BAD!!!
-	#update
-	
+		
 	# Background copy. Be sure to switch off all fileevents on channel.
 	fileevent  $s readable {}
 	fileevent  $s writable {}
@@ -849,7 +880,11 @@ proc ::tinyhttpd::PutResponse {token httpcode abspath} {
 	set timing($token) [list [clock clicks -milliseconds] 0]
 	
 	# Initialize the stream copy.
-	CopyStart $s $token
+	if {$useFcopy} {
+	    CopyStart $s $token
+	} else {
+	    SetWritable $s $token
+	}
     }
 }
 
@@ -910,7 +945,6 @@ proc ::tinyhttpd::CopyStart {s token} {
     upvar 0 $token state    
     variable priv
     
-    Debug 6 "CopyStart::"
     if {$state(haverange) && [string is integer $state(range,end)]} {
 	set offset [tell $state(fd)]
 	if {[expr {$offset + $state(chunk)}] > $state(range,end)} {
@@ -941,7 +975,7 @@ proc ::tinyhttpd::CopyDone {token bytes {error {}}} {
     variable timing
     upvar 0 $token state
 
-    Debug 6 "CopyDone::"
+    #Debug 6 "CopyDone::"
 
     set s  $state(s)
     set fd $state(fd)
@@ -963,6 +997,66 @@ proc ::tinyhttpd::CopyDone {token bytes {error {}}} {
     }
 }
 
+# tinyhttpd::Write --
+# 
+#       Read a chunk from file and write to socket.
+#       This is instead of using 'fcopy'.
+#       Must only be called when socket is writable using fileevent.
+
+proc tinyhttpd::Write {s token} {
+    variable $token
+    upvar 0 $token state    
+        
+    fileevent $s writable {}
+
+    set fd $state(fd)
+
+    if {$state(haverange) && [string is integer $state(range,end)]} {
+	set offset [tell $state(fd)]
+	if {[expr {$offset + $state(chunk)}] > $state(range,end)} {
+	    set state(chunk) [expr {$state(range,end) - $offset + 1}]
+	}
+    }
+
+    if {[string length [fconfigure $s -error]]} {
+	Finish $token network-error
+    } elseif {[catch {eof $s} iseof] || $iseof} {
+	Finish $token eof
+    } elseif {[catch {eof $fd} iseof] || $iseof} {
+	Finish $token
+    } elseif {$state(haverange) && \
+      ([tell $state(fd)] >= [expr $state(range,end) + 1])} {
+	Finish $token	
+    } else {
+	if {[catch {
+	    set data [read $fd $state(chunk)]
+	    set n [string length $data]
+	    incr state(currentsize) $n
+	    if {$n >= 0} {
+		puts -nonewline $s $data
+	    }	    
+	} err]} {
+	    Finish $token network-error
+	} else {
+	    lappend timing($token) [clock clicks -milliseconds] $state(currentsize)
+	    
+	    # This is a trick to put this event at the back of the queue to
+	    # avoid using any 'update'.
+	    after idle [list after 0 [list \
+	      [namespace current]::SetWritable $s $token]]
+	}
+    }
+}
+
+proc tinyhttpd::SetWritable {s token} {    
+    
+    # We could have been closed since this event comes async.
+    if {[lsearch [file channels] $s] >= 0} {
+	fileevent $s writable  \
+	  [list [namespace current]::Write $s $token]
+    }
+}
+
 # tinyhttpd::Finish
 #
 #	Ends a request/response transaction.
@@ -1011,17 +1105,22 @@ proc ::tinyhttpd::PutHtmlDirList {token abspath} {
     set s   $state(s)
     set cmd $state(cmd)
 
+    # Do this always since line end translations may screw up the byte counts.
+    fconfigure $s -buffering full -translation binary
+
     # No default html file exists, return directory listing.
     set modTime [clock format [file mtime $abspath]  \
-      -format "%a, %d %b %Y %H:%M:%S GMT" -gmt 1]
+      -format "%a, %d %b %Y %H:%M:%S GMT" -timezone :UTC]
+    if {[string equal $cmd "GET"]} {
+	set html [BuildHtmlForDir $token]
+    }
     if {[catch {
 	puts $s [format $http(responseheader) $modTime]
 	
 	if {[string equal $cmd "GET"]} {
-	    set html [BuildHtmlForDir $token]
 	    puts $s "Content-Length: [string length $html]"
 	    puts $s ""
-	    puts $s $html
+	    puts -nonewline $s $html
 	} else {
 	    puts $s ""
 	}
@@ -1053,8 +1152,8 @@ proc ::tinyhttpd::BuildHtmlForDir {token} {
     Debug 2 "BuildHtmlForDir:"
     
     set style   $options(-style)
-    set relpath [string trimright $state(rpath) /]
-    set abspath $state(targetabspath)
+    set relpath [string trimright $state(rpathun) /]
+    set abspath $state(targetabspath)    
     
     Debug 4 "\t relpath=$relpath"
     Debug 4 "\t abspath=$abspath"
@@ -1066,23 +1165,33 @@ proc ::tinyhttpd::BuildHtmlForDir {token} {
     # Start by finding the directory content. 
     set allFiles [glob -directory $abspath -nocomplain *]
     set num [llength $allFiles]
-    if {$relpath != ""} {
+    if {$relpath ne ""} {
 	set rellist [file split $relpath]
 	set uplist  [lrange $rellist 0 end-1]
 	set upname  [lindex $uplist end]
-	if {$upname == ""} {
+	if {$upname eq ""} {
 	    set upname /
 	}
 	set uprpath ../
     }
+    if {[info exists state(lang)]} {
+	set lang $state(lang)
+    } else {
+	set lang "en"
+    }
     
     # Build the complete html page dynamically.    
     set dirname [string trim [lindex [file split $abspath] end] "\\/"]
-    set htmlStuff    [format $html(head,$style) $dirname]
-    if {$relpath != ""} {
-	append htmlStuff [format $html(updir,$style) $uprpath $imgup $upname]
+    set map [list @dirlisting [Mc $lang dirlisting]  \
+      @contains [Mc $lang contains $dirname $num]]
+    set htmlStuff [string map $map $html(head,$style)]
+    if {$relpath ne ""} {
+	set map [list @Parent [Mc $lang Parent]]
+	append htmlStuff [format [string map $map $html(updir,$style)] $uprpath $imgup $upname]
     }
-    append htmlStuff [format $html(tabletop,$style) $num]
+    set map [list @Name [Mc $lang Name] @Size [Mc $lang Size]   \
+      @Type [Mc $lang Type] @Date [Mc $lang Date]]
+    append htmlStuff [string map $map $html(tabletop,$style)]
     
     # Loop over all files and directories in our directory.
     set i 0
@@ -1097,7 +1206,7 @@ proc ::tinyhttpd::BuildHtmlForDir {token} {
 	set name [file tail $f]
 	set encodedname [uriencode::quotepath $name]
 	if {1} {
-	    if {$relpath == ""} {
+	    if {$relpath eq ""} {
 		set link "/$encodedname"
 	    } else {
 		set link "/$relpath/$encodedname"
@@ -1105,14 +1214,13 @@ proc ::tinyhttpd::BuildHtmlForDir {token} {
 	} else {
 	    set link $encodedname
 	}
-	#puts "\t link=$link"
 
 	# Is file or directory?
 	if {[file isdirectory $f]} {
 	    append link /
 	    set img $imgfolder
 	    set size -
-	    set type "Directory"
+	    set type [Mc $lang Directory]
 	} else {
 	    set img $imgfile
 	    if {[catch {file size $f} bytes]} {
@@ -1122,18 +1230,19 @@ proc ::tinyhttpd::BuildHtmlForDir {token} {
 	    set type $name
 	}
 	if {[string equal $style "css"]} {
-	    set color $html(bgcolor,css,[expr $i%2])
+	    set class $html(class,css,[expr $i%2])
 	    append htmlStuff [format $html(line,css) \
-	      $color $link $img $link $name $size $type $dateAndTime]
+	      $class $link $img $name $size $type $dateAndTime]
 	} else {
 	    append htmlStuff [format $html(line,plain) \
-	      $link $img $link $name $size $type $dateAndTime]
+	      $link $img $name $size $type $dateAndTime]
 	}
 	incr i
     }
     
     # And the end.
-    append htmlStuff $html(dirbottom,$style)
+    set map [list @powered [Mc $lang powered]]
+    append htmlStuff [string map $map $html(dirbottom,$style)]
     
     return $htmlStuff
 }
@@ -1208,7 +1317,7 @@ proc ::tinyhttpd::PutPlain404 {token} {
 #       
 # Results:
 
-proc ::tinyhttpd::stop { } {    
+proc ::tinyhttpd::stop {} {    
     variable priv
     variable options
     
@@ -1223,12 +1332,12 @@ proc ::tinyhttpd::stop { } {
 # 
 # 
 
-proc ::tinyhttpd::anyactive { } {
+proc ::tinyhttpd::anyactive {} {
 
     return [expr {[llength [getTokenList]] > 1} ? 1 : 0]
 }
 
-proc tinyhttpd::getTokenList { } {
+proc tinyhttpd::getTokenList {} {
     
     set ns [namespace current]
     return [concat  \
@@ -1243,7 +1352,7 @@ proc tinyhttpd::getTokenList { } {
 # 
 # 
 
-proc ::tinyhttpd::bytestransported { } {
+proc ::tinyhttpd::bytestransported {} {
     variable timing
     
     set totbytes 0
@@ -1257,7 +1366,7 @@ proc ::tinyhttpd::bytestransported { } {
 # 
 # 
 
-proc ::tinyhttpd::avergaebytespersec { } {
+proc ::tinyhttpd::avergaebytespersec {} {
     variable timing
     
     set totbytes 0
@@ -1267,14 +1376,14 @@ proc ::tinyhttpd::avergaebytespersec { } {
 	incr totms [expr [lindex $timing($key) end-1] - \
 	  [lindex $timing($key) 0]]
     }
-    return [expr 1000 * $totbytes / [expr $totms + 1]]
+    return [expr {1000 * $totbytes / ($totms+1)}]
 }
 
 # tinyhttpd::cleanup --
 # 
 # 
 
-proc ::tinyhttpd::cleanup { } {
+proc ::tinyhttpd::cleanup {} {
     variable priv
     variable options
     variable timing
@@ -1370,6 +1479,10 @@ proc ::tinyhttpd::unixpath {path} {
     return $upath
 }
 
+# tinyhttpd::mount --
+# 
+#       Mounts a directory on the httpd virtual file tree.
+
 proc ::tinyhttpd::mount {path name} {
     variable mounts
     
@@ -1384,14 +1497,38 @@ proc ::tinyhttpd::mount {path name} {
 
 proc ::tinyhttpd::unmount {name} {
     variable mounts
-    
     unset -nocomplain mounts($name)
 }
 
-proc ::tinyhttpd::allmounted { } {
+proc ::tinyhttpd::allmounted {} {
     variable mounts
+    return [array get mounts]
+}
+
+# tinyhttpd::mountfile --
+# 
+#       Mounts a file on the httpd virtual file tree.
+#       Returns relative mounted path.
+
+proc ::tinyhttpd::mountfile {fileName} {
+    variable fmounts
+    variable priv
     
-    return [array names mounts]
+    set tail [uriencode::quotepath [file tail $fileName]]
+    set name [file join [uuid] $tail]
+    set rpath [file join $priv(rpath,mnt) $name]
+    set fmounts($name) $fileName
+    return $rpath
+}
+
+proc ::tinyhttpd::unmountfile {name} {
+    variable fmounts
+    unset -nocomplain fmounts($name)
+}
+
+proc ::tinyhttpd::allfmounts {} {
+    variable fmounts
+    return [array get fmounts]
 }
 
 proc ::tinyhttpd::registercgicommand {path cmd} {
@@ -1412,10 +1549,53 @@ proc ::tinyhttpd::unregistercgicommand {path} {
     unset -nocomplain cgibin($name)
 }
 
-proc ::tinyhttpd::allcgibins { } {
+proc ::tinyhttpd::allcgibins {} {
     variable cgibin
     
-    return [array names cgibin]
+    return [array get cgibin]
+}
+
+# Simple message catalog support.
+
+proc ::tinyhttpd::Mc {lang src args} {
+    variable msg
+    
+    if {[info exists msg($lang,$src)]} {
+	return [eval {format $msg($lang,$src)} $args]
+    } else {
+	return $src
+    }
+}
+
+proc ::tinyhttpd::McSet {lang src {dst ""}} {
+    variable msg
+
+    set msg($lang,$src) $dst
+}
+
+proc ::tinyhttpd::McLoad {} {
+    variable msg
+    variable langfile
+
+    if {[file exists $langfile]} {
+	set fid [open $langfile "r"]
+	# @@@ Encoding???
+	fconfigure $fid -encoding utf-8
+	eval [read $fid]
+	close $fid
+    }
+}
+
+# tinyhttpd::uuid --
+# 
+#       Simplified uuid generator. See the uuid package for a better one.
+
+proc ::tinyhttpd::uuid {} {
+    set MAX_INT 0x7FFFFFFF
+    # Bugfix Eric Hassold from Evolane
+    set hex1 [format {%x} [expr {[clock clicks] & $MAX_INT}]]
+    set hex2 [format {%x} [expr {int($MAX_INT*rand())}]]
+    return $hex1-$hex2
 }
 
 proc ::tinyhttpd::Debug {num str} {
@@ -1435,6 +1615,27 @@ proc ::tinyhttpd::LogMsg {msg} {
 	    puts $priv(logfd) "[clock format [clock clicks]]: $msg"
 	}
     }
+}
+
+# Various test code.
+
+if {0} {
+    
+    ::tinyhttpd::registercgicommand  hello.html  Cgibin
+
+    proc Cgibin {token} {
+	variable $token
+	upvar 0 $token state
+
+	set sock $state(s)
+	::tinyhttpd::putheader $token 200 -headers {Content-Type text/html}
+	puts $sock {<HTML><BODY><H1>Hello World!</H1></BODY></HTML>}
+	close $sock
+    }
+
+    
+    
+    
 }
 
 #-------------------------------------------------------------------------------
