@@ -3,9 +3,22 @@
 #      This file is part of The Coccinella application. 
 #      It implements handling and parsing of emoticons (smileys).
 #      
-#  Copyright (c) 2004  Mats Bengtsson
+#  Copyright (c) 2004-2008  Mats Bengtsson
 #  
-# $Id: Emoticons.tcl,v 1.26 2004/12/20 15:16:45 matben Exp $
+#   This program is free software: you can redistribute it and/or modify
+#   it under the terms of the GNU General Public License as published by
+#   the Free Software Foundation, either version 3 of the License, or
+#   (at your option) any later version.
+#   
+#   This program is distributed in the hope that it will be useful,
+#   but WITHOUT ANY WARRANTY; without even the implied warranty of
+#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#   GNU General Public License for more details.
+#   
+#   You should have received a copy of the GNU General Public License
+#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#  
+# $Id: Emoticons.tcl,v 1.68 2008-06-09 09:50:59 matben Exp $
 
 package provide Emoticons 1.0
 
@@ -17,18 +30,21 @@ namespace eval ::Emoticons:: {
     ::hooks::register prefsSaveHook          ::Emoticons::SavePrefsHook
     ::hooks::register prefsCancelHook        ::Emoticons::CancelPrefsHook
     ::hooks::register prefsUserDefaultsHook  ::Emoticons::UserDefaultsHook
-
     ::hooks::register initHook               ::Emoticons::Init
 
+    ::hooks::register launchFinalHook        ::Emoticons::ParseCommandLine
+    if {[tk windowingsystem] eq "aqua"} {
+	::hooks::register macOpenDocument    ::Emoticons::MacOpenDocument
+    }
+    
     variable priv
     set priv(defaultSet) "default"
 }
 
-proc ::Emoticons::Init { } {
-    global  this
+proc ::Emoticons::Init {} {
+    global  this jprefs
     
     variable priv
-    upvar ::Jabber::jprefs jprefs
 
     # 'tmpicons(name,key)' map from a text string (key) and iconset name
     # to an image name.
@@ -42,20 +58,21 @@ proc ::Emoticons::Init { } {
     
     # We need the 'vfs::zip' package and if not using starkit we also need
     # the 'Memchan' package which is not automatically checked for.
+    # 'rechan' is the tclkits built in version of 'Memchan'.
     if {[catch {package require vfs::zip}]} {
 	set priv(havezip) 0
-    } elseif {[info exists starkit::topdir]} {
+    } elseif {![catch {package require rechan}]} {
+	set priv(havezip) 1
+    } elseif {![catch {package require Memchan}]} {
 	set priv(havezip) 1
-    } elseif {[catch {package require Memchan}]} {
-	set priv(havezip) 0
     } else {
-	set priv(havezip) 1
+	set priv(havezip) 0
     }
 
     # Cache stuff we need later.
-    set priv(havepng)      [::Plugins::HaveImporterForMime image/png]
-    set priv(QuickTimeTcl) [::Plugins::HavePackage QuickTimeTcl]
-    set priv(Img)          [::Plugins::HavePackage Img]
+    set priv(havepng) 1
+    set priv(QuickTimeTcl) [expr ![catch {package require QuickTimeTcl}]]
+    set priv(Img)          [expr ![catch {package require Img}]]
     if {$priv(Img)} {
 	set priv(needtmp)   0
 	set priv(pngformat) [list -format png]
@@ -63,48 +80,85 @@ proc ::Emoticons::Init { } {
 	set priv(needtmp)   1
 	set priv(pngformat) {}
     }
-    ::Debug 4 "sets=[::Emoticons::GetAllSets]"
-    ::Debug 4 "\t [::Emoticons::GetPrefSetPathExists]"
+
+    # This also sets the 'state' array.
+    GetAllSets
     
     # Load set.
     # Even if we succeed to get the vfs::zip package, it doesn't check
     # the Memchan package, and can therefore still fail.
-    if {[catch {
-	::Emoticons::LoadTmpIconSet [::Emoticons::GetPrefSetPathExists]
-    } err]} {
-	::Debug 4 "\t catch: $err"
-	set priv(havezip) 0
-	set jprefs(emoticonSet) $priv(defaultSet)
-	::Emoticons::LoadTmpIconSet [::Emoticons::GetPrefSetPathExists]
+    # NB: We may have "None" set which has a value "-".
+
+    if {$jprefs(emoticonSet) ne "-"} {
+	if {[catch {
+	    LoadTmpIconSet [GetPrefSetPathExists]
+	} err]} {
+	    ::Debug 4 "\t catch: $err"
+	    set priv(havezip) 0
+	    set jprefs(emoticonSet) $priv(defaultSet)
+	    LoadTmpIconSet [GetPrefSetPathExists]
+	}
+	SetPermanentSet $jprefs(emoticonSet)
+    } else {
+	# empty
+    }
+    if {[tk windowingsystem] eq "win32"} {
+	InitWin
+    }
+}
+
+proc ::Emoticons::InitWin {} {
+    global  this
+    
+    if {[catch {package require RegisterFileType}]} {
+	return
+    }
+    # Find the exe we are running. Starkits?
+    if {[info exists ::starkit::topdir]} {
+	set exe [file nativename [info nameofexecutable]]
+	set cmd "\"$exe\" -file \"%1\""
+    } else {
+	set exe [file nativename [info nameofexecutable]]
+	set app [file nativename $this(script)]
+	set cmd "\"$exe\" \"$app\" -file \"%1\""
+    }
+    catch {
+	RegisterFileType::RegisterFileType .jisp jispArchive \
+	  "Emoticon Archive"  $cmd
     }
-    ::Emoticons::SetPermanentSet $jprefs(emoticonSet)
 }
 
 proc ::Emoticons::Exists {word} {
     variable smiley
-    
-    if {[info exists smiley($word)]} {
-	return 1
-    } else {
-	return 0
-    }
+    return [info exists smiley($word)]
+}
+
+proc ::Emoticons::Get {word} {
+    variable smiley
+    return $smiley($word)
 }
 
 proc ::Emoticons::Make {w word} {
     variable smiley
     
     if {[info exists smiley($word)]} {
-	$w image create end -image $smiley($word) -name $word
+	$w image create insert -image $smiley($word) -name $word
     }
 }
 
-proc ::Emoticons::GetAllSets { } {
-    global  this
+proc ::Emoticons::None {} {
+    global jprefs
+    return [expr {$jprefs(emoticonSet) eq "-"}]
+}
+
+proc ::Emoticons::GetAllSets {} {
+    global this
     variable priv
     variable state
     
-    set setList {}
-    foreach path [list $this(emoticonsPath) $this(altEmoticonsPath)] {
+    set setList [list]
+    set paths [::Theme::GetPathsFor $this(emoticons)]
+    foreach path $paths {
 	foreach f [glob -nocomplain -directory $path *] {
 	    set name [file tail $f]
 	    set name [file rootname $name]
@@ -127,24 +181,16 @@ proc ::Emoticons::GetAllSets { } {
 #       Gets the full path to our emoticons file/folder.
 #       Verifies that it exists and that can be mounted if zip archive.
 
-proc ::Emoticons::GetPrefSetPathExists { } {
-    global  this
+proc ::Emoticons::GetPrefSetPathExists {} {
+    global  this jprefs
+    variable state
     variable priv
-    upvar ::Jabber::jprefs jprefs
-
-    # Start with the failsafe set.
-    set path [file join $this(emoticonsPath) $priv(defaultSet)]
     
-    foreach dir [list $this(emoticonsPath) $this(altEmoticonsPath)] {
-	set f [file join $dir $jprefs(emoticonSet)]
-	set fjisp ${f}.jisp
-	if {[file exists $f]} {
-	    set path $f
-	    break
-	} elseif {[file exists $fjisp] && $priv(havezip)} {
-	    set path $fjisp
-	    break
-	}
+    set name $jprefs(emoticonSet)
+    if {[info exists state($name,path)]} {
+	set path $state($name,path)
+    } else {
+	set path [file join $this(emoticonsPath) $priv(defaultSet)]
     }
     set jprefs(emoticonSet) [file rootname [file tail $path]]
     return $path
@@ -163,6 +209,7 @@ proc ::Emoticons::LoadTmpIconSet {path} {
 	if {$priv(havezip)} {
 	    set mountpath [file join [file dirname $path] $name]
 	    if {[catch {
+		#puts "path=$path, mountpath=$mountpath"
 		set fdzip [vfs::zip::Mount $path $mountpath]
 	    } err]} {
 		return -code error $err
@@ -173,7 +220,7 @@ proc ::Emoticons::LoadTmpIconSet {path} {
 	    set zipdir [lindex [glob -nocomplain -directory $mountpath *] 0]
 	    set dir $zipdir
 	} else {
-	    return -code error "cannot read jisp archive without vfs::zip"
+	    return -code error "Cannot read jisp archive without vfs::zip"
 	}
     }
     set icondefPath [file join $dir icondef.xml]
@@ -181,6 +228,7 @@ proc ::Emoticons::LoadTmpIconSet {path} {
 	return -code error "missing icondef.xml file in archive"
     }
     set fd [open $icondefPath]
+    fconfigure $fd -encoding utf-8
     set xmldata [read $fd]
     close $fd
     
@@ -196,10 +244,16 @@ proc ::Emoticons::LoadTmpIconSet {path} {
 }
 
 proc ::Emoticons::ParseIconDef {name dir xmldata} {
+    variable tmpicons
+    variable tmpiconsInv
 
     set token [tinydom::parse $xmldata]
     set xmllist [tinydom::documentElement $token]
-    
+
+    # @@@ Any images shall be freed!!!
+    array unset tmpicons $name,*
+    array unset tmpiconsInv $name,*
+
     foreach elem [tinydom::children $xmllist] {
 	
 	switch -- [tinydom::tagname $elem] {
@@ -242,7 +296,7 @@ proc ::Emoticons::ParseIcon {name dir xmllist} {
 		lappend keyList [tinydom::chdata $elem]
 	    }
 	    object - graphic {
-		# graphic does not comply with JEP-0038!
+		# graphic does not comply with XEP-0038!
 		set object [tinydom::chdata $elem]
 		array set attrArr [tinydom::attrlist $elem]
 		set mime $attrArr(mime)
@@ -252,8 +306,7 @@ proc ::Emoticons::ParseIcon {name dir xmllist} {
     
     switch -- $mime {
 	image/gif {
-	    set im [image create photo -format gif  \
-	      -file [file join $dir $object]]
+	    set im [::Utils::CreateGif [file join $dir $object]]
 	    foreach key $keyList {
 		set tmpicons($name,$key) $im
 	    }
@@ -262,8 +315,8 @@ proc ::Emoticons::ParseIcon {name dir xmllist} {
 	    # If we rely on QuickTimeTcl here we cannot be in vfs.
 	    set f [file join $dir $object]
 	    if {$priv(needtmp)} {
-		#set tmp [file join $this(tmpPath) $object]
 		set tmp [::tfileutils::tempfile $this(tmpPath) [file rootname $object]]
+		append tmp [file extension $object]
 		file copy -force $f $tmp
 		set f $tmp
 	    }
@@ -286,14 +339,12 @@ proc ::Emoticons::ParseIcon {name dir xmllist} {
 proc ::Emoticons::SetPermanentSet {selected} {
     variable state
     variable smiley
-
-    #puts "::Emoticons::SetPermanentSet selected=$selected"
-    set setsLoaded {}
+    
+    set setsLoaded [list]
     foreach ind [array names state *,loaded] {
 	set name [string map [list ",loaded" ""] $ind]
 	lappend setsLoaded $name
     }
-    #puts "\t setsLoaded=$setsLoaded"
     if {[info exists smiley]} {
 	FreeSmileyArr
     }
@@ -305,6 +356,8 @@ proc ::Emoticons::SetPermanentSet {selected} {
 	}
     }
     unset -nocomplain state
+    
+    MenuButtonsUpdate
 }
 
 proc ::Emoticons::SetSmileyArr {name} {
@@ -313,7 +366,6 @@ proc ::Emoticons::SetSmileyArr {name} {
     variable smiley
     variable smileyInv
     
-    #puts "::Emoticons::SetSmileyArr name=$name"
     foreach ind [array names tmpiconsInv $name,*] {
 	set im [string map [list "$name," ""] $ind]
 	foreach key $tmpiconsInv($ind) {
@@ -323,14 +375,12 @@ proc ::Emoticons::SetSmileyArr {name} {
     }
 }
 
-proc ::Emoticons::FreeSmileyArr { } {
+proc ::Emoticons::FreeSmileyArr {} {
     variable smiley
     variable smileyInv
     
-    #puts "::Emoticons::FreeSmileyArr"
-    
     # This could create empty images for any open dialogs. BAD?
-    # MEMLEAK!
+    # @@@ MEMLEAK!
     #eval {image delete} [array names smileyInv]
     unset -nocomplain smiley smileyInv
 }
@@ -341,13 +391,12 @@ proc ::Emoticons::FreeTmpSet {name} {
     variable tmpicons
     variable tmpiconsInv
     
-    #puts "::Emoticons::FreeTmpSet name=$name"
-    set ims {}
+    set ims [list]
     foreach ind [array names tmpiconsInv $name,*] {
 	lappend ims [string map [list "$name," ""] $ind]
     }
     # This could create empty images for any open dialogs. BAD?
-    # MEMLEAK!
+    # @@@ MEMLEAK!
     #eval {image delete} $ims
     array unset meta $name,*
     array unset tmpicons $name,*
@@ -355,13 +404,13 @@ proc ::Emoticons::FreeTmpSet {name} {
     array unset state $name,*
 }
 
-proc ::Emoticons::FreeAllTmpSets { } {
+proc ::Emoticons::FreeAllTmpSets {} {
     variable meta
     variable state
     variable tmpicons
     variable tmpiconsInv
 
-    set ims {}
+    set ims [list]
     foreach {key photo} [array get tmpicons ] {
 	lappend ims $photo
     }
@@ -369,6 +418,13 @@ proc ::Emoticons::FreeAllTmpSets { } {
     unset -nocomplain meta state tmpicons tmpiconsInv
 }
 
+namespace eval ::Emoticons {
+    
+    # Keep track of all menu buttons to be able to do "live updates" when
+    # the emoticon set changes.
+    variable allButtons [list]
+}
+
 # Emoticons::MenuButton --
 # 
 #       A kind of general menubutton for inserting smileys into a text widget.
@@ -383,48 +439,70 @@ proc ::Emoticons::FreeAllTmpSets { } {
 
 proc ::Emoticons::MenuButton {w args} {
     global  prefs this    
-    variable smiley
+    variable allButtons
 
+    variable $w
+    upvar 0 $w state    
+
+    set state(w)    $w
+    set state(args) $args
+    
     # If we have -compound left -image ... -label ... working.
     set prefs(haveMenuImage) 0
-    if {([package vcompare [info tclversion] 8.4] >= 0) &&  \
-      ![string equal $this(platform) "macosx"]} {
+    if {![string equal $this(platform) "macosx"]} {
 	set prefs(haveMenuImage) 1
+    }    
+    ttk::menubutton $w -style MiniMenubutton -compound image
+    eval {MenuButtonConfigure $w} $args
+    if {[None]} {
+	$w state {disabled}
     }
-    if {[string match "mac*" $this(platform)]} {
-	set btbd 2
-    } else {
-	set btbd 1
-    }
+    lappend allButtons $w
+    bind $w <Destroy> +[namespace code [list MenuButtonFree %W]]
+    
+    return $w
+}
 
-    # Workaround for missing -image option on my macmenubutton.
-    if {[string equal $this(platform) "macintosh"] && \
-      [string length [info command menubuttonOrig]]} {
-	set menubuttonImage menubuttonOrig
-    } else {
-	set menubuttonImage button
-    }
+proc ::Emoticons::MenuButtonConfigure {w args} {
+    variable smiley
     
     # Button image.
     set btim ""
-    set size 16
     foreach key {:) :-) ;) ;-) :( :-(} {
 	if {[info exists smiley($key)]} {
 	    set btim $smiley($key)
-	    set size [image width $btim]
-	    if {$size < 16} {
-		set size 16
-	    }
 	    break
 	}
     }
-    set wmenu ${w}.m
-    $menubuttonImage $w -image $btim -bd $btbd -width $size -height $size
-    
-    eval {::Emoticons::BuildMenu $wmenu} $args
+    $w configure -image $btim
+    set wmenu $w.m
+    destroy $wmenu
+    eval {BuildMenu $wmenu} $args
+    $w configure -menu $wmenu
+    if {[None]} {
+	$w state {disabled}
+    } else {
+	$w state {!disabled}
+    }
+}
 
-    bind $w <Button-1> [list [namespace current]::PostMenu $w $wmenu %X %Y]
-    return $w
+proc ::Emoticons::MenuButtonFree {w} {
+    variable $w
+    variable allButtons
+    
+    lprune allButtons $w
+    unset -nocomplain $w
+}
+
+proc ::Emoticons::MenuButtonsUpdate {} {
+    variable allButtons
+
+    foreach w $allButtons {
+	variable $w
+	upvar 0 $w state    
+
+	eval {MenuButtonConfigure $w} $state(args)
+    }
 }
 
 proc ::Emoticons::BuildMenu {wmenu args} {
@@ -490,7 +568,7 @@ proc ::Emoticons::SortLength {str1 str2} {
 
 proc ::Emoticons::PostMenu {w m x y} {
 
-    if {[string equal [$w cget -state] "normal"]} {
+    if {![string equal [$w cget -state] "disabled"]} {
 	tk_popup $m [expr int($x)] [expr int($y)]
     }
 }
@@ -515,9 +593,17 @@ proc ::Emoticons::InsertTextLegend {w name args} {
     $w delete 1.0 end
     
     # Meta data:
+    set metadata [dict create]
+    # TRANSLATORS; see Emoticons in preferences
+    dict set metadata author      [mc "Author"]
+    dict set metadata description [mc "Description"]
+    dict set metadata home        [mc "Home"]
+    dict set metadata name        [mc "Name"]
+    dict set metadata version     [mc "Version"]
+    dict set metadata creation    [mc "Date"]
     foreach ind [array names meta $name,*] {
 	set key [string map [list "$name," ""] $ind]
-	$w insert insert "[string totitle $key]:\t" tmeta
+	$w insert insert "[dict get $metadata $key]:\t" tmeta
 	foreach val $meta($ind) {
 	    $w insert insert "$val, " tmeta
 	}
@@ -525,7 +611,12 @@ proc ::Emoticons::InsertTextLegend {w name args} {
 	$w insert insert "\n"
     }
     
-    $w insert insert "\t[mc Image]\t[mc Text]\n"
+    set msg "\t"
+    append msg [mc "Image"]
+    append msg "\t"
+    append msg [mc "Text"]
+    append msg "\n"
+    $w insert insert $msg
     
     # Smileys:
     foreach ind [lsort -dictionary [array names tmpiconsInv $name,*]] {
@@ -542,130 +633,322 @@ proc ::Emoticons::InsertTextLegend {w name args} {
     $w configure -state disabled
 }
 
+proc ::Emoticons::ImportSet {} {
+    global  this
+    
+    set types [list [list [mc "Iconset Archive"] {.jisp}]]
+    set fileName [tk_getOpenFile -filetypes $types \
+      -title [mc "Import Iconset"]]
+    if {[file exists $fileName]} {
+	ImportFile $fileName
+    }    
+    return $fileName
+}
+
+proc ::Emoticons::ImportFile {fileName} {
+    global  this
+    
+    set tail [file tail $fileName]
+    set name [file rootname $tail]
+    if {[lsearch [GetAllSets] $name] >= 0} {
+	::UI::MessageBox -icon error -title [mc "Error"] \
+	  -message "Iconset \"$name\" already exists."
+	return
+    }
+    file copy $fileName $this(altEmoticonsPath)
+    set dst [file join $this(altEmoticonsPath) $tail]
+    if {[catch {
+	LoadTmpIconSet $dst
+    } err]} {
+	::UI::MessageBox -icon error -title [mc "Error"] \
+	  -message "Failed loading iconset \"$name\". $err"
+	return
+    }
+}
+
+# Emoticons::ParseCommandLine --
+# 
+#       A launchFinalHook that detects any jisp archive files on the command
+#       line.
+
+proc ::Emoticons::ParseCommandLine {args} {
+    global  argv
+   
+    if {$args eq {}} {
+	set args $argv
+    }
+    set idx [lsearch $args -file]
+    if {$idx < 0} {
+	return
+    }
+    set fileName [lindex $args [incr idx]]
+    if {[file extension $fileName] ne ".jisp"} {
+	return
+    }
+    ImportFile $fileName
+}
+
+proc ::Emoticons::MacOpenDocument {args} {
+    
+    foreach fileName $args {
+	if {[file extension $fileName] eq ".jisp"} {
+	    ImportFile $fileName
+	}
+    }
+}
+
 # Preference page --------------------------------------------------------------
 
-proc  ::Emoticons::InitPrefsHook { } {
+proc  ::Emoticons::InitPrefsHook {} {
+    global jprefs
     variable priv
-    upvar ::Jabber::jprefs jprefs
 
     set jprefs(emoticonSet) $priv(defaultSet)
     
     # Do NOT store the complete path!
-    ::PreferencesUtils::Add [list  \
-      [list ::Jabber::jprefs(emoticonSet) jprefs_emoticonSet $jprefs(emoticonSet)]]
+    ::PrefUtils::Add [list  \
+      [list jprefs(emoticonSet) jprefs_emoticonSet $jprefs(emoticonSet)]]
 }
 
 proc ::Emoticons::BuildPrefsHook {wtree nbframe} {
 
-    $wtree newitem {Jabber Emoticons} -text [mc {Emoticons}]
+    ::Preferences::NewTableItem {Jabber Emoticons} [mc "Emoticons"]
     
     set wpage [$nbframe page {Emoticons}]    
-    ::Emoticons::BuildPrefsPage $wpage
+    BuildPrefsPage $wpage
 }
 
 proc ::Emoticons::BuildPrefsPage {wpage} {
-    variable wpreftext 
+    global jprefs
+    variable wprefpage $wpage
+    variable wpreftext
+    variable wprefmb
     variable tmpSet
     variable priv
-    upvar ::Jabber::jprefs jprefs
     
-    set fontS  [option get . fontSmall {}]    
-    set fontSB [option get . fontSmallBold {}]    
+    set allSets [GetAllSets]
 
-    set wpop $wpage.pop
-    set wfr $wpage.fr
-    set wpreftext $wfr.t
-    set wysc $wfr.ysc
-    set allSets [::Emoticons::GetAllSets]
-    
     # This should never happen!
-    if {[llength $allSets] == 0} {
+    if {$allSets eq {}} {
 	set allSets None
     }
-    label $wpage.l -text [mc preficonsel]
-    pack  $wpage.l -side top -anchor w -padx 8 -pady 4
-    eval {tk_optionMenu $wpop [namespace current]::tmpSet} $allSets
-    labelframe $wfr -labelwidget $wpop -padx 6 -pady 4
-    pack $wfr -side top -anchor w -padx 8 -pady 4 -fill both -expand 1
+
+    set wc $wpage.c
+    ttk::frame $wc -padding [option get . notebookPageSmallPadding {}]
+    pack $wc -side top -fill both -expand 1 \
+      -anchor [option get . dialogAnchor {}]
+        
+    ttk::label $wc.l -text [mc "The selected iconset will become the default."]
+    pack  $wc.l  -side top -anchor w -pady 4
     
-    scrollbar $wysc -orient vertical -command [list $wpreftext yview]
-    text $wpreftext -yscrollcommand [list $wysc set] -width 20 -height 10
-    grid $wpreftext -row 0 -column 0 -sticky news
-    grid $wysc -row 0 -column 1 -sticky ns
+    set wmb $wc.mb    
+    set wprefmb $wmb
+    set menuDef [list]
+    foreach name $allSets {
+	if {$name eq "default"} {
+	    lappend menuDef [list [mc "Default"] -value default]
+	} else {
+	    lappend menuDef [list $name -value $name]
+	}
+    }
+    lappend menuDef separator
+    lappend menuDef [list [mc "None"] -value "-"]
+    ui::optionmenu $wmb -menulist $menuDef -variable [namespace current]::tmpSet \
+      -command [namespace code PrefsSetCmd]
+    pack $wmb -side top -anchor w
+    
+    set wfr $wc.fr
+    ttk::frame $wfr -padding [option get . groupSmallPadding {}]
+    pack $wfr -side top -anchor w -fill both -expand 1
+    
+    ttk::scrollbar $wfr.ysc -orient vertical -command [list $wfr.t yview]
+    text $wfr.t -yscrollcommand [list $wfr.ysc set] -width 20 -height 14 \
+      -highlightthickness 0 -bd 0
+    
+    grid  $wfr.t   $wfr.ysc  -sticky ns
+    grid  $wfr.t   -sticky news
     grid columnconfigure $wfr 0 -weight 1
     grid rowconfigure    $wfr 0 -weight 1
-
-    trace add variable [namespace current]::tmpSet write  \
-      [namespace current]::PopCmd
-
-    if {[lsearch $allSets $jprefs(emoticonSet)] < 0} {
+    
+    set wpreftext $wfr.t
+    
+    ttk::button $wc.imp -text [mc "Import Iconset"]... \
+      -command [namespace current]::ImportSetToPrefs
+    pack $wc.imp -side top -anchor w -pady 4
+    
+    if {$jprefs(emoticonSet) eq "-"} {
+	set tmpSet "-"
+    } elseif {[lsearch $allSets $jprefs(emoticonSet)] < 0} {
 	set tmpSet $priv(defaultSet)
     } else {
 	set tmpSet $jprefs(emoticonSet)
     }
+    PrefsSetCmd $tmpSet
+    
+    if {[tk windowingsystem] ne "aqua"} {
+	if {![catch {package require tkdnd}]} {
+	    DnDInit $wpreftext
+	}
+    }
 }
 
-proc ::Emoticons::PopCmd {name1 name2 op} {
+proc ::Emoticons::PrefsSetCmd {value} {
+    global jprefs
+    variable wprefpage
     variable wpreftext 
     variable state
     variable tmpSet
     variable priv
-    upvar ::Jabber::jprefs jprefs
-    upvar $name1 var
+    
+    if {$tmpSet eq "-"} {
+	$wpreftext configure -state normal
+	$wpreftext delete 1.0 end
+	$wpreftext configure -state disabled
+    } else {
+	if {![info exists state($tmpSet,loaded)]} {
+	    if {[catch {
+		LoadTmpIconSet $state($tmpSet,path)
+	    } err]} {
+		puts "catch LoadTmpIconSet err=$err"
+		set str [mc "Cannot load iconset %s." $tmpSet]
+		append str "\n"
+		append str [mc "Error"]
+		append str ": $err"
+		::UI::MessageBox -icon error -title [mc "Error"] \
+		  -message $str -parent [winfo toplevel $wprefpage]
+		set priv(havezip) 0
+		set jprefs(emoticonSet) $priv(defaultSet)
+		set tmpSet $priv(defaultSet)
+		LoadTmpIconSet [GetPrefSetPathExists]
+		return
+	    }
+	}
+	InsertTextLegend $wpreftext $tmpSet
+    }
+}
 
-    if {![info exists state($var,loaded)]} {
-	if {[catch {
-	    ::Emoticons::LoadTmpIconSet $state($var,path)
-	} err]} {
-	    ::UI::MessageBox -icon error -title [mc Error] \
-	      -message "Failed loading iconset $var. $err" \
-	      -parent [winfo toplevel $wpreftext]
-	    set priv(havezip) 0
-	    set jprefs(emoticonSet) $priv(defaultSet)
-	    set tmpSet $priv(defaultSet)
-	    ::Emoticons::LoadTmpIconSet [::Emoticons::GetPrefSetPathExists]
-	    return
+proc ::Emoticons::ImportSetToPrefs {} {
+    global  this
+    variable wprefpage
+    variable wprefmb
+    
+    set types [list [list [mc "Iconset Archive"] {.jisp}]]
+    set fileName [tk_getOpenFile -parent [winfo toplevel $wprefpage] \
+      -filetypes $types -title [mc "Import Iconset"]]
+    if {[file exists $fileName]} {
+	ImportFileToPrefs $fileName
+    }
+}
+
+proc ::Emoticons::ImportFileToPrefs {fileName} {
+    global  this
+    variable wprefpage
+    variable wprefmb
+    
+    set tail [file tail $fileName]
+    set name [file rootname $tail]
+    if {[lsearch [GetAllSets] $name] >= 0} {
+	::UI::MessageBox -icon error -title [mc "Error"] \
+	  -message [mc "Iconset %s already exists." $name] \
+	  -parent [winfo toplevel $wprefpage]
+	return
+    }
+    file copy $fileName $this(altEmoticonsPath)
+    set dst [file join $this(altEmoticonsPath) $tail]
+    if {[catch {
+	LoadTmpIconSet $dst
+    } err]} {
+	set str [mc "Cannot load iconset %s." $name]
+	append str "\n"
+	append str [mc "Error"]
+	append str ": $err"
+	::UI::MessageBox -icon error -title [mc "Error"] \
+	  -message $str -parent [winfo toplevel $wprefpage]
+    } else {
+	if {[winfo exists $wprefmb]} {
+	    set mDef [$wprefmb cget -menulist]
+	    set idx [lsearch $mDef separator]
+	    set mDef [linsert $mDef $idx [list $name -value $name]]
+	    $wprefmb configure -menulist $mDef
 	}
     }
-    ::Emoticons::InsertTextLegend $wpreftext $var
 }
 
-proc ::Emoticons::FreePrefsPage { } {
+proc ::Emoticons::DnDInit {win} {
     
-    trace remove variable [namespace current]::tmpSet write  \
-      [namespace current]::PopCmd
+    dnd bindtarget $win text/uri-list <Drop> \
+      [namespace code [list DnDDrop %W %D %T]]
+    dnd bindtarget $win text/uri-list <DragEnter> \
+      [namespace code [list DnDEnter %W %A %D %T]]
+    dnd bindtarget $win text/uri-list <DragLeave> \
+      [namespace code [list DnDLeave %W %D %T]]
 }
 
-proc ::Emoticons::SavePrefsHook { } {
-    variable tmpSet
-    upvar ::Jabber::jprefs jprefs
-
-    if {![string equal $jprefs(emoticonSet) $tmpSet]} {
-	::Emoticons::SetPermanentSet $tmpSet
-
+proc ::Emoticons::DnDDrop {win data dndtype} {
+    
+    # Take only first file.
+    set f [lindex $data 0]
+    if {[file extension $f] eq ".jisp"} {
+	
+	# Strip off any file:// prefix.
+	set f [string map {file:// ""} $f]
+	set f [::uri::urn::unquote $f]
+	ImportFileToPrefs $f
     }
-    ::Emoticons::FreePrefsPage
-    set jprefs(emoticonSet) $tmpSet
 }
 
-proc ::Emoticons::CancelPrefsHook { } {
+proc ::Emoticons::DnDEnter {win action data dndtype} {
+    set f [lindex $data 0]
+    if {[file extension $f] eq ".jisp"} {
+	focus $win
+	set act "default"
+    } else {
+	set act "none"
+    }
+    return $act
+}
+
+proc ::Emoticons::DnDLeave {win data dndtype} {	
+    focus [winfo toplevel $win] 
+}
+
+proc ::Emoticons::FreePrefsPage {} {
+    
+}
+
+proc ::Emoticons::SavePrefsHook {} {
+    global jprefs
+    variable tmpSet
+    
+    set new 0
+    if {![string equal $jprefs(emoticonSet) $tmpSet]} {
+	set new 1
+    }
+    FreePrefsPage
+    set jprefs(emoticonSet) $tmpSet
+    if {$new} {
+	SetPermanentSet $tmpSet
+    }
+}
+
+proc ::Emoticons::CancelPrefsHook {} {
+    global jprefs
     variable tmpSet
-    upvar ::Jabber::jprefs jprefs
 
     # Since the menubutton is used both for viewing and setting,
     # I think we skip this warning.
     if {![string equal $jprefs(emoticonSet) $tmpSet]} {
 	#::Preferences::HasChanged
     }
-    ::Emoticons::FreePrefsPage
-    ::Emoticons::FreeAllTmpSets
+    FreePrefsPage
+    FreeAllTmpSets
 }
 
-proc ::Emoticons::UserDefaultsHook { } {
+proc ::Emoticons::UserDefaultsHook {} {
+    global jprefs
     variable tmpSet
-    upvar ::Jabber::jprefs jprefs
     
-    set allSets [::Emoticons::GetAllSets]
+    set allSets [GetAllSets]
     if {[lsearch $allSets $jprefs(emoticonSet)] < 0} {
 	set tmpSet $priv(defaultSet)
     } else {
