@@ -3,9 +3,22 @@
 #      This file is part of The Coccinella application. 
 #      It implements a dialog for jabber messages.
 #      
-#  Copyright (c) 2002  Mats Bengtsson
+#  Copyright (c) 2002-2008  Mats Bengtsson
 #  
-# $Id: GotMsg.tcl,v 1.39 2004/12/21 15:14:42 matben Exp $
+#   This program is free software: you can redistribute it and/or modify
+#   it under the terms of the GNU General Public License as published by
+#   the Free Software Foundation, either version 3 of the License, or
+#   (at your option) any later version.
+#   
+#   This program is distributed in the hope that it will be useful,
+#   but WITHOUT ANY WARRANTY; without even the implied warranty of
+#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#   GNU General Public License for more details.
+#   
+#   You should have received a copy of the GNU General Public License
+#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#  
+# $Id: GotMsg.tcl,v 1.59 2008-06-09 09:50:59 matben Exp $
 
 package provide GotMsg 1.0
 
@@ -13,8 +26,8 @@ namespace eval ::GotMsg:: {
 
     # Add all event hooks.
     ::hooks::register quitAppHook        ::GotMsg::QuitAppHook
-    ::hooks::register closeWindowHook    ::GotMsg::CloseHook
     ::hooks::register presenceHook       ::GotMsg::PresenceHook    
+    #::hooks::register newMessageHook     ::GotMsg::MessageHook
     
     # Wait for this variable to be set.
     variable finished  
@@ -27,39 +40,48 @@ namespace eval ::GotMsg:: {
     set locals(updateDatems)  [expr 1000*60]
 }
 
-proc ::GotMsg::QuitAppHook { } {
+proc ::GotMsg::QuitAppHook {} {
     global  wDlgs
     
     ::UI::SaveWinGeom $wDlgs(jgotmsg)
 }
 
+# @@@ Enable this when solved the uidmsg vs. uuid mixup.
+proc ::GotMsg::MessageHook {xmldata uuid} {
+    global jprefs
+
+    set body   [wrapper::getcdata [wrapper::getfirstchildwithtag $xmldata body]]
+    if {$jprefs(showMsgNewWin) && ($body ne "")} {
+	GotMsg $uuid
+    }
+}
+
 # GotMsg::GotMsg --
 #
 #       Called when we get an incoming message.
 #       Calls 'GotMsg::Show' if not mapped.
 #
 # Arguments:
-#       id          the message id, see Inbox.
+#       uid          the message uuid
 #       
 # Results:
 #       may show message window.
 
-proc ::GotMsg::GotMsg {id} {
+proc ::GotMsg::GotMsg {uid} {
     global  wDlgs
     
     variable w
     variable wbtnext
-    variable msgIdDisplay
-    upvar ::Jabber::jstate jstate
     
-    ::Debug 2 "GotMsg::GotMsg id=$id"
+    ::Debug 2 "GotMsg::GotMsg uid=$uid"
+    
     set w $wDlgs(jgotmsg)
-    
+        
     # Queue up this message or show right away?
     if {[winfo exists $w]} {
-	$wbtnext configure -state normal
+	$wbtnext state {!disabled}
     } else {
-	Show $id
+	Show $uid
     }
 }
 
@@ -68,7 +90,7 @@ proc ::GotMsg::GotMsg {id} {
 #       Fills in all entries etc in message window.
 
 proc ::GotMsg::Show {thisMsgId} {
-    global  prefs
+    global  prefs jprefs
     
     variable w
     variable msgIdDisplay
@@ -83,8 +105,6 @@ proc ::GotMsg::Show {thisMsgId} {
     variable wpresence
     variable body
     variable date
-    upvar ::Jabber::jstate jstate
-    upvar ::Jabber::jprefs jprefs
     
     ::Debug 2 "::GotMsg::Show thisMsgId=$thisMsgId"
     
@@ -92,31 +112,33 @@ proc ::GotMsg::Show {thisMsgId} {
     Build
     
     set msgIdDisplay $thisMsgId
-    ::MailBox::MarkMsgAsRead $thisMsgId
-    
-    set subject [::MailBox::Get $thisMsgId subject]
-    set jid     [::MailBox::Get $thisMsgId from]
-    set date    [::MailBox::Get $thisMsgId date]
-    set body    [::MailBox::Get $thisMsgId message]
-    
-    set jid [::Jabber::JlibCmd getrecipientjid $jid]
+    lassign [::MailBox::GetContentList $thisMsgId] subject jid date body
+    set jid [::Jabber::Jlib getrecipientjid $jid]
     set jidtxt $jid
     set smartdate [::Utils::SmartClockFormat [clock scan $date]]
     
     # Split jid into jid2 and resource.
     jlib::splitjid $jid jid2 res
     
+    set jlib [::Jabber::GetJlib]
+    
     # Use nick name.
-    set displayName [$jstate(roster) getname $jid2]
-    if {[string length $displayName]} {
-	set username "${username} <${jid}>"
+    set rname [$jlib roster getname $jid2]
+    set ujid [jlib::unescapejid $jid]
+    if {[string length $rname]} {
+	set username "$rname <$ujid>"
     } else {
-	set username $jid
+	set username $ujid
     }
-    if {[jlib::jidequal $jid $jstate(mejidres)]} {
+    set myjid [::Jabber::Jlib myjid]
+    if {[jlib::jidequal $jid $myjid]} {
 	set show [::Jabber::GetMyStatus]
     } else {
-	array set presArr [$jstate(roster) getpresence $jid2 -resource $res]
+	if {$res eq ""} {
+	    array set presArr [lindex [$jlib roster getpresence $jid2] 0]
+	} else {
+	    array set presArr [$jlib roster getpresence $jid2 -resource $res]
+	}
 	set show $presArr(-type)
 	if {[info exists presArr(-show)]} {
 	    set show $presArr(-show)
@@ -129,17 +151,18 @@ proc ::GotMsg::Show {thisMsgId} {
     # Insert the actual body of the message.
     $wtext configure -state normal
     $wtext delete 1.0 end
-    ::Text::Parse $wtext $body normal
+    $wtext mark set insert end
+    ::Text::ParseMsg normal $jid $wtext $body normal
     $wtext insert end \n
     $wtext configure -state disabled
     
     # If no more messages after this one...
     if {[::MailBox::IsLastMessage $msgIdDisplay]} {
-	$wbtnext configure -state disabled
+	$wbtnext state {disabled}
     }
     
     # Run display message hook (speech).
-    set opts [list -subject $subject -from $jid -time $date]
+    set opts [list -subject $subject -from $jid -time $date -msgid $thisMsgId]
     eval {::hooks::run displayMessageHook $body} $opts
 }
 
@@ -152,8 +175,8 @@ proc ::GotMsg::Show {thisMsgId} {
 # Results:
 #       shows window.
 
-proc ::GotMsg::Build { } {
-    global  this prefs wDlgs
+proc ::GotMsg::Build {} {
+    global  this prefs wDlgs jprefs
 
     variable w
     variable finished 
@@ -169,83 +192,98 @@ proc ::GotMsg::Build { } {
     variable wbtnext
     variable wpresence
     variable locals
-    upvar ::Jabber::jstate jstate
-    upvar ::Jabber::jprefs jprefs
     
     ::Debug 2 "::GotMsg::Build"
 
     set w $wDlgs(jgotmsg)
     if {[winfo exists $w]} {
+	raise $w
 	return
     }
     set finished 0
     
     # Toplevel with class GotMsg.
-    ::UI::Toplevel $w -class GotMsg -usemacmainmenu 1 -macstyle documentProc
-    wm title $w [mc {Incoming Message}]
+    ::UI::Toplevel $w -class GotMsg \
+      -usemacmainmenu 1 -macstyle documentProc -closecommand ::GotMsg::CloseHook
+    # TRANSLATORS; Window title of popup dialog when receiving a message (instead of a chat message!)
+    wm title $w [mc "Incoming Message"]
     
     set bg [option get . backgroundGeneral {}]
     
     # Global frame.
-    frame $w.frall -borderwidth 1 -relief raised
-    pack  $w.frall -fill both -expand 1 -ipadx 0   
+    ttk::frame $w.frall
+    pack $w.frall -fill both -expand 1
 
-    pack [frame $w.frall.pad -height 8] -side bottom
+    set wbox $w.frall.f
+    ttk::frame $wbox -padding [option get . dialogPadding {}]
+    pack $wbox -fill both -expand 1
 
     # Button part.
-    set frbot [frame $w.frall.frbot -borderwidth 0]
+    set frbot $wbox.b
+    ttk::frame $frbot
     set wbtnext $frbot.btnext
-    set bwidth [expr [::Utils::GetMaxMsgcatWidth Next Reply] + 2]
-    pack [button $wbtnext -text [mc Next] -default active \
-      -width $bwidth -state normal -command [list ::GotMsg::NextMsg]] \
-      -side right -padx 5
-    pack [button $frbot.btreply -text [mc Reply]   \
-      -width $bwidth -command [list ::GotMsg::Reply]]  \
-      -side right -padx 5
-    pack $frbot -side bottom -fill x -padx 10 -pady 0
-    pack [checkbutton $w.frall.ch -text " [mc jainmsgshow]" \
-      -variable ::Jabber::jprefs(showMsgNewWin)] \
-      -side bottom -anchor w -padx 8
+    ttk::button $wbtnext -text [mc "Next"] -default active \
+      -width -8 -command [list ::GotMsg::NextMsg]
+    ttk::button $frbot.btfwd -text [mc "Forward"]   \
+      -width -8 -command [list ::GotMsg::Forward]
+    ttk::button $frbot.btreply -text [mc "Reply"]   \
+      -width -8 -command [list ::GotMsg::Reply]
+    set padx [option get . buttonPadX {}]
 
+    pack $wbtnext -side right
+    pack $frbot.btfwd -side right -padx $padx
+    pack $frbot.btreply -side right
+    pack $frbot -side bottom -fill x
+
+    ttk::checkbutton $wbox.ch -style Small.TCheckbutton \
+      -text [mc "Show incoming messages in this window"] \
+      -variable jprefs(showMsgNewWin)
+    pack  $wbox.ch  -side bottom -anchor w -pady 4
+    
     # From, subject, and time fields.
-    set frmid [frame $w.frall.frmid -borderwidth 0]
-    pack $frmid -side top -fill x -padx 0 -pady 0
+    set frmid $wbox.frmid
+    ttk::frame $frmid
+    pack $frmid -side top -fill x
     
     # From field.
-    set wfrom     $frmid.from
-    set wpresence $wfrom.icon
-    labelframe $wfrom -text [mc From]
-    pack $wfrom -side top -fill both -padx 10 -pady 4
-    entry $wfrom.username -width 10 \
-      -textvariable [namespace current]::username -state disabled \
-      -borderwidth 1 -relief sunken -background $bg
-    pack  $wfrom.username -side left -fill x -expand 1
-    label $wfrom.time -textvariable [namespace current]::smartdate
-    pack  $wfrom.time -side left -padx 8 -pady 0
-    label $wpresence -compound left -image "" \
+    set wfrom     $frmid.lfrom
+    set wpresence $frmid.ifrom
+    ttk::label $frmid.lfrom -text [mc "From"]:
+    ttk::label $frmid.efrom \
+      -textvariable [namespace current]::username
+    ttk::label $frmid.time -style Small.TLabel \
+      -textvariable [namespace current]::smartdate
+    ttk::label $frmid.ifrom -style Small.TLabel \
+      -compound left -compound left -image "" \
       -textvariable [namespace current]::prestext
-    pack  $wpresence -side right -padx 4 -pady 0
-        
+
     # Subject field.
-    pack [frame $frmid.fsub -border 0] -side top -fill x -expand 1 -padx 6
-    pack [label $frmid.fsub.l -text "[mc Subject]:"  \
-      -anchor e] -side left -padx 2 -pady 1
-    pack [entry $frmid.fsub.e -width 10 \
-      -borderwidth 1 -relief sunken -background $bg \
-      -textvariable [namespace current]::subject -state disabled] \
-      -side left -padx 4 -pady 1 -fill x -expand 1
+    ttk::label $frmid.lsub -text [mc "Subject"]: -anchor e
+    ttk::label $frmid.esub \
+      -textvariable [namespace current]::subject
     
+    grid  $frmid.lfrom  $frmid.efrom  $frmid.time   -sticky e -padx 2 -pady 2
+    grid  $frmid.lsub   $frmid.esub   $frmid.ifrom  -sticky e -padx 2 -pady 2
+    grid  $frmid.efrom  $frmid.esub   -sticky w
+    grid  $frmid.time   $frmid.ifrom  -sticky w
+    grid columnconfigure $frmid 1 -weight 1
+        
     # Text.
-    set wtxtfr $w.frall.frtxt
-    pack [frame $wtxtfr] -side top -fill both -expand 1 -padx 4 -pady 4
-    set wtext $wtxtfr.text
-    set wysc $wtxtfr.ysc
-    text $wtext -height 6 -width 48 -wrap word  \
-      -borderwidth 1 -relief sunken -yscrollcommand [list $wysc set]
+    set wtxtfr $wbox.frtxt
+    set wtext  $wtxtfr.text
+    set wysc   $wtxtfr.ysc
+    ttk::frame $wtxtfr -padding {0 4 0 0}
+    pack  $wtxtfr -side top -fill both -expand 1
+    text $wtext -highlightthickness 0 -height 6 -width 48 -wrap word  \
+      -borderwidth 1 -relief sunken \
+      -yscrollcommand [list ::UI::ScrollSet $wysc \
+      [list grid $wysc -column 1 -row 0 -sticky ns]]
     $wtext tag configure normal
-    scrollbar $wysc -orient vertical -command [list $wtext yview]
-    grid $wtext -column 0 -row 0 -sticky news
-    grid $wysc -column 1 -row 0 -sticky ns
+    ttk::scrollbar $wysc -orient vertical -command [list $wtext yview]
+    bindtags $wtext [linsert [bindtags $wtext] 0 ReadOnlyText]
+
+    grid  $wtext  -column 0 -row 0 -sticky news
+    grid  $wysc   -column 1 -row 0 -sticky ns
     grid columnconfigure $wtxtfr 0 -weight 1
     grid rowconfigure $wtxtfr 0 -weight 1
     
@@ -257,17 +295,22 @@ proc ::GotMsg::Build { } {
     set locals(updateDateid) [after $locals(updateDatems) \
       [namespace current]::UpdateDate]
     
+    bind $wtext <<Copy>> {
+	::JUI::CopyEvent %W
+	break
+    }
+    
     # Grab and focus.
     focus $w
 }
 
-proc ::GotMsg::UpdateDate { } {
+proc ::GotMsg::UpdateDate {} {
     variable w
     variable locals
     variable date
     variable smartdate
     
-    if {![winfo exists $w]} {
+    if {![winfo exists $w] || ![info exists date]} {
 	return
     }
     set smartdate [::Utils::SmartClockFormat [clock scan $date]]
@@ -278,14 +321,11 @@ proc ::GotMsg::UpdateDate { } {
 }
 
 proc ::GotMsg::CloseHook {wclose} {
-    global  wDlgs
     variable locals
     
-    if {[string equal $wDlgs(jgotmsg) $wclose]} {
-	::UI::SaveWinGeom $wclose
-	if {$locals(updateDateid) != ""} {
-	    after cancel $locals(updateDateid)
-	}
+    ::UI::SaveWinGeom $wclose
+    if {$locals(updateDateid) != ""} {
+	after cancel $locals(updateDateid)
     }   
 }
 
@@ -319,16 +359,33 @@ proc ::GotMsg::PresenceHook {pjid2 type args} {
     }
 }
 
-proc ::GotMsg::NextMsg { } {
+proc ::GotMsg::NextMsg {} {
     
     variable msgIdDisplay
     
     # Query the mailbox for next message id.
     set nextid [::MailBox::GetNextMsgID $msgIdDisplay]
-    Show $nextid  
+    if {$nextid ne ""} {
+	Show $nextid  
+    }
 }
 
-proc ::GotMsg::Reply { } {
+proc ::GotMsg::Forward {} {
+    variable jid
+    variable subject
+    variable date
+    variable body
+    
+    if {![regexp -nocase {^ *fwd:} $subject]} {
+	set fwdsubject "Fwd: $subject"
+    } else {
+	set fwdsubject $subject
+    }
+    ::NewMsg::Build -subject $fwdsubject -forwardmessage $body -time $date \
+      -from $jid
+}
+
+proc ::GotMsg::Reply {} {
     variable jid
     variable subject
     variable date
