@@ -4,9 +4,22 @@
 #	It maps 'key' (see below) to the local absolute native file path.
 #	Directories may also be time stamped with "best before date".
 #
-#  Copyright (c) 2002-2004  Mats Bengtsson
+#  Copyright (c) 2002-2005  Mats Bengtsson
+#  
+#   This program is free software: you can redistribute it and/or modify
+#   it under the terms of the GNU General Public License as published by
+#   the Free Software Foundation, either version 3 of the License, or
+#   (at your option) any later version.
+#   
+#   This program is distributed in the hope that it will be useful,
+#   but WITHOUT ANY WARRANTY; without even the implied warranty of
+#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#   GNU General Public License for more details.
+#   
+#   You should have received a copy of the GNU General Public License
+#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-# $Id: FileCache.tcl,v 1.19 2004/12/10 10:01:43 matben Exp $
+# $Id: FileCache.tcl,v 1.28 2008-02-06 13:57:25 matben Exp $
 # 
 #       The input key can be: 
 #               1) a full url, must be uri encoded 
@@ -45,7 +58,7 @@
 #       MakeCacheFileName fileName
 #       Dump
 
-package require tinyfileutils
+package require uuid
 package require uriencode
 
 package provide FileCache 1.0
@@ -88,7 +101,7 @@ namespace eval ::FileCache:: {
     
 }
 
-proc ::FileCache::InitHook { } {
+proc ::FileCache::InitHook {} {
     global  prefs this
         
     # Init the file cache settings.
@@ -97,8 +110,8 @@ proc ::FileCache::InitHook { } {
     SetCacheDir $prefs(incomingPath)
 }
 
-proc ::FileCache::QuitHook { } {
-    global  prefs
+proc ::FileCache::QuitHook {} {
+    global  prefs this
     
     # Should we clean up our 'incoming' directory?
     if {$prefs(clearCacheOnQuit)} {
@@ -136,7 +149,7 @@ proc ::FileCache::Set {key {fileName ""}} {
     if {!$readcache} {
 	ReadTable
     }
-    if {$fileName == ""} {
+    if {$fileName eq ""} {
 	set absFileName $key
     } else {
 	if {[string equal [file dirname $fileName] "."]} {
@@ -315,7 +328,7 @@ proc ::FileCache::IsLocal {key} {
     set islocal 0
     if {[regexp {[^:]*://[^/]+/(.*)} $key match path]} {
 	if {![string match "../*" $path]} {
-	    set path [file nativename [uriencode::decodefile $path]]
+	    set path [file nativename [::uri::urn::unquote $path]]
 	    set abspath [file join $basedir $path]
 	    if {[file exists $abspath] && [file isfile $abspath]} {
 		set islocal 1
@@ -366,7 +379,7 @@ proc ::FileCache::MakeCacheFileName {uri} {
     if {![file isdirectory $cachedir]} {
 	return -code error "the cache directory not set"
     }
-    set name [::tfileutils::newuid][file extension $uri]
+    set name [uuid::uuid generate][file extension $uri]
     set cacheToName($name) [file tail $uri]
     set fileName [file join $cachedir $name]
     return $fileName
@@ -401,7 +414,7 @@ proc ::FileCache::HasCacheName {fileName} {
 # An alternative would be to store 'Set $key $value' and
 # then source the file when reading the table.
 
-proc ::FileCache::WriteTable { } {
+proc ::FileCache::WriteTable {} {
     variable indexFile
     variable cache
     variable readcache
@@ -428,7 +441,7 @@ proc ::FileCache::WriteTable { } {
 # 
 #       Reads the cache table and sets our internal state variables.
 
-proc ::FileCache::ReadTable { } {
+proc ::FileCache::ReadTable {} {
     variable indexFile
     variable cache
     variable readcache
@@ -531,7 +544,7 @@ proc ::FileCache::Accept {fileName} {
     return $accept
 }
 
-proc ::FileCache::ClearCache { } {
+proc ::FileCache::ClearCache {} {
     global  prefs
     variable cache
     variable totbytes
@@ -550,7 +563,7 @@ proc ::FileCache::ClearCache { } {
     cd $oldcwd
 }
 
-proc ::FileCache::Dump { } {
+proc ::FileCache::Dump {} {
     variable cache
     
     puts "::FileCache::Dump:"
@@ -561,7 +574,7 @@ proc ::FileCache::Dump { } {
 
 #--- UI part etc ---------------------------------------------------------------
 
-proc ::FileCache::InitPrefsHook { } {
+proc ::FileCache::InitPrefsHook {} {
     global  prefs
     variable maxbytes
     
@@ -570,7 +583,7 @@ proc ::FileCache::InitPrefsHook { } {
     set prefs(checkCache) "launch"
     set prefs(cacheSize)  $maxbytes
 
-    ::PreferencesUtils::Add [list  \
+    ::PrefUtils::Add [list  \
       [list prefs(checkCache)      prefs_checkCache      $prefs(checkCache)] \
       [list prefs(incomingPath)    prefs_incomingPath    $prefs(incomingPath)] \
       [list prefs(cacheSize)       prefs_cacheSize       $prefs(cacheSize)]]
@@ -582,81 +595,90 @@ proc ::FileCache::InitPrefsHook { } {
 
 proc ::FileCache::BuildPrefsHook {wtree nbframe} {
     
-    if {![$wtree isitem Whiteboard]} {
-	$wtree newitem {Whiteboard} -text [mc Whiteboard]
+    if {![::Preferences::HaveTableItem Whiteboard]} {
+	::Preferences::NewTableItem {Whiteboard} [mc "Whiteboard"]
     }
-    $wtree newitem {Whiteboard {File Cache}} -text [mc {File Cache}]
+    # TRANSLATORS: see preferences, whiteboard cache settings
+    ::Preferences::NewTableItem {Whiteboard {File Cache}} [mc "File Cache"]
 
     set wpage [$nbframe page {File Cache}]    
-    ::FileCache::BuildPage $wpage
+    BuildPage $wpage
 }
 
 proc ::FileCache::BuildPage {page} {
     global  prefs
     variable tmpPrefs
         
-    set fontS [option get . fontSmall {}]
-    set fontSB [option get . fontSmallBold {}]
-
     set tmpPrefs(checkCache) $prefs(checkCache)
     set tmpPrefs(mbsize)     [expr wide($prefs(cacheSize)/1e6)]
     
-    set pca $page.fr
-    labelframe $pca -text [mc {File Cache}]
-    pack $pca -side top -anchor w -padx 8 -pady 4
-    message $pca.msg -width 300 -text [mc preffilecache]
-    pack $pca.msg -side top -anchor w -pady 2
+    set wc $page.c
+    ttk::frame $wc -padding [option get . notebookPageSmallPadding {}]
+    pack $wc -side top -anchor [option get . dialogAnchor {}]
 
+    set pca $wc.fr
+    ttk::frame $pca
+    pack  $pca  -side top -anchor w
+    
     set frca $pca.cas
-    pack [frame $frca] -fill x -padx 10
-    pack [label $frca.dsk -text "[mc {Disk Cache}]:"] -side left
-    pack [entry $frca.emb -width 6  \
-      -textvariable [namespace current]::tmpPrefs(mbsize)] -side left
-    pack [label $frca.mb -text [mc {MBytes}]] -side left
-    pack [button $frca.bt -padx 12 -text [mc {Clear Disk Cache Now}] \
-      -command [namespace current]::ClearCache -font $fontS]  \
-      -side right 
+    ttk::frame $frca
+    pack $frca -fill x -pady 2
+    
+    ttk::label $frca.dsk -text [mc "Disk cache size"]:
+    ttk::entry $frca.emb \
+      -width 6 -textvariable [namespace current]::tmpPrefs(mbsize)
+    ttk::label $frca.mb -text MB
+    ttk::button $frca.bt -text [mc "Clear Cache"] \
+      -command [namespace current]::ClearCache
+    
+    pack  $frca.dsk  $frca.emb  $frca.mb  $frca.bt  -side left
 
-    set frfo $pca.fo
-    pack [frame $frfo] -fill x -padx 10
-    pack [label $frfo.fo -text "[mc {Cache folder}]:"] -side left
-    pack [button $frfo.bt -padx 6 -text "[mc {Choose}]..." \
-      -command [namespace current]::SetCachePath -padx 12 -font $fontS]  \
-      -side right 
-    
-    
-    set pwhen $page.frw
-    labelframe $pwhen -text [mc prefcachecmp]
-    pack $pwhen -side top -anchor w -padx 8 -pady 4
-    set frw $pwhen.cas
-    pack [frame $frw] -side left -padx 16 -pady 2
+    ::balloonhelp::balloonforwindow $frca.emb [mc "Amount of cache to store recent whiteboard files to increase speed."]
+
+    if {0} {
+	set frfo $pca.fo
+	pack [ttk::frame $frfo] -pady 2
+	pack [ttk::label $frfo.fo -text "Cache folder":] -side left
+	pack [ttk::button $frfo.bt -text "Choose"... \
+	  -command [namespace current]::SetCachePath]  \
+	  -side right 
+    }
+        
+    set pwhen $wc.frw
+    ttk::labelframe $pwhen -text [mc "Synchronise cache"] \
+      -padding [option get . groupSmallPadding {}]
+    pack $pwhen -side top -fill x -pady 8
     foreach  \
-      val {always       launch             never}   \
-      txt {{Every time} {Once per session} {Never}} {
-	radiobutton $frw.$val -text [mc $txt]   \
+      val { always       launch              never } \
+      txt { Always     "Once per session"  Never } {
+	set Txt [dict create]
+	dict set Val always [mc "Always"]
+	dict set Val launch [mc "Once per session"]
+	dict set Val never [mc "Never"]
+	ttk::radiobutton $pwhen.$val -text [dict get $Val $val]   \
 	  -variable [namespace current]::tmpPrefs(checkCache) -value $val
-	grid $frw.$val -sticky w
+	grid $pwhen.$val -sticky w
     }
 }
 
-proc ::FileCache::SetCachePath { } {
+proc ::FileCache::SetCachePath {} {
     global  this prefs
     
-    set dir [tk_chooseDirectory -title [mc {Pick Cache Folder}] \
+    set dir [tk_chooseDirectory -title [mc "Open Folder"] \
       -initialdir $prefs(incomingPath) -mustexist 1]
-    if {$dir != ""} {
+    if {$dir ne ""} {
 	set prefs(incomingPath) $dir
     }
 }
 
-proc ::FileCache::UserDefaultsHook { } {
+proc ::FileCache::UserDefaultsHook {} {
     global  prefs
     variable tmpPrefs
     
     
 }
 
-proc ::FileCache::SaveHook { } {
+proc ::FileCache::SaveHook {} {
     global  prefs
     variable tmpPrefs
     variable maxbytes
@@ -669,7 +691,7 @@ proc ::FileCache::SaveHook { } {
     SetBestBefore $prefs(checkCache) $prefs(incomingPath)
 }
 
-proc ::FileCache::CancelHook { } {
+proc ::FileCache::CancelHook {} {
     global  prefs
     variable tmpPrefs
     
@@ -680,7 +702,7 @@ proc ::FileCache::CancelHook { } {
     }
 }
 
-proc ::FileCache::UserDefaultsHook { } {
+proc ::FileCache::UserDefaultsHook {} {
     global  prefs
     variable tmpPrefs
     
