@@ -3,139 +3,45 @@
 #      This file is part of The Coccinella application. We collect some handy 
 #      small utility procedures here.
 #      
-#  Copyright (c) 1999-2003  Mats Bengtsson
+#  Copyright (c) 1999-2005  Mats Bengtsson
 #  
-#  See the README file for license, bugs etc.
+#   This program is free software: you can redistribute it and/or modify
+#   it under the terms of the GNU General Public License as published by
+#   the Free Software Foundation, either version 3 of the License, or
+#   (at your option) any later version.
+#   
+#   This program is distributed in the hope that it will be useful,
+#   but WITHOUT ANY WARRANTY; without even the implied warranty of
+#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#   GNU General Public License for more details.
+#   
+#   You should have received a copy of the GNU General Public License
+#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #  
-# $Id: Utils.tcl,v 1.40 2005/01/09 14:33:52 matben Exp $
+# $Id: Utils.tcl,v 1.79 2008-09-14 13:02:27 sdevrieze Exp $
+
+package require uri
+package provide Utils 1.0
 
 namespace eval ::Utils:: {
 
 }
-    
-# InvertArray ---
-#
-#    Inverts an array so that ...
-#    No spaces allowed; no error checking made that the inverse is unique.
-
-proc InvertArray {arrName invArrName} {
-    
-    # Pretty tricky to make it work. Perhaps the new array should be unset?
-    upvar $arrName locArr
-    upvar $invArrName locInvArr
-    foreach name [array names locArr] {
-        set locInvArr($locArr($name)) $name
-    }
-}
-
-# max, min ---
-#
-#    Finds max and min of numerical values. From the WikiWiki page.
-
-proc max {args} {
-    lindex [lsort -real $args] end
-}
-
-proc min {args} {
-    lindex [lsort -real $args] 0
-}
-
-# lprune --
-# 
-#       Removes element from list, silently.
-
-proc lprune {listName elem} {
-    upvar $listName listValue
-    
-    set idx [lsearch $listValue $elem]
-    if {$idx >= 0} {
-	uplevel set $listName [list [lreplace $listValue $idx $idx]]
-    }
-    return ""
-}
-
-# lrevert --
-# 
-#       Revert the order of the list elements.
-
-proc lrevert {args} {
-    set tmp {}
-    set args [lindex $args 0]
-    for {set i [expr [llength $args] - 1]} {$i >= 0} {incr i -1} {
-	lappend tmp [lindex $args $i]
-    }
-    return $tmp
-}
-
-# listintersect --
-# 
-#       Intersections of two lists.
-
-proc listintersect {alist blist} {
-    set tmp {}
-    foreach a $alist {
-	if {[lsearch $blist $a] >= 0} {
-	    lappend tmp $a
-	}
-    }
-    return $tmp
-}
-
-# listintersectnonempty --
-# 
-#       Is intersection of two lists non empty.
-
-proc listintersectnonempty {alist blist} {
-    foreach a $alist {
-	if {[lsearch $blist $a] >= 0} {
-	    return 1
-	}
-    }
-    return 0
-}
-    
-# getdirname ---
-#
-#       Returns the path from 'filePath' thus stripping of any file name.
-#       This is a workaround for the strange [file dirname ...] which strips
-#       off "the last thing."
-#       We need actual files here, not fake ones.
-#    
-# Arguments:
-#       filePath       the path.
-
-proc getdirname {filePath} {
-    
-    if {[file isfile $filePath]} {
-	return [file dirname $filePath]
-    } else {
-	return $filePath
-    }
-}
 
 # SecondCoccinella --
 # 
 #       This gets called if a second instance of this app is launched,
-#       before it kills itself
+#       before it kills itself.
 
 proc SecondCoccinella {args} {
     
-    wm deiconify .
-    raise .
-    
-    ::Debug 2 "--> relaunchHook $args"
+    set w [::JUI::GetMainWindow]
+    wm deiconify $w
+    raise $w
     eval {::hooks::run relaunchHook} $args
 }
 
 #--- Utilities for general usage -----------------------------------------------
 
-namespace eval ::Utils:: {
-    
-    # Running counter for GenerateHexUID.
-    variable uid 0
-    variable maxuidpersec 10000
-}
-
 # ::Utils::FontEqual --
 # 
 #       Compares two fonts to see if identical.
@@ -153,6 +59,16 @@ proc ::Utils::FontEqual {font1 font2} {
     return $ans
 }
 
+proc ::Utils::FontBold {fontName} {    
+    array set fontA [font actual $fontName]
+    return [list $fontA(-family) $fontA(-size) bold]
+}
+
+proc ::Utils::FontItalic {fontName} {    
+    array set fontA [font actual $fontName]
+    return [list $fontA(-family) $fontA(-size) italic]
+}
+
 # ::Utils::GetMaxMsgcatWidth --
 # 
 #       Returns the max string length for the current catalog of the
@@ -217,7 +133,7 @@ proc ::Utils::GetFilePathFromUrl {url} {
     if {[regexp {[^:]+://[^:/]+(:[0-9]+)?/(.*)} $url match port path]} {
 	return $path
     } else {
-	return ""
+	return
     }
 }
 
@@ -226,7 +142,7 @@ proc ::Utils::GetDomainNameFromUrl {url} {
     if {[regexp {[^:]+://([^:/]+)(:[0-9]+)?/.*} $url match domain port]} {
 	return $domain
     } else {
-	return ""
+	return
     }
 }
 
@@ -237,6 +153,9 @@ proc ::Utils::GetDomainNameFromUrl {url} {
 # Arguments:
 #       secs        Number of seconds since system defined time.
 #                   This must be local time.
+#       args:   -detail
+#               -weekdays
+#               -showsecs
 #       
 # Results:
 #       nice time string that still can be used by 'clock scan'
@@ -246,8 +165,13 @@ proc ::Utils::SmartClockFormat {secs args} {
     array set opts {
 	-weekdays 0
 	-detail   0
+	-showsecs 1
+	-today    0
     }
     array set opts $args
+    if {$secs eq ""} {
+	return
+    }
     
     # 'days': 0=today, -1=yesterday etc.
     set secs00 [clock scan "today 00:00"]
@@ -258,7 +182,11 @@ proc ::Utils::SmartClockFormat {secs args} {
 	    set date "tomorrow"
 	}
 	^0$ {
-	    set date "today"
+	    if {$opts(-today)} {
+		set date "today"
+	    } else {
+		set date ""
+	    }
 	}
 	^-1$ {
 	    set date "yesterday"
@@ -275,11 +203,17 @@ proc ::Utils::SmartClockFormat {secs args} {
 	    set date [clock format $secs -format "%y-%m-%d"]
 	}
     }
+    if {$opts(-showsecs)} {
+	set fmt "%H:%M:%S"
+    } else {
+	set fmt "%H:%M"
+    }
+
     if {$opts(-detail) && ($days == 0)} {
 	set now [clock seconds]
 	set minutes [expr ($now - $secs)/60]
 	if {$minutes == 0} {
-	    set time [clock format $secs -format "%H:%M:%S"]
+	    set time [clock format $secs -format $fmt]
 	} elseif {$minutes == 1} {
 	    set time "one minute ago"
 	} elseif {$minutes == 2} {
@@ -289,52 +223,150 @@ proc ::Utils::SmartClockFormat {secs args} {
 	} elseif {$minutes < 60} {
 	    set time "$minutes minutes ago"
 	} else {
-	    set time [clock format $secs -format "%H:%M:%S"]
+	    set time [clock format $secs -format $fmt]
 	}
     } else {	
-	set time [clock format $secs -format "%H:%M:%S"]
+	set time [clock format $secs -format $fmt]
     }
-    return "$date $time"
+    return [string trim "$date $time"]
 }
 
 proc ::Utils::IsToday {secs} {
     return [expr ($secs - [clock scan "today 00:00"])/(60*60*24) >= 0 ? 1 : 0]
 }
 
-proc ::Utils::UnixGetWebBrowser { } {
-    global  this prefs env
+proc ::Utils::FormatBytes {bytes} {
     
-    set browser ""
-    if {$this(platform) == "unix"} {
-	if {[info exists env(BROWSER)]} {
-	    if {[llength [auto_execok $env(BROWSER)]] > 0} {
-		set browser $env(BROWSER)
-	    }
-	}
-	set cmd [auto_execok $prefs(webBrowser)]
-	if {$cmd == {}} {
-	    foreach name {firefox galeon konqueror mozilla-firefox \
-	      mozilla-firebird mozilla netscape iexplorer opera} {
-		if {[llength [set e [auto_execok $name]]] > 0} {
-		    set browser [lindex $e 0]
-		    break
-		}
-	    }
-	}
-	set prefs(webBrowser) $browser
+    if {$bytes < 1000} {
+	set str "$bytes bytes"
+    } elseif {$bytes < 1000000} {
+	set str "[format %.1f [expr $bytes/1000.0] ]Kb"
+    } else {
+	set str "[format %.1f [expr $bytes/1000000.0] ]Mb"
     }
+
+    return $str
+}
+
+# Utils::UnixGetWebBrowser, UnixGetAllWebBrowsers ... --
+# 
+#       These functions get the execution paths to the applications,
+#       not the application names. Always from the auto_execok command.
+
+proc ::Utils::UnixGetWebBrowser {} {
+    global  prefs env
+    
+    set browser {}
+    set e [auto_execok $prefs(webBrowser)]
+    if {[llength $e]} {
+	set browser [lindex $e 0]
+    }
+    if {$browser eq {}} {
+	if {[info exists env(BROWSER)]} {
+	    if {[llength [set e [auto_execok $env(BROWSER)]]]} {
+		set browser [lindex $e 0]
+	    }
+	}
+    }
+    if {$browser eq {}} {
+	set browser [KDEGetBrowser]
+    }
+    if {$browser eq {}} {
+	set browser [lindex [UnixGetAllWebBrowsers] 0]
+    }
+    set prefs(webBrowser) $browser
     return $browser
 }
 
+proc ::Utils::UnixGetAllWebBrowsers {} {
+    set browsers [KDEGetBrowser]
+    foreach name {
+	firefox galeon konqueror mozilla-firefox google-chrome
+	mozilla-firebird mozilla netscape iexplorer opera
+    } {
+	if {[llength [set e [auto_execok $name]]]} {
+	    lappend browsers [lindex $e 0]
+	}
+    }
+    return [lsort -unique $browsers]
+}
+
+proc ::Utils::UnixGetEmailClient {} {
+    global  prefs
+    
+    set mail ""
+    if {$prefs(mailClient) eq "gmail"} {
+	return "gmail"
+    }
+    set e [auto_execok $prefs(mailClient)]
+    if {[llength $e]} {
+	set mail [lindex $e 0]
+    }
+    if {$mail eq {}} {
+	set mail [KDEGetEmailClient]
+    }
+    if {$mail eq {}} {
+	set mail [lindex [UnixGetAllEmailClients] 0]
+    }
+    return $mail
+}
+
+proc ::Utils::UnixGetAllEmailClients {} {
+    set mailers [KDEGetEmailClient]
+    foreach name {thunderbird kmail} {
+	if {[llength [set e [auto_execok $name]]]} {
+	    lappend mailers [lindex $e 0]
+	}
+    }
+    # Special.
+    lappend mailers gmail
+    return [lsort -unique $mailers]
+}
+
+proc ::Utils::KDEGetBrowser {} {
+    set name [KDEGetConfigValue ~/.kde/share/config/kdeglobals BrowserApplication]
+    if {[llength [set e [auto_execok $name]]]} {
+	return $e
+    } else {
+	return {}
+    }
+}
+
+proc ::Utils::KDEGetEmailClient {} {
+    set name [KDEGetConfigValue ~/.kde/share/config/emaildefaults EmailClient]
+    if {[llength [set e [auto_execok $name]]]} {
+	return $e
+    } else {
+	return
+    }
+}
+
+proc ::Utils::KDEGetConfigValue {f key} {
+    
+    set ans ""
+    if {[file exists $f] && [file readable $f]} {
+	set fd [open $f r]
+	set lines [split [read $fd] "\n"]
+	foreach line $lines {
+	    if {[string match "$key=*" $line]} {
+		set RE [format {%s=(.+)$} $key]
+		regexp $RE $line - ans
+	    }
+	}
+	close $fd
+    }
+    return $ans
+}
+
 proc ::Utils::UnixOpenUrl {url} {
     global  prefs
 
-    if {$prefs(webBrowser) == ""} {
+    if {$prefs(webBrowser) eq ""} {
 	set browser [UnixGetWebBrowser]
     } else {
 	set browser $prefs(webBrowser)
     }
-    if {$browser != ""} {
+    if {$browser ne ""} {
 	if {[catch {eval exec $browser -remote \"openURL($url, new-tab)\"}]} {
 	    if {[catch {exec $browser -remote $url}]} {
 		if {[catch {exec $browser $url &}]} {
@@ -343,10 +375,10 @@ proc ::Utils::UnixOpenUrl {url} {
 	    }
 	}
     } 
-    if {$browser == ""} {
-	::UI::MessageBox -icon error -type ok -message \
+    if {$browser eq ""} {
+	::UI::MessageBox -icon error -type ok -title [mc "Error"] -message \
 	  "Couldn't localize a web browser on this system.\
-	  Define a shell variable env(BROWSER) to point to a web browser."
+	  Define a shell variable \"BROWSER\" to point to a web browser."
     }
 }
 
@@ -365,15 +397,12 @@ proc ::Utils::OpenURLInBrowser {url} {
 	windows,* {	    
 	    ::Windows::OpenUrl $url
 	}
-	macintosh,* {    
-	    ::Mac::OpenUrl $url
-	}
     }
 }
 
 
 proc ::Utils::ValidMinutes {str} {
-    if {[regexp {[0-9]*} $str match]} {
+    if {[regexp {^[0-9]*$} $str]} {
 	return 1
     } else {
 	bell
@@ -381,28 +410,6 @@ proc ::Utils::ValidMinutes {str} {
     }
 }
 
-# Utils::GenerateHexUID --
-#
-#       Makes a unique hex string stamped by time.
-#       Can generate max 'maxuidpersec' (uniques) uid's per second.
-#       
-#       See also ::tfileutils::newuid !!!
-
-proc ::Utils::GenerateHexUID { } {
-    variable uid
-    variable maxuidpersec
-    
-    set rem [expr [incr uid] % $maxuidpersec]
-    set secs [clock seconds]
-    
-    # Remove any leading 0 to avoid octal interpretation.
-    set hex1 [format %x [string trimleft \
-      [clock format $secs -format "%y%j%H"] 0]]
-    set hex2 [format %x [string trimleft \
-      [clock format $secs -format "%M%S${rem}"] 0]]
-    return ${hex1}${hex2}
-}
-
 # Utils::GetDirIfExist --
 #
 #       Returns $dir only if actually exists, else returns $this(path).
@@ -435,14 +442,114 @@ proc ::Utils::GetFontListFromName {fontSpec} {
 #       for our built in http server.
 
 proc ::Utils::GetHttpFromFile {filePath} {
-    global  prefs this
-    
-    set relPath [::tfileutils::relative $this(httpdRootPath) $filePath]
-    set relPath [uriencode::quotepath $relPath]
-    set ip [::Network::GetThisPublicIPAddress]
+    global  prefs
+
+    set relPath [::tinyhttpd::mountfile $filePath]
+    set ip [::Network::GetThisPublicIP]
     return "http://${ip}:$prefs(httpdPort)/$relPath"
 }
 
+proc ::Utils::IsAnimatedGif {fileName} {
+    
+    # If we can read -index 1 then this is likely an animated gif. CRUDE!
+    set code [catch {
+	image create photo -file $fileName -format {gif89 -index 1}
+    } name]
+    catch {image delete $name}
+    return [expr {$code == 0 ? 1 : 0}]
+}
+
+proc ::Utils::CreateGif {fileName {imageName ""}} {
+    
+    if {[IsAnimatedGif $fileName]} {
+	return [::anigif::anigif $fileName $imageName]
+    } else {
+	if {$imageName eq ""} {
+	    return [image create photo -file $fileName -format gif]
+	} else {
+	    return [image create photo $imageName -file $fileName -format gif]
+	}
+    }
+}
+
+proc ::Utils::ImageFromData {data {mime ""}} {
+    global  this
+    
+    set type ""
+    lassign [split $mime /] x type
+    if {$type eq "gif"} {
+	return [image create photo -data $data -format gif]
+    } else {
+	if {![catch {image create photo -data $data} name]} {
+	    return $name
+	} 
+	set tmpfile [::tfileutils::tempfile $this(tmpPath) vcard]
+	if {$type ne ""} {
+	    append tmpfile .$type
+	}
+	#puts "tmpfile=$tmpfile"
+	set fd [open $tmpfile {CREAT WRONLY}]
+	fconfigure $fd -translation binary
+	puts -nonewline $fd [::base64::decode $data]
+	close $fd
+	if {![catch {image create photo -file $tmpfile} name]} {
+	    return $name
+	} else {
+	    return
+	}
+    }
+}
+
+# Utils::LanguageMenubutton --
+# 
+#       Make a language menubutton selector.
+
+proc ::Utils::LanguageMenubutton {w varName args} {
+    global  this prefs
+
+    # Add entries here for new message catalogs.
+    array set code2Name {
+	cs    {Čeština}
+	da    {Dansk}
+	de    {Deutsch}
+	en    {English}
+	es    {Castellano}
+	fr    {Français}
+	it    {Italiano}
+	ko    {한국어}
+	nl    {Nederlands}
+	pl    {Polski}
+	pt    {Português}
+	ru    {русский}
+	sv    {Svenska}
+	zh_cn {简体中文}
+    }
+    set langs [list]
+    foreach f [glob -nocomplain -tails -directory $this(msgcatPath) *.msg] {
+	set code [file rootname $f]
+	if {[info exists code2Name($code)]} {
+	    set str $code2Name($code)
+	} else {
+	    set str $code
+	}
+	lappend langs $str
+	set langCode2Str($code) $str
+    }
+    if {$prefs(messageLocale) eq ""} {
+	set $varName [lindex [split [::msgcat::mclocale] _] 0]
+    } else {
+	set $varName $prefs(messageLocale)
+    }
+    set menuDef [list]
+    foreach {code str} [array get langCode2Str] {
+	lappend menuDef [list $str -value $code]
+    }
+    set menuDef [lsort -index 0 $menuDef]
+    ui::optionmenu $w -menulist $menuDef -direction flush -variable $varName
+
+    return $w
+}
+
 #--- Animation -----------------------------------------------------------------
 
 namespace eval ::Utils:: {
@@ -472,7 +579,10 @@ proc ::Utils::AnimateHandle {uid} {
 	set val [lindex $anim($uid,vlist) $pos]
 	regsub -all {\\|&} $val {\\\0} val
 	regsub -all {%v} $cmd $val cmd
-	uplevel #0 $cmd
+	if {[catch {uplevel #0 $cmd}]} {
+	    AnimateStop $uid
+	    return
+	}
 	if {[incr pos] >= [llength $anim($uid,vlist)]} {
 	    set pos 0
 	}
@@ -498,17 +608,44 @@ namespace eval ::Text:: {
     # Unique counter to produce specified link tags.
     variable idurl 1000
         
-    variable urlRegexp {^(http://|https://|www\.|ftp://|ftp\.)([^ \t\r\n]+)}
+    # @@@ Use uri package here!
+    variable urlRE {^(http://|https://|www\.|ftp://|ftp\.)([^ \t\r\n]+)}
+    set httpRE $::uri::http::url
+    set ftpRE $::uri::ftp::url
     variable urlColor
     array set urlColor {fg blue activefg red}
+
+    # Plugin model for parsing URI's
+    variable reguri
+    lappend reguri $urlRE ::Text::UrlButton
 }
 
-# Text::Parse, ... --
-# 
-#       It takes a text widget, the text, and a default tag, and parses
-#       smileys and urls.
+proc ::Text::RegisterURI {re cmd} {
+    variable reguri
+    
+    lappend reguri $re $cmd
+}
 
-proc ::Text::Parse {w str tag} {
+proc ::Text::GetREUriList {} {
+    variable reguri
+    return $reguri
+}
+
+# Text::ParseMsg --
+# 
+#       Parses message text (body).
+#       
+# Arguments:
+#       type        chat|groupchat|normal
+#       jid         JID to send to
+#       w           text widget
+#       str         the text
+#       tagList     current text tag list
+#       
+# Results:
+#       none
+
+proc ::Text::ParseMsg {type jid w str tagList} {
     
     # Split string into words and whitespaces.
     set wsp {[ \t\r\n]+}
@@ -516,9 +653,15 @@ proc ::Text::Parse {w str tag} {
     if {$len == 0} {
 	return
     }
+    
+    # Have hook for complete text.
+    if {[hooks::run textPreParseHook $type $jid $w $str $tagList] eq "stop"} {	    
+	return
+    }
+    
     set start 0
     while {[regexp -start $start -indices -- $wsp $str match]} {
-	foreach {matchStart matchEnd} $match break
+	lassign $match matchStart matchEnd
 	
 	# The "space" part.
 	set space [string range $str $matchStart $matchEnd]
@@ -529,59 +672,78 @@ proc ::Text::Parse {w str tag} {
 	set word [string range $str $start $matchStart]
 	set start $matchEnd
 	
-	# Process the actual word.
-	ParseWord $w $word $tag
+	# Process the actual word:
+	# Run first the hook to see if anyone else wants to parse it.
+	if {[hooks::run textParseWordHook $type $jid $w $word $tagList] ne "stop"} {	    
+	    ParseWord $w $word $tagList
+	}
 	
 	# Insert the whitespace after word.
-	$w insert end $space $tag
+	$w insert insert $space $tagList
     }
     
     # And the final word.
     set word [string range $str $start end]
-    ParseWord $w $word $tag
+    if {[hooks::run textParseWordHook $type $jid $w $word $tagList] ne "stop"} {	    
+	ParseWord $w $word $tagList
+    }
 }
 
-proc ::Text::ParseWord {w word tag} {
+# Text::Parse, ... --
+# 
+#       It takes a text widget, the text, and a default tag, and parses
+#       smileys and urls.
+
+proc ::Text::Parse {w str tagList} {
+
+    ParseMsg {} {} $w $str $tagList
+}
+
+proc ::Text::ParseWord {w word tagList} {
     
     if {[::Emoticons::Exists $word]} {
 	::Emoticons::Make $w $word
-    } elseif {![ParseUrl $w $word]} {
-	$w insert end $word $tag
+    } elseif {![ParseURI $w $word]} {
+	$w insert insert $word $tagList
     }
 }
 
-proc ::Text::ParseUrl {w word} {
-    variable urlRegexp
+proc ::Text::ParseURI {w word} {
+    variable reguri
     variable idurl
     variable urlColor
-    
-    if {[regexp $urlRegexp $word]} {
-	set urltag url${idurl}
-	set urlfg       [option get $w urlForeground       Text]
-	set urlactivefg [option get $w urlActiveForeground Text]
-	if {$urlfg == ""} {
-	    set urlfg $urlColor(fg)
+
+    foreach {re cmd} $reguri {
+	if {[regexp $re $word]} {
+	    set urltag url$idurl
+	    set urlfg       [option get $w urlForeground       Text]
+	    set urlactivefg [option get $w urlActiveForeground Text]
+	    if {$urlfg eq ""} {
+		set urlfg $urlColor(fg)
+	    }
+	    if {$urlactivefg eq ""} {
+		set activefg $urlColor(activefg)
+	    }
+	    $w tag configure $urltag -foreground $urlfg -underline 1
+	    $w tag bind $urltag <Button-1>   \
+	      [list $cmd [string map {% %%} $word]]
+	    $w tag bind $urltag <Any-Enter>  \
+	      [list ::Text::UrlEnter $w $urltag $activefg]
+	    $w tag bind $urltag <Any-Leave>  \
+	      [list ::Text::UrlLeave $w $urltag $urlfg]
+	    set tags [$w tag names "insert - 1 chars"]
+	    $w insert insert $word [concat $tags $urltag]
+	    incr idurl
+	    return 1
 	}
-	if {$urlactivefg == ""} {
-	    set activefg $urlColor(activefg)
-	}
-	$w tag configure $urltag -foreground $urlfg -underline 1
-	$w tag bind $urltag <Button-1>  \
-	  [list ::Text::UrlButton [string map {% %%} $word]]
-	$w tag bind $urltag <Any-Enter>  \
-	  [list ::Text::UrlEnter $w $urltag $activefg]
-	$w tag bind $urltag <Any-Leave>  \
-	  [list ::Text::UrlLeave $w $urltag $urlfg]
-	$w insert end $word $urltag
-	incr idurl
-	return 1
-    } else {
-	return 0
     }
+    return 0
 }
 
 proc ::Text::UrlButton {url} {
-    if {![regexp {^http://.+} $url]} {
+
+    # If no xxx:// assume http://
+    if {![regexp {^[a-z]+://} $url]} {
 	set url "http://$url"
     }
     if {[::Utils::IsWellformedUrl $url]} {
@@ -610,10 +772,10 @@ proc ::Text::InsertURL {w str url tag} {
     set urltag url${idurl}
     set urlfg       [option get $w urlForeground       Text]
     set urlactivefg [option get $w urlActiveForeground Text]
-    if {$urlfg == ""} {
+    if {$urlfg eq ""} {
 	set urlfg $urlColor(fg)
     }
-    if {$urlactivefg == ""} {
+    if {$urlactivefg eq ""} {
 	set activefg $urlColor(activefg)
     }
     $w tag configure $urltag -foreground $urlfg -underline 1
@@ -623,20 +785,28 @@ proc ::Text::InsertURL {w str url tag} {
       [list ::Text::UrlEnter $w $urltag $activefg]
     $w tag bind $urltag <Any-Leave>  \
       [list ::Text::UrlLeave $w $urltag $urlfg]
-    $w insert end $str [list $tag $urltag]
+    $w insert insert $str [list $tag $urltag]
     incr idurl
 }
 
+proc ::Text::TransformSelToPureText {w } {
+    variable puretext
+    
+    unset -nocomplain puretext($w)
+    set puretext($w) ""
+    foreach {key value index} [$w dump sel.first sel.last] {
+	TransformToPureTextCallback $w $key $value $index
+    }
+    return $puretext($w)
+}
+
 proc ::Text::TransformToPureText {w args} {    
     variable puretext
     
-    if {[winfo class $w] != "Text"} {
-	error {TransformToPureText needs a text widget here}
-    }
     unset -nocomplain puretext($w)
     set puretext($w) ""
     foreach {key value index} [$w dump 1.0 end] {
-	::Text::TransformToPureTextCallback $w $key $value $index
+	TransformToPureTextCallback $w $key $value $index
     }
     return $puretext($w)
 }
@@ -673,4 +843,48 @@ proc ::Text::TransformToPureTextCallback {w key value index} {
     }
 }
 
+namespace eval ::Text {
+    
+    variable tagOptions
+    set tagOptions {
+	-background
+	-borderwidth
+	-font
+	-foreground
+	-justify
+	-lmargin1
+	-lmargin2
+	-offset
+	-overstrike
+	-relief
+	-rmargin
+	-spacing1
+	-spacing2
+	-spacing3
+	-tabs
+	-underline
+	-wrap
+    }
+}
+
+proc ::Text::ConfigureTags {w} {
+    variable tagOptions
+
+    set tags [$w tag names]
+
+    foreach tag $tags {
+	set optsL [list]
+	foreach opt $tagOptions {
+	    set optName [string totitle [string trimleft $opt "-"]]
+	    set value [option get $w $tag$optName {}]
+	    if {$value ne ""} {
+		lappend optsL $opt $value
+	    }
+	}
+	if {[llength $optsL]} {
+	    eval {$w tag configure $tag} $optsL
+	}
+    }
+}
+
 #-------------------------------------------------------------------------------
