@@ -3,11 +3,22 @@
 #      This file is part of The Coccinella application. It implements procedures
 #      for transfering the items of a canvas to and from files.
 #      
-#  Copyright (c) 2002-2004  Mats Bengtsson
+#  Copyright (c) 2002-2005  Mats Bengtsson
 #  
-#  See the README file for license, bugs etc.
+#   This program is free software: you can redistribute it and/or modify
+#   it under the terms of the GNU General Public License as published by
+#   the Free Software Foundation, either version 3 of the License, or
+#   (at your option) any later version.
+#   
+#   This program is distributed in the hope that it will be useful,
+#   but WITHOUT ANY WARRANTY; without even the implied warranty of
+#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#   GNU General Public License for more details.
+#   
+#   You should have received a copy of the GNU General Public License
+#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #  
-# $Id: CanvasFile.tcl,v 1.18 2004/12/06 15:27:01 matben Exp $
+# $Id: CanvasFile.tcl,v 1.33 2008-03-25 08:52:31 matben Exp $
  
 package require can2svg
 package require svg2can
@@ -24,24 +35,25 @@ namespace eval ::CanvasFile:: {}
 #       canvas, and closes the file.
 #
 # Arguments:
-#       wtop        the toplevel's namespace (::.main.::)
+#       w           toplevel widget path
 #       filePath    the file with canvas data.
 #       
 # Results:
 #       none
 
-proc ::CanvasFile::DrawCanvasItemFromFile {wtop filePath args} {
+proc ::CanvasFile::DrawCanvasItemFromFile {w filePath args} {
     
-    set wCan [::WB::GetCanvasFromWtop $wtop]
+    set wcan [::WB::GetCanvasFromWtop $w]
 
     # Opens the data file.
     if {[catch {open $filePath r} fd]} {
 	set tail [file tail $filePath]
-	::UI::MessageBox -icon error -type ok -parent $wCan \
-	  -message [mc messfailopread $tail $fd]
+	::UI::MessageBox -icon error -title [mc "Error"] -type ok  \
+	  -message [mc "Cannot read %s: %s" $tail $fd]
 	return
     }
-    eval {::CanvasFile::FileToCanvas $wCan $fd $filePath} $args
+    fconfigure $fd -encoding utf-8
+    eval {FileToCanvas $wcan $fd $filePath} $args
     close $fd
 }
 
@@ -49,20 +61,21 @@ proc ::CanvasFile::DrawCanvasItemFromFile {wtop filePath args} {
 # 
 #       Just a wrapper for FileToCanvas.
 
-proc ::CanvasFile::OpenCanvas {w fileName args} {
+proc ::CanvasFile::OpenCanvas {wcan fileName args} {
     
-    set wtop [::UI::GetToplevelNS $w]
+    set w [winfo toplevel $wcan]
 
     # Opens the data file.
     if {[catch {open $fileName r} fd]} {
 	set tail [file tail $fileName]
-	::UI::MessageBox -message [mc messfailopread $tail $fd] \
-	  -icon error -type ok
+	::UI::MessageBox -message [mc "Cannot read %s: %s" $tail $fd] \
+	  -icon error -title [mc "Error"] -type ok
 	return
     }
-    ::CanvasCmd::DoEraseAll $wtop     
-    ::undo::reset [::WB::GetUndoToken $wtop]
-    eval {FileToCanvas $w $fd $fileName} $args
+    ::CanvasCmd::DoEraseAll $wcan   
+    ::undo::reset [::WB::GetUndoToken $wcan]
+    fconfigure $fd -encoding utf-8
+    eval {FileToCanvas $wcan $fd $fileName} $args
     close $fd
 }
 	  
@@ -85,6 +98,7 @@ proc ::CanvasFile::OpenCanvas {w fileName args} {
 proc ::CanvasFile::FileToCanvas {w fd absPath args} {
     
     # Dispatch to the actual file reader depending on file format version.
+    set version 0
     if {[gets $fd line] >= 0} { 
 	if {![regexp -nocase {^ *# *version: *([0-9]+)} $line match version]} {
 	    set version 1
@@ -114,7 +128,7 @@ proc ::CanvasFile::FileToCanvas {w fd absPath args} {
 #       Lines can also contain 'image create ...' commands.
 #       The file must be opened and file id given as 'fd'.
 
-proc ::CanvasFile::FileToCanvasVer1 {w fd absPath args} {
+proc ::CanvasFile::FileToCanvasVer1 {wcan fd absPath args} {
     global  prefs
     
     Debug 2 "FileToCanvasVer1 absPath=$absPath"
@@ -173,7 +187,6 @@ proc ::CanvasFile::FileToCanvasVer1 {w fd absPath args} {
 	# a complete mess is the result.
 	
 	if {![regexp {(^| )([^/ ]+/[0-9]+)} $tags match junk oldUtag]} {
-	    puts "FileToCanvas:: Warning, didn't match tags! tags=$tags"
 	    continue
 	}
 	if {$updateUtags} {
@@ -198,7 +211,7 @@ proc ::CanvasFile::FileToCanvasVer1 {w fd absPath args} {
 	    # Use the image create command on the previous line if exist.
 	    # Extract the complete file path.
 	    
-	    if {$previousImageOrMovieCmd != ""} {
+	    if {$previousImageOrMovieCmd ne ""} {
 		set ind [lsearch -exact $previousImageOrMovieCmd "-file"]
 		if {$ind >= 0} {
 		    set filePath [lindex $previousImageOrMovieCmd [expr $ind + 1]]
@@ -215,7 +228,6 @@ proc ::CanvasFile::FileToCanvasVer1 {w fd absPath args} {
 		    set filePath [file nativename $filePath]
 		}
 	    } else {
-	        puts "FileToCanvas:: couldn't localize image/window"
 	        continue
 	    }
 		
@@ -239,7 +251,7 @@ proc ::CanvasFile::FileToCanvasVer1 {w fd absPath args} {
 	    set x [lindex $line 2]
 	    set y [lindex $line 3]
 	    set opts [list -coords [list $x $y] -tags $ittag]
-	    if {$zoomOpts != ""} {
+	    if {$zoomOpts ne ""} {
 		lappend opts -zoom-factor $zoomOpts
 	    }
 	    
@@ -250,10 +262,10 @@ proc ::CanvasFile::FileToCanvasVer1 {w fd absPath args} {
 	    }
 	    
 	    # Let the import procedure do the job; manufacture an option list.
-	    ::Import::DoImport $w $opts -file $filePath -where $where
+	    ::Import::DoImport $wcan $opts -file $filePath -where $where
 	    
 	} else {
-	    set wtop [::UI::GetToplevelNS $w]
+	    set w [winfo toplevel $wcan]
 	    
 	    # Draw ordinary item not image nor window (movie).
 	    if {[string equal $where "all"] || [string equal $where "local"]} {
@@ -264,7 +276,7 @@ proc ::CanvasFile::FileToCanvasVer1 {w fd absPath args} {
 		} else {
 		    set cmdlocal $cmdnl
 		}
-		eval {$w} $cmdlocal
+		eval {$wcan} $cmdlocal
 	    }
 	    
 	    # Encode all newlines as \n .
@@ -272,11 +284,11 @@ proc ::CanvasFile::FileToCanvasVer1 {w fd absPath args} {
 	    
 	    # Write to other clients.
 	    if {[string equal $where "all"] || [string equal $where "remote"]} {
-		::CanvasUtils::Command $wtop $cmdoneline remote
+		::CanvasUtils::Command $w $cmdoneline remote
 	    } elseif {![string equal $where "local"]} {
 		
 		# Write only to specified client with ip number 'where'.
-		::CanvasUtils::Command $wtop $cmdoneline $where
+		::CanvasUtils::Command $w $cmdoneline $where
 	    }
 		
 	    if {[string equal $where "all"] || [string equal $where "local"]} {
@@ -284,7 +296,7 @@ proc ::CanvasFile::FileToCanvasVer1 {w fd absPath args} {
 		# Run all registered hookd like speech.
 		if {[string equal $type "text"]} {
 		    ::hooks::run whiteboardTextInsertHook me  \
-		      [$w itemcget $ittag -text]
+		      [$wcan itemcget $ittag -text]
 		}
 	    }
 	}
@@ -298,7 +310,7 @@ proc ::CanvasFile::FileToCanvasVer1 {w fd absPath args} {
 #       Reads a canvas file version 2 into canvas. 
 #       Handles any 'import' commands.
 
-proc ::CanvasFile::FileToCanvasVer2 {w fd absPath args} {
+proc ::CanvasFile::FileToCanvasVer2 {wcan fd absPath args} {
     global  prefs
     
     Debug 2 "FileToCanvasVer2 absPath=$absPath args='$args'"
@@ -323,7 +335,7 @@ proc ::CanvasFile::FileToCanvasVer2 {w fd absPath args} {
 	set updateUtags 0
     }
     set dirPath [file dirname $absPath]
-    set wtop [::UI::GetToplevelNS $w]
+    set w [winfo toplevel $wcan]
     
     set nimports 0
     set undoCmdList {}
@@ -366,17 +378,17 @@ proc ::CanvasFile::FileToCanvasVer2 {w fd absPath args} {
 			set cmdlocal [subst -nocommands -novariables $cmdlocal]
 			set cmdlocal [::CanvasUtils::FontHtmlToPointSize $cmdlocal]
 		    }
-		    eval {$w} $cmdlocal
+		    eval {$wcan} $cmdlocal
 		}
 		
 		# Write to other clients.
 		if {[string equal $where "all"] || \
 		  [string equal $where "remote"]} {
-		    ::CanvasUtils::Command $wtop $line "remote"
+		    ::CanvasUtils::Command $w $line "remote"
 		} elseif {![string equal $where "local"]} {
 		    
 		    # Write only to specified client with ip number 'where'.
-		    ::CanvasUtils::Command $wtop $line $where
+		    ::CanvasUtils::Command $w $line $where
 		}
 		
 		# Speak...
@@ -384,7 +396,7 @@ proc ::CanvasFile::FileToCanvasVer2 {w fd absPath args} {
 		    all - local {
 			if {[string equal $type "text"]} {
 			    ::hooks::run whiteboardTextInsertHook me \
-			      [$w itemcget $utag -text]
+			      [$wcan itemcget $utag -text]
 			}
 		    }
 		}
@@ -413,7 +425,7 @@ proc ::CanvasFile::FileToCanvasVer2 {w fd absPath args} {
 
 		# This is typically an image or movie (QT or Snack).
 		set errMsg [eval {
-		    ::Import::HandleImportCmd $w $line  \
+		    ::Import::HandleImportCmd $wcan $line  \
 		      -addundo 0 -basepath $dirPath  \
 		      -progress [list ::Import::ImportProgress $line]  \
 		      -command  [list ::Import::ImportCommand $line]
@@ -432,40 +444,45 @@ proc ::CanvasFile::FileToCanvasVer2 {w fd absPath args} {
     return $nimports
 }
 
-# CanvasFile::CanvasToFile --
+# CanvasFile::CanvasToChannel --
 #
 #       Writes line by line to file. Each line contains an almost complete 
 #       canvas command except for the widget path. 
 #       The file must be opened and file id given as 'fd'.
 #
 # Arguments:
-#       w                canvas widget path.
+#       wcan             canvas widget path.
 #       fd               file descriptor of the saved file.
 #       absPath          absolute path to the saved file.
+#       args:            -pathtype absolute|relative(D)
+#                        anyone for GetOneLiner*
 #       
 # Results:
 #       none
 
-proc ::CanvasFile::CanvasToFile {w fd absPath args} {
+proc ::CanvasFile::CanvasToChannel {wcan fd absPath args} {
     global  this prefs
     
-    Debug 2 "::CanvasFile::CanvasToFile absPath=$absPath"
+    Debug 2 "::CanvasFile::CanvasToChannel absPath=$absPath"
     
     # When saving images or movies, save relative or absolute path names?
     # It is perhaps best to choose a path relative the actual file path of the 
     # file?
     
-    array set argsArr {
+    array set argsA {
 	-keeputag     1
+	-pathtype     relative
+    }
+    array set argsA $args
+    if {$argsA(-pathtype) eq "relative"} {
+	set argsA(-basepath) $absPath
     }
-    array set argsArr $args
-    set argsArr(-basepath) $absPath
     
     puts $fd "# Version: 2"
     
-    foreach id [$w find all] {
-	set line [eval {::CanvasUtils::GetOneLinerForAny $w $id} \
-	  [array get argsArr]]
+    foreach id [$wcan find all] {
+	set line [eval {::CanvasUtils::GetOneLinerForAny $wcan $id}  \
+	  [array get argsA]]
 	if {$line != {}} {
 	    puts $fd $line		    
 	}
@@ -480,10 +497,11 @@ proc ::CanvasFile::DataToFile {filePath canvasList} {
     
     if {[catch {open $filePath w} fd]} {
 	set tail [file tail $filePath]
-	::UI::MessageBox -message [mc messfailopread $tail $fd] \
-	  -icon error -type ok
+	::UI::MessageBox -message [mc "Cannot read %s: %s" $tail $fd] \
+	  -icon error -title [mc "Error"] -type ok
 	return
     }
+    fconfigure $fd -encoding utf-8
     puts $fd "# Version: 2"
 
     # Be sure to strip off the "CANVAS:" prefix. ???
@@ -504,17 +522,17 @@ proc ::CanvasFile::DataToFile {filePath canvasList} {
 #       open dialog.
 #
 # Arguments:
-#       wtop        toplevel window. (.) If not "." then ".top."; extra dot!
-#       filePath      absolute path to the save file.
+#       wcan        canvas widget
+#       filePath    absolute path to the save file.
 #       
 # Results:
 #       none
 
-proc ::CanvasFile::OpenCanvasFileDlg {wtop {filePath {}}} {
+proc ::CanvasFile::OpenCanvasFileDlg {wcan {filePath {}}} {
     global  prefs this
+    variable regOpen
     
-    set w [::UI::GetToplevel $wtop]
-    set wCan [::WB::GetCanvasFromWtop $wtop]
+    set w [winfo toplevel $wcan]
     
     if {[string length $filePath] == 0} {
 	set typelist {
@@ -522,16 +540,21 @@ proc ::CanvasFile::OpenCanvasFileDlg {wtop {filePath {}}} {
 	    {"XML/SVG"    {.svg}}
 	    {"Text"       {.txt}}
 	}
-	set ans [::UI::MessageBox -icon warning -type okcancel -default ok \
-	  -parent $w -message [mc messcanerasewarn]]
-	if {$ans == "cancel"} {
+
+	foreach {key name} [array get regOpen *,name] {
+	    lappend typelist [list $regOpen($name,label) $regOpen($name,suffix)]
+	}
+
+	set ans [::UI::MessageBox -icon warning -title [mc "Warning"] -type okcancel -default ok \
+	  -parent $w -message [mc "Opening a whiteboard file clears the current whiteboard!"]]
+	if {$ans eq "cancel"} {
 	    return
 	}
 	set userDir [::Utils::GetDirIfExist $prefs(userPath)]
-	set ans [tk_getOpenFile -title [mc {Open Canvas}]  \
+	set ans [tk_getOpenFile -title [mc "Open File"]  \
 	  -filetypes $typelist -defaultextension ".can"  \
 	  -initialdir $userDir]
-	if {$ans == ""} {
+	if {$ans eq ""} {
 	    return
 	}
 	set prefs(userPath) [file dirname $ans]
@@ -539,15 +562,21 @@ proc ::CanvasFile::OpenCanvasFileDlg {wtop {filePath {}}} {
     } else {
 	set fileName $filePath
     }  
+    set ext [file extension $fileName]
     
-    switch -- [file extension $fileName] {
+    switch -- $ext {
 	.svg {
-	    ::undo::reset [::WB::GetUndoToken $wtop]
-	    ::CanvasCmd::DoEraseAll $wtop     
-	    ::CanvasFile::SVGFileToCanvas $wtop $fileName
+	    ::undo::reset [::WB::GetUndoToken $wcan]
+	    ::CanvasCmd::DoEraseAll $wcan
+	    ::CanvasFile::SVGFileToCanvas $w $fileName
 	}
 	.can {	    
-	    OpenCanvas $wCan $fileName
+	    OpenCanvas $wcan $fileName
+	}
+	default {
+	    if {[info exists regOpen($ext,suff-openProc)]} {
+		uplevel #0 $regOpen($ext,suff-openProc) [list $wcan $fileName]
+	    }
 	}
     }
 }
@@ -558,19 +587,19 @@ proc ::CanvasFile::OpenCanvasFileDlg {wtop {filePath {}}} {
 #       displays a Save As dialog instead.
 #       
 # Arguments:
-#       wtop        toplevel window. (.) If not "." then ".top."; extra dot!
+#       wcan        canvas widget
 #       
 # Results:
 #       file save dialog shown if needed, file written.
 
-proc ::CanvasFile::Save {wtop} {
-    upvar ::WB::${wtop}::state state
+proc ::CanvasFile::Save {wcan} {
+    set w [winfo toplevel $wcan]
+    upvar ::WB::${w}::state state
     
-    if {$state(fileName) == ""} {
-	set fileName [::CanvasFile::SaveAsDlg $wtop]
+    if {$state(fileName) eq ""} {
+	set fileName [SaveAsDlg $wcan]
     } else {
-	set wCan [::WB::GetCanvasFromWtop $wtop]
-	set fileName [::CanvasFile::SaveCanvas $wCan $state(fileName)]
+	set fileName [SaveCanvas $wcan $state(fileName)]
     }
     return $fileName
 }
@@ -580,41 +609,72 @@ proc ::CanvasFile::Save {wtop} {
 #       Displays a Save As dialog and acts correspondingly.
 #       
 # Arguments:
-#       wtop        toplevel window. (.) If not "." then ".top."; extra dot!
+#       wcan        canvas widget
 #       
 # Results:
 #       file save dialog shown, file written.
 
-proc ::CanvasFile::SaveAsDlg {wtop} {
-    upvar ::WB::${wtop}::state state
+proc ::CanvasFile::SaveAsDlg {wcan} {
+    set w [winfo toplevel $wcan]
+    upvar ::WB::${w}::state state
     
-    set fileName [::CanvasFile::SaveCanvasFileDlg $wtop]
-    if {$fileName != ""} {
+    set fileName [SaveCanvasFileDlg $wcan]
+    if {$fileName ne ""} {
 	set state(fileName) $fileName
+	set state(unsaved) 0
+	if {([info tclversion] >= 8.5) && ([tk windowingsystem] eq "aqua")} {
+	    wm attributes $w -modified 0
+	}
+	set title [wm title $w]
+	if {[string first " : " $title] < 0} {
+	    wm title $w "$title : [file tail $fileName]"
+	}
+    }
+}
+
+proc ::CanvasFile::IsUnsaved {wcan} {
+    set w [winfo toplevel $wcan]
+    upvar ::WB::${w}::state state
+    
+    return $state(unsaved)
+}
+
+proc ::CanvasFile::SetUnsaved {wcan} {
+    set w [winfo toplevel $wcan]
+    upvar ::WB::${w}::state state
+    
+    set state(unsaved) 1
+    if {([info tclversion] >= 8.5) && ([tk windowingsystem] eq "aqua")} {
+	wm attributes $w -modified 1
     }
 }
 
 # SaveCanvasFileDlg --
 #
 #       Creates a standard file save dialog, opens the file, and calls
-#       'CanvasToFile' to write into it, closes it.
+#       'CanvasToChannel' to write into it, closes it.
 #
 # Arguments:
-#       wtop        toplevel window. (.) If not "." then ".top."; extra dot!
+#       wcan        canvas widget
 #       
 # Results:
 #       fileName or empty
 
-proc ::CanvasFile::SaveCanvasFileDlg {wtop} {
+proc ::CanvasFile::SaveCanvasFileDlg {wcan} {
     global  prefs this
-        
-    set wCan [::WB::GetCanvasFromWtop $wtop]
+    variable regSave
+            
     set typelist {
 	{"Canvas"            {.can}}
 	{"XML/SVG"           {.svg}}
 	{"Postscript File"   {.ps} }
 	{"Text"              {.txt}}
     }
+    
+    foreach {key name} [array get regSave *,name] {
+	lappend typelist [list $regSave($name,label) $regSave($name,suffix)]
+    }
+    
     set userDir [::Utils::GetDirIfExist $prefs(userPath)]
     set opts [list -initialdir $userDir]
     if {$prefs(haveSaveFiletypes)} {
@@ -623,13 +683,13 @@ proc ::CanvasFile::SaveCanvasFileDlg {wtop} {
     if {[string match "mac*" $this(platform)]} {
 	lappend opts -message "Pick .svg suffix for SVG, .can as default"
     }
-    set fileName [eval {tk_getSaveFile -title [mc {Save Canvas}] \
+    set fileName [eval {tk_getSaveFile -title [mc "Save"] \
       -defaultextension ".can"} $opts]
-    if {$fileName == ""} {
-	return ""
+    if {$fileName eq ""} {
+	return
     }
     set prefs(userPath) [file dirname $fileName]
-    ::CanvasFile::SaveCanvas $wCan $fileName
+    SaveCanvas $wcan $fileName
     
     return $fileName
 }
@@ -639,8 +699,9 @@ proc ::CanvasFile::SaveCanvasFileDlg {wtop} {
 #       Write canvas to specified file name taking any specific file
 #       extension as an indication of format.
 
-proc ::CanvasFile::SaveCanvas {wCan fileName args} {
+proc ::CanvasFile::SaveCanvas {wcan fileName args} {
     global  prefs this
+    variable regSave
     
     set ext [file extension $fileName]
 
@@ -648,61 +709,67 @@ proc ::CanvasFile::SaveCanvas {wCan fileName args} {
 	.svg {
 	    
 	    # Not completely sure about -usetags.
-	    ::can2svg::canvas2file $wCan $fileName -uritype file -usetags all \
+	    ::can2svg::canvas2file $wcan $fileName -uritype file -usetags all \
 	      -allownewlines 0  \
 	      -windowitemhandler ::CanvasUtils::GetSVGForeignFromWindowItem
 	}
 	.ps {
-	    eval {$wCan postscript} $prefs(postscriptOpts) {-file $fileName}
+	    eval {$wcan postscript} $prefs(postscriptOpts) {-file $fileName}
 	    if {[string equal $this(platform) "macintosh"]} {
 		file attributes $ans -type TEXT -creator vgrd
 	    }
 	}
 	default {
 	    
-	    # If not .txt make sure it's .can extension.
-	    if {$ext != ".txt"} {
-		set fileName "[file rootname $fileName].can"
+	    if {[info exists regSave($ext,suff-saveProc)]} {
+		uplevel #0 $regSave($ext,suff-saveProc) [list $wcan $fileName]
+	    } else {
+		
+		# If not .txt make sure it's .can extension.
+		if {$ext ne ".txt"} {
+		    set fileName "[file rootname $fileName].can"
+		}
+		
+		# Opens the data file.
+		if {[catch {open $fileName w} fd]} {
+		    set tail [file tail $fileName]
+		    ::UI::MessageBox -icon error -title [mc "Error"] -type ok \
+		      -message [mc "Cannot write to %s: %s" $tail $fd]
+		    return
+		}	    
+		fconfigure $fd -encoding utf-8
+		CanvasToChannel $wcan $fd $fileName
+		close $fd
 	    }
-	    
-	    # Opens the data file.
-	    if {[catch {open $fileName w} fd]} {
-		set tail [file tail $fileName]
-		::UI::MessageBox -icon error -type ok \
-		  -message [mc messfailopwrite $tail $fd]
-		return ""
-	    }	    
-	    ::CanvasFile::CanvasToFile $wCan $fd $fileName
-	    close $fd
 	}
     }
 }
 
-# CanvasCmd::DoSaveAsItem --
+# CanvasFile::DoSaveAsItem --
 # 
 # 
 
-proc ::CanvasCmd::DoSaveAsItem {wtop} {
+proc ::CanvasFile::DoSaveAsItem {wcan} {
     global  prefs this
 	
-    set wCan [::WB::GetCanvasFromWtop $wtop]
     set typelist {
 	{"Canvas"            {.can}}
     }
-    set ans [tk_getSaveFile -title [mc {Save Canvas Item}] \
+    set ans [tk_getSaveFile -title [mc "Save Canvas Item"] \
       -defaultextension ".can" -initialdir $this(altItemPath) \
       -initialfile Untitled.can]
-    if {$ans == ""} {
+    if {$ans eq ""} {
 	return
     }
     set fileName $ans
     if {[catch {open $fileName w} fd]} {
 	set tail [file tail $fileName]
-	::UI::MessageBox -icon error -type ok \
-	  -message [mc messfailopwrite $tail $fd]
+	::UI::MessageBox -icon error -title [mc "Error"] -type ok \
+	  -message [mc "Cannot write to %s: %s" $tail $fd]
 	return
     }	    
-    ::CanvasFile::CanvasToFile $wCan $fd $fileName -keeputag 0
+    fconfigure $fd -encoding utf-8
+    CanvasToChannel $wcan $fd $fileName -keeputag 0
     close $fd
 }
 
@@ -748,7 +815,7 @@ proc ::CanvasFile::ImagePathTranslation {optList absFilePath} {
     set ind [lsearch $optList "-file"]
     if {$ind >= 0} {
 	set absImPath [lindex $optList [expr $ind + 1]]
-	if {$absImPath != ""} {
+	if {$absImPath ne ""} {
 	    set newOptList [lreplace $optList   \
 	      [expr $ind + 1] [expr $ind + 1]   \
 	      [::tfileutils::relative $absFilePath $absImPath]]
@@ -765,28 +832,29 @@ proc ::CanvasFile::ImagePathTranslation {optList absFilePath} {
 # 
 #       Imports an svg file to canvas.
 
-proc ::CanvasFile::SVGFileToCanvas {wtop filePath} {
+proc ::CanvasFile::SVGFileToCanvas {w filePath} {
     
-    set wCan [::WB::GetCanvasFromWtop $wtop]
+    set wcan [::WB::GetCanvasFromWtop $w]
 
     # Opens the data file.
     if {[catch {open $filePath r} fd]} {
 	set tail [file tail $filePath]
-	::UI::MessageBox -icon error -type ok -parent $wCan \
-	  -message [mc messfailopread $tail $fd]
+	::UI::MessageBox -icon error -title [mc "Error"] -type ok -parent $w \
+	  -message [mc "Cannot read %s: %s" $tail $fd]
 	return
     }
+    fconfigure $fd -encoding utf-8
     set xml [read $fd]
     set xmllist [tinydom::documentElement [tinydom::parse $xml]]
     
     # Update the utags...
-    set cmdList [svg2can::parsesvgdocument $xmllist  \
-      -imagehandler [list ::CanvasFile::SVGImageHandler $wtop] \
-      -foreignobjecthandler [list ::CanvasUtils::SVGForeignObjectHandler $wtop]]
+    set cmdList [svg2can::parsesvgdocument $xmllist \
+      -imagehandler [list ::CanvasFile::SVGImageHandler $w] \
+      -foreignobjecthandler [list ::CanvasUtils::SVGForeignObjectHandler $w]]
     foreach cmd $cmdList {
 	set utag [::CanvasUtils::NewUtag]
 	set cmd [::CanvasUtils::ReplaceUtag $cmd $utag]
-	eval $wCan $cmd
+	eval $wcan $cmd
     }
     close $fd
 }
@@ -796,136 +864,107 @@ proc ::CanvasFile::SVGFileToCanvas {wtop filePath} {
 #       Callback for svg to canvas translator to be able to garbage collect
 #       image when window closed.
 
-proc ::CanvasFile::SVGImageHandler {wtop cmd} {
+proc ::CanvasFile::SVGImageHandler {w cmd} {
     
-    #puts "::CanvasFile::SVGImageHandler cmd=$cmd"
     set idx [lsearch -regexp $cmd {-[a-z]+}]
     array set argsArr [lrange $cmd $idx end]
-    return [::WB::CreateImageForWtop $wtop "" -file $argsArr(-file)]
+    return [::WB::CreateImageForWtop $w "" -file $argsArr(-file)]
 }
 
-# Perhaps a mechanism for components to register new open/save formats?
+# CanvasFile::FlattenToDir --
+# 
+#       Takes an existing canvas file and puts it inside 'dir' together
+#       with all its referenced files. Files may be renamed to avoid
+#       name collisions. The 'dir' must already exist and can be a VFS.
 
-proc ::CanvasFile::RegisterOpenFormat {} {
-    
-    
-}
-
-proc ::CanvasFile::RegisterSaveFormat {} {
-    
-    
-}
-
-#
-# These could go in a component for themselves.
-# Only a first incomplete sketch!!!
-#
-
-namespace eval ::CanvasFile:: {
-    
-    variable mkdbuid 0
-}
-
-proc ::CanvasFile::CanvasToMetakit {w fileName} {
-    variable mkdbuid
-
-    package require Mk4tcl
-    
-    set tag canvasmkfile[incr mkdbuid]
-    mk::file open $tag $fileName
-    set view [mk::view layout $tag.canvas command:S file:B data:S]
-
-    foreach id [$w find all] {
-	set line [::CanvasUtils::GetOneLinerForAny $w $id]
-
-	# Replace any -file options with a database entry.
-	set ind [lsearch -exact $line -file]
-	if {$ind >= 0} {
-	    set f [lindex $line [incr ind]]
-	    if {[file exists $f]} {
-		lset line $ind [file tail $f]
-		set cursor [mk::row append $view command $line]
-		set fd [open $f RDONLY]
-		fconfigure $fd -translation binary
-		puts ".........cursor=$cursor"
-		set dbd [mk::channel $cursor file write]
-		set n [fcopy $fd $dbd]
-		puts "n=$n"
-		close $fd
-		close $dbd
-		mk::file commit $tag
-	    }
-	} else {
-	    set cursor [mk::row append $view command $line]
-	}
-    }
-    mk::file close $tag
-}
-
-proc ::CanvasFile::MetakitToCanvas {w fileName} {
+proc ::CanvasFile::FlattenToDir {fileName dir} {
     global  this
-    variable mkdbuid
     
-    package require Mk4tcl
-    
-    set tag canvasmkfile[incr mkdbuid]
-    mk::file open $tag $fileName
-    set views [mk::file views $tag]
-    puts "views=$views"
-    set wtop [::UI::GetToplevelNS $w]
-    
-    # Handle only 'canvas' view.
-    if {[lsearch $views canvas] < 0} {
-	return -code error "no canvas viw in metakit file"
-    }
-    set i 0
-    set view $tag.canvas
-    
-    mk::loop c $view {
-	set line [mk::get $c command]
-	puts "line=$line"
-	set ind [lsearch -exact $line -file]
-	if {$ind >= 0} {
-	    set tail [lindex $line [incr ind]]
-	    puts "..........c=$c"
-	    set dbd [mk::channel $c file read]
-	    set tmp [file join $this(tmpPath) ${tag}${i}${tail}]
-	    set fd [open $tmp {WRONLY CREAT}]
-	    fconfigure $fd -translation binary
-	    set n [fcopy $dbd $fd]
-	    puts "n=$n"
-	    close $fd
-	    close $dbd
-	    lset line $ind $tmp
-	    incr i
-	}
+    set origfd [open $fileName {RDONLY}]
+    fconfigure $origfd -encoding utf-8
 
-	set utag [::CanvasUtils::NewUtag]
-	set line [::CanvasUtils::ReplaceUtag $line $utag]
+    set tmp [::tfileutils::tempfile $this(tmpPath) ""]
+    set tmpfd [open $tmp {CREAT WRONLY}]
+    fconfigure $tmpfd -encoding utf-8
+    
+    # Must process all -file options to point to the ones in 'dir'.
+    while {[gets $origfd line] >= 0} { 
+	if {[regexp {(^ *#|^[ \n\t]*$)} $line]} {
+	    puts $tmpfd $line
+	    continue
+	}
 	set cmd [lindex $line 0]
-	set type [lindex $line 1]
-
-	switch -- $cmd {
-	    create {
-		# If html font sizes, then translate these to point sizes.
-		if {[string equal $type "text"]} {
-		    set cmdlocal [subst -nocommands -novariables $line]
-		    set cmdlocal [::CanvasUtils::FontHtmlToPointSize $cmdlocal]
-		    # Wrong size
-		    set line $cmdlocal
+	if {$cmd ne "import"} {
+	    puts $tmpfd $line
+	    continue
+	}
+	set idx [lsearch -exact $line -file]
+	if {$idx >= 0} {
+	    set src [lindex $line [incr idx]]
+	    	    
+	    # Protect for duplicate tails.
+	    set tail [file tail $src]
+	    if {[info exists tailA($tail)]} {
+		set suff [file extension $src]
+		set root [file rootname $tail]
+		set n 0
+		set newTail ${root}-[incr n]$suff
+		while {[info exists tailA($newTail)]} {
+		    set newTail ${root}-[incr n]$suff
 		}
-		::CanvasUtils::Command $wtop $line
+		set dstTail $newTail
+	    } else {
+		set dstTail $tail
 	    }
-	    import {
-		set errMsg [::Import::HandleImportCmd $w $line -addundo 0  \
-		  -progress [list ::Import::ImportProgress $line]  \
-		  -command  [list ::Import::ImportCommand $line]]
+	    set tailA($dstTail) 1
+	    set dst [file join $dir $dstTail]
 
-	    }
+	    # Map from original file name to vfs file name.
+	    set map($src) $dst
+
+	    lset line $idx $dstTail
+	    puts $tmpfd $line
 	}
     }
+    close $origfd
+    close $tmpfd    
+
+    # Save as an ordinary can file inside vfs.
+    set canFile [file join $dir [file tail $fileName]]
+    file copy $tmp $canFile
     
-    mk::file close $tag
+    # Copy all files. Do not overwrite if already exists.
+    foreach {src dst} [array get map] {
+	if {![file exists $dst]} {
+	    file copy $src $dst
+	}
+    }
 }
 
+# CanvasFile::RegisterSaveFormat, RegisterOpenFormat --
+# 
+#       Mechanisms for components to register new open/save formats.
+
+proc ::CanvasFile::RegisterSaveFormat {name label suffix saveProc} {
+    variable regSave
+    
+    set regSave($name,name)     $name
+    set regSave($name,label)    $label
+    set regSave($name,suffix)   $suffix
+    set regSave($name,saveProc) $saveProc
+    
+    set regSave($suffix,suff-saveProc) $saveProc
+}
+
+proc ::CanvasFile::RegisterOpenFormat {name label suffix openProc} {
+    variable regOpen
+    
+    set regOpen($name,name)     $name
+    set regOpen($name,label)    $label
+    set regOpen($name,suffix)   $suffix
+    set regOpen($name,openProc) $openProc
+
+    set regOpen($suffix,suff-openProc) $openProc
+}
+    
 #-------------------------------------------------------------------------------
