@@ -3,15 +3,24 @@
 #      This file is part of The Coccinella application. It implements the
 #      text commands associated with the text tool.
 #      
-#  Copyright (c) 2000-2004  Mats Bengtsson
+#  Copyright (c) 2000-2005  Mats Bengtsson
 #  
-#  See the README file for license, bugs etc.
+#   This program is free software: you can redistribute it and/or modify
+#   it under the terms of the GNU General Public License as published by
+#   the Free Software Foundation, either version 3 of the License, or
+#   (at your option) any later version.
+#   
+#   This program is distributed in the hope that it will be useful,
+#   but WITHOUT ANY WARRANTY; without even the implied warranty of
+#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#   GNU General Public License for more details.
+#   
+#   You should have received a copy of the GNU General Public License
+#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #  
-# $Id: CanvasText.tcl,v 1.5 2004/11/07 14:22:58 matben Exp $
+# $Id: CanvasText.tcl,v 1.15 2008-03-25 08:52:31 matben Exp $
 
-#  All code in this file is placed in one common namespace.
-
-package require sha1pure
+package require sha1
 
 package provide CanvasText 1.0
 
@@ -28,12 +37,14 @@ namespace eval ::CanvasText:: {
 proc ::CanvasText::Init {wcan} {
     
     namespace eval [namespace current]::${wcan} {
-	set buffer(str) ""
+	set buffer(str)  ""
+	set buffer(text) ""
     }
-    bind $wcan <Destroy> [list [namespace current]::Free %W]
+    # @@@ window destroyed before this!
+    bind $wcan <Destroy> [list [namespace current]::OnDestroy %W]
 }
 
-proc ::CanvasText::Free {wcan} {
+proc ::CanvasText::OnDestroy {wcan} {
     
     EvalBufferedText $wcan
     
@@ -52,62 +63,62 @@ proc ::CanvasText::Free {wcan} {
 #       Typically when the user clicks the text tool button.
 #       
 # Arguments:
-#       w      the canvas widget.
+#       wcan   the canvas widget.
 #       
 # Results:
 #       none
 
 # Could this be done with 'bindtags' instead???
 
-proc ::CanvasText::EditBind {w} {
+proc ::CanvasText::EditBind {wcan} {
     global  this
     
-    $w bind text <Button-1> {
+    $wcan bind text <Button-1> {
 	::CanvasText::Hit %W [%W canvasx %x] [%W canvasy %y]
     }
-    $w bind text <B1-Motion> {
+    $wcan bind text <B1-Motion> {
 	::CanvasText::Drag %W [%W canvasx %x] [%W canvasy %y]
     }
-    $w bind text <Double-Button-1> {
+    $wcan bind text <Double-Button-1> {
 	::CanvasText::SelectWord %W [%W canvasx %x] [%W canvasy %y]
     }
-    $w bind text <Delete> {
+    $wcan bind text <Delete> {
 	::CanvasText::Delete %W
     }
     
     # Swallow any commands on mac's. 
     if {[string match "mac*" $this(platform)]} {
-	$w bind text <Command-Any-Key> {# nothing}
+	$wcan bind text <Command-Any-Key> {# nothing}
 	# Testing... for chinese input method...
-	$w bind text <Command-t> {
+	$wcan bind text <Command-t> {
 	    break
 	}
     }
-    $w bind text <BackSpace> {
+    $wcan bind text <BackSpace> {
 	::CanvasText::Delete %W
     }
-    $w bind text <Return> {
+    $wcan bind text <Return> {
 	::CanvasText::NewLine %W
     }
-    $w bind text <KeyPress> {
+    $wcan bind text <KeyPress> {
 	::CanvasText::Insert %W %A
     }
-    $w bind text <Right> {
+    $wcan bind text <Right> {
 	::CanvasText::MoveRight %W
     }
-    $w bind text <Left> {
+    $wcan bind text <Left> {
 	::CanvasText::MoveLeft %W
     }
-    $w bind text <Up> {
+    $wcan bind text <Up> {
 	::CanvasText::SetCursor %W [::CanvasText::UpDownLine %W -1]
     }
-    $w bind text <Down> {
+    $wcan bind text <Down> {
 	::CanvasText::SetCursor %W [::CanvasText::UpDownLine %W 1]
     }
-    $w bind text <Shift-Left> {
+    $wcan bind text <Shift-Left> {
 	::CanvasText::KeySelect %W [::CanvasText::PrevPos %W insert]
     }
-    $w bind text <Shift-Right> {
+    $wcan bind text <Shift-Right> {
 	::CanvasText::KeySelect %W [::CanvasText::NextPos %W insert]
     }
 
@@ -116,41 +127,41 @@ proc ::CanvasText::EditBind {w} {
     # <KeyPress> class binding will also fire and insert the character,
     # which is wrong.  Ditto for Escape, and Tab.
     
-    $w bind text <Alt-KeyPress> {# nothing}
-    $w bind text <Meta-KeyPress> {# nothing}
-    $w bind text <Control-KeyPress> {# nothing}
-    $w bind text <Escape> {# nothing}
-    $w bind text <KP_Enter> {# nothing}
-    $w bind text <Tab> {# nothing}
+    $wcan bind text <Alt-KeyPress>      {# nothing}
+    $wcan bind text <Meta-KeyPress>     {# nothing}
+    $wcan bind text <Control-KeyPress>  {# nothing}
+    $wcan bind text <Escape>            {# nothing}
+    $wcan bind text <KP_Enter>          {# nothing}
+    $wcan bind text <Tab>               {# nothing}
 
     # Additional emacs-like bindings.
-    $w bind text <Control-d> {
+    $wcan bind text <Control-d> {
 	::CanvasText::Delete %W 1
     }
-    $w bind text <Control-a> {
+    $wcan bind text <Control-a> {
 	::CanvasText::InsertBegin %W
     }
-    $w bind text <Control-e> {
+    $wcan bind text <Control-e> {
 	::CanvasText::InsertEnd %W
     }
     
     # These may interfere with canvas widget bindings for scrolling.
-    $w bind text <Control-Left> {
+    $wcan bind text <Control-Left> {
 	::CanvasText::SetCursor %W [::CanvasText::PrevWord %W]
     }
-    $w bind text <Control-Right> {
+    $wcan bind text <Control-Right> {
 	::CanvasText::SetCursor %W [::CanvasText::NextWord %W]
     }
-    $w bind text <Control-Up> {
+    $wcan bind text <Control-Up> {
 	::CanvasText::SetCursor %W 0
     }
-    $w bind text <Control-Down> {
+    $wcan bind text <Control-Down> {
 	::CanvasText::SetCursor %W end
     }
-    $w bind text <Home> {
+    $wcan bind text <Home> {
 	::CanvasText::InsertBegin %W
     }
-    $w bind text <End> {
+    $wcan bind text <End> {
 	::CanvasText::InsertEnd %W
     }
 }
@@ -161,42 +172,41 @@ proc ::CanvasText::EditBind {w} {
 #       else if text item has focus copy complete text item.
 #       
 # Arguments:
-#       c      the canvas widget.
+#       wcan      the canvas widget.
 #       
 # Results:
 #       none
 
-proc ::CanvasText::Copy {c} {
+proc ::CanvasText::Copy {wcan} {
     variable priv
     
-    Debug 2 "::CanvasText::Copy select item=[$c select item]"
+    Debug 2 "::CanvasText::Copy select item=[$wcan select item]"
 
-    if {[$c select item] != {}}	 { 
+    if {[$wcan select item] ne {}} { 
 	clipboard clear
-	set t [$c select item]
-	set text [$c itemcget $t -text]
-	set start [$c index $t sel.first]
-	set end [$c index $t sel.last]
+	set t [$wcan select item]
+	set text [$wcan itemcget $t -text]
+	set start [$wcan index $t sel.first]
+	set end [$wcan index $t sel.last]
 	set str [string range $text $start $end]
 	clipboard append $str
 	
 	# Keep track of font in clipboard and a hash to see if changed
 	# before pasting it.
-	set priv(font) [$c itemcget $t -font]
-	set priv(sha1) [sha1pure::sha1 "$priv(magic)$str"]
-	#OwnClipboard $c
+	set priv(font) [$wcan itemcget $t -font]
+	set priv(sha1) [sha1::sha1 "$priv(magic)$str"]
+	#OwnClipboard $wcan
     }
-    puts "priv(font)=$priv(font) "
 }
 
-proc ::CanvasText::OwnClipboard {c} {
+proc ::CanvasText::OwnClipboard {wcan} {
     
     # this creates some weird behaviour???
-    selection own -command [list [namespace current]::LostClipboard $c] \
-      -selection CLIPBOARD $c
+    selection own -command [list [namespace current]::LostClipboard $wcan] \
+      -selection CLIPBOARD $wcan
 }
 
-proc ::CanvasText::LostClipboard {c} {
+proc ::CanvasText::LostClipboard {wcan} {
     variable priv
     
     puts "::CanvasText::LostClipboard"
@@ -210,29 +220,29 @@ proc ::CanvasText::LostClipboard {c} {
 #       make a new item.
 #
 # Arguments:
-#       w      the canvas widget.
+#       wcan   the canvas widget.
 #       x,y    the mouse coordinates.
 #       forceNew   make new item regardless if text item under the mouse.
 #       
 # Results:
 #       none
 
-proc ::CanvasText::SetFocus {w x y {forceNew 0}} {
+proc ::CanvasText::SetFocus {wcan x y {forceNew 0}} {
     global  prefs fontSize2Points
     
-    set wtop [::UI::GetToplevelNS $w]
-    upvar ::WB::${wtop}::state state
+    set w [winfo toplevel $wcan]
+    array set state [::WB::GetStateArray $w]
 
-    focus $w
-    set id [::CanvasUtils::FindTypeFromOverlapping $w $x $y "text"]
+    focus $wcan
+    set id [::CanvasUtils::FindIdFromOverlapping $wcan $x $y "text"]
 
     Debug 2 "SetFocus:: id=$id"
     
     # If we have an unsent buffer, be sure to send it first.
     if {$prefs(batchText)} {
-	EvalBufferedText $w
+	EvalBufferedText $wcan
     }
-    if {($id == "") || ([$w type $id] != "text") || $forceNew} {
+    if {($id eq "") || ([$wcan type $id] ne "text") || $forceNew} {
 	
 	# No text item under cursor, make a new empty text item.
 	set utag [::CanvasUtils::NewUtag]
@@ -253,15 +263,16 @@ proc ::CanvasText::SetFocus {w x y {forceNew 0}} {
 	set cmdremote [concat $cmd -font [list $fontsRemote]]
 	set undocmd   [list delete $utag]
 
-	set redo [list ::CanvasUtils::CommandExList $wtop  \
+	set redo [list ::CanvasUtils::CommandExList $w  \
 	  [list [list $cmdlocal local] [list $cmdremote remote]]]
-	set undo [list ::CanvasUtils::Command $wtop $undocmd]
+	set undo [list ::CanvasUtils::Command $w $undocmd]
 	eval $redo
-	undo::add [::WB::GetUndoToken $wtop] $undo $redo
+	undo::add [::WB::GetUndoToken $wcan] $undo $redo
+	::CanvasFile::SetUnsaved $wcan
 
-	$w focus $utag
-	$w select clear
-	$w icursor $utag 0
+	$wcan focus $utag
+	$wcan select clear
+	$wcan icursor $utag 0
     }
 }
 
@@ -271,66 +282,66 @@ proc ::CanvasText::SetFocus {w x y {forceNew 0}} {
 #       with focus. Handles newlines as well.
 #
 # Arguments:
-#       w      the canvas widget.
+#       wcan   the canvas widget.
 #       char   the char or text string to insert.
 #       
 # Results:
 #       none
 
-proc ::CanvasText::Insert {w char} {
+proc ::CanvasText::Insert {wcan char} {
     global  this prefs
     variable priv
         
-    upvar ::CanvasText::${w}::buffer buffer
+    upvar ::CanvasText::${wcan}::buffer buffer
         
     set punct {[.,;?!]}
     set nl_ "\\n"
     
     # First, find out if there are any text item with focus.
     # If not, then make one.
-    if {[$w focus] == ""} {
+    if {[$wcan focus] eq ""} {
 	
     }
-    set wtop [::UI::GetToplevelNS $w]
+    set w [winfo toplevel $wcan]
     
-    # Find the 'itno'.
-    set utag [::CanvasUtils::GetUtag $w focus]
-    if {$utag == "" || $char == ""}	 {
+    # Find the 'utag'.
+    set utag [::CanvasUtils::GetUtag $wcan focus]
+    if {$utag eq "" || $char eq ""}	 {
 	Debug 4 "Insert:: utag == {}"
 	return
     }
-    set itfocus [$w focus]
+    set itfocus [$wcan focus]
     
     # The index of the insertion point.
-    set ind [$w index $itfocus insert]
+    set ind [$wcan index $itfocus insert]
 
     # Mac text bindings: delete selection before inserting.
     if {[string match "mac*" $this(platform)] ||   \
-      ($this(platform) == "windows")} {
+      ($this(platform) eq "windows")} {
 	if {![catch {selection get} s]} {
-	    if {$s != ""} {
-		Delete $w
+	    if {$s ne ""} {
+		Delete $wcan
 		selection clear
 	    }
 	}
     }
     
     # If this is an empty text item then reuse any cached font.
-    if {([$w itemcget $itfocus -text] == "") && ($priv(font) != {})} {
-	if {[string equal $priv(sha1) [sha1pure::sha1 "$priv(magic)$char"]]} {
-	    ::CanvasUtils::ItemConfigure $w $itfocus -font $priv(font)
+    if {([$wcan itemcget $itfocus -text] eq "") && ($priv(font) ne {})} {
+	if {[string equal $priv(sha1) [sha1::sha1 "$priv(magic)$char"]]} {
+	    ::CanvasUtils::ItemConfigure $wcan $itfocus -font $priv(font)
 	}
     }
     
     # The actual canvas text insertion; note that 'ind' is found above.
     set cmd [list insert $itfocus insert $char]
     set undocmd [list dchars $utag $ind [expr $ind + [string length $char]]]
-    set redo [list ::CanvasUtils::Command $wtop $cmd]
-    set undo [list ::CanvasUtils::Command $wtop $undocmd]    
-    eval {$w} $cmd
-    undo::add [::WB::GetUndoToken $wtop] $undo $redo
-        
-    Debug 9 "\t utag = $utag, ind = $ind, char: $char"
+    set redo [list ::CanvasUtils::Command $w $cmd]
+    set undo [list ::CanvasUtils::Command $w $undocmd]    
+    eval {$wcan} $cmd
+    undo::add [::WB::GetUndoToken $wcan] $undo $redo
+
+    ::CanvasFile::SetUnsaved $wcan
     
     # Need to treat the case with actual newlines in char string.
     # Write to all other clients; need to make a one liner first.
@@ -341,14 +352,15 @@ proc ::CanvasText::Insert {w char} {
 	    set buffer(utag) $utag
 	}
 	append buffer(str) $oneliner
+	set buffer(text) [$wcan itemcget $utag -text]
 	
 	if {[string match *${punct}* $char]} {
-	    EvalBufferedText $w
+	    EvalBufferedText $wcan
 	} else {
-	    ScheduleTextBuffer $w
+	    ScheduleTextBuffer $wcan
 	}
     } else {
-	::WB::SendMessageList $wtop [list [list insert $utag $ind $oneliner]]
+	::WB::SendMessageList $w [list [list insert $utag $ind $oneliner]]
     }
 }
 
@@ -360,17 +372,17 @@ proc ::CanvasText::Insert {w char} {
 # w -		The canvas window.
 # pos -		The desired new position for the cursor in the text item.
 
-proc ::CanvasText::SetCursor {w pos} {
+proc ::CanvasText::SetCursor {wcan pos} {
 
-    set id [$w focus]
-    $w select clear
-    $w icursor $id $pos
+    set id [$wcan focus]
+    $wcan select clear
+    $wcan icursor $id $pos
 }
 
-proc ::CanvasText::PrevPos {w pos} {
+proc ::CanvasText::PrevPos {wcan pos} {
     
-    set id [$w focus]
-    set ind [$w index $id $pos]
+    set id [$wcan focus]
+    set ind [$wcan index $id $pos]
     if {$ind == 0} {
 	return 0
     } else {
@@ -378,11 +390,11 @@ proc ::CanvasText::PrevPos {w pos} {
     }
 }
 
-proc ::CanvasText::NextPos {w pos} {
+proc ::CanvasText::NextPos {wcan pos} {
     
-    set id [$w focus]
-    set str [$w itemcget $id -text]
-    set ind [$w index $id $pos]
+    set id [$wcan focus]
+    set str [$wcan itemcget $id -text]
+    set ind [$wcan index $id $pos]
     if {$ind == [string length $str]} {
 	return $ind
     } else {
@@ -390,11 +402,11 @@ proc ::CanvasText::NextPos {w pos} {
     }
 }
 
-proc ::CanvasText::NextWord {w} {
+proc ::CanvasText::NextWord {wcan} {
     
-    set id [$w focus]
-    set str [$w itemcget $id -text]
-    set ind [$w index $id insert]
+    set id [$wcan focus]
+    set str [$wcan itemcget $id -text]
+    set ind [$wcan index $id insert]
     set next [tcl_startOfNextWord $str $ind]
     if {$next == -1} {
 	set next end
@@ -402,11 +414,11 @@ proc ::CanvasText::NextWord {w} {
     return $next
 }
 
-proc ::CanvasText::PrevWord {w} {
+proc ::CanvasText::PrevWord {wcan} {
     
-    set id [$w focus]
-    set str [$w itemcget $id -text]
-    set ind [$w index $id insert]
+    set id [$wcan focus]
+    set str [$wcan itemcget $id -text]
+    set ind [$wcan index $id insert]
     set prev [tcl_startOfPreviousWord $str $ind]
     if {$prev == -1} {
 	set prev 0
@@ -419,31 +431,31 @@ proc ::CanvasText::PrevWord {w} {
 #       Move insert cursor one step to the right.
 #
 # Arguments:
-#       w      the canvas widget.
+#       wcan   the canvas widget.
 #       
 # Results:
 #       none
 
-proc ::CanvasText::MoveRight {w} {
+proc ::CanvasText::MoveRight {wcan} {
     global  this
     
-    set foc [$w focus]
+    set foc [$wcan focus]
     
     # Mac text bindings: remove selection then move insert to end.
     if {[string match "mac*" $this(platform)] ||  \
-      $this(platform) == "windows"} {
+      $this(platform) eq "windows"} {
 	
 	# If selection.
 	if {![catch {selection get} s]} {
-	    if {$s != ""} {
-		$w icursor $foc [expr [$w index $foc sel.last] + 1]
-		$w select clear
+	    if {$s ne ""} {
+		$wcan icursor $foc [expr [$wcan index $foc sel.last] + 1]
+		$wcan select clear
 	    }
 	} else {
-	    $w icursor $foc [expr [$w index $foc insert] + 1]
+	    $wcan icursor $foc [expr [$wcan index $foc insert] + 1]
 	}
     } else {
-	$w icursor $foc [expr [$w index $foc insert] + 1]
+	$wcan icursor $foc [expr [$wcan index $foc insert] + 1]
     }
 }
 
@@ -452,15 +464,15 @@ proc ::CanvasText::MoveRight {w} {
 #       Move insert cursor one step to the left.
 #
 # Arguments:
-#       w      the canvas widget.
+#       wcan   the canvas widget.
 #       
 # Results:
 #       none
 
-proc ::CanvasText::MoveLeft {w} {
+proc ::CanvasText::MoveLeft {wcan} {
     global  this
     
-    set foc [$w focus]
+    set foc [$wcan focus]
     
     # Mac text bindings: remove selection then move insert to first.
     if {[string match "mac*" $this(platform)] ||  \
@@ -468,25 +480,25 @@ proc ::CanvasText::MoveLeft {w} {
 	
 	# If selection.
 	if {![catch {selection get} s]} {
-	    if {$s != ""} {
-		$w icursor $foc [expr [$w index $foc sel.first] + 0]
-		$w select clear
+	    if {$s ne ""} {
+		$wcan icursor $foc [expr [$wcan index $foc sel.first] + 0]
+		$wcan select clear
 	    }
 	} else {
-	    $w icursor $foc [expr [$w index $foc insert] - 1]
+	    $wcan icursor $foc [expr [$wcan index $foc insert] - 1]
 	}
     } else {
-	$w icursor $foc [expr [$w index $foc insert] - 1]
+	$wcan icursor $foc [expr [$wcan index $foc insert] - 1]
     }
 }
 
 # Find index of new character. Only for left justified text.
 
-proc ::CanvasText::UpDownLine {w dir} {
+proc ::CanvasText::UpDownLine {wcan dir} {
     
-    set id [$w focus]
-    set ind [$w index $id insert]
-    set str [$w itemcget $id -text]
+    set id [$wcan focus]
+    set ind [$wcan index $id insert]
+    set str [$wcan itemcget $id -text]
     
     # Up one line.
     if {$dir == -1} {
@@ -525,30 +537,30 @@ proc ::CanvasText::UpDownLine {w dir} {
 #       It moves the cursor to the new position, then extends the selection
 #       to that position.
 
-proc ::CanvasText::KeySelect {w new} {
+proc ::CanvasText::KeySelect {wcan new} {
     
-    set id [$w focus]
-    set insert [$w index $id insert]
+    set id [$wcan focus]
+    set insert [$wcan index $id insert]
     if {$new == $insert} {
 	return
     }
-    if {[$w select item] == ""} {
-	$w select from $id insert
+    if {[$wcan select item] eq ""} {
+	$wcan select from $id insert
 	if {$new >= $insert} {
-	    $w select to $id [expr {$new - 1}]
+	    $wcan select to $id [expr {$new - 1}]
 	} else {
-	    $w select to $id $new 
+	    $wcan select to $id $new 
 	}
     } else {
-	set first [$w index $id sel.first]
-	set last  [$w index $id sel.last]
+	set first [$wcan index $id sel.first]
+	set last  [$wcan index $id sel.last]
 	incr last
 	set right  [expr {$new > $insert} ? 1 : 0]
 	set inside [expr {($new >= $first) && ($new <= $last)} ? 1 : 0]
 	if {$new == $first} {
-	    $w select clear
+	    $wcan select clear
 	} elseif {$new == $last && $right} {
-	    $w select clear
+	    $wcan select clear
 	} else {
 	    if {$new >= $first} {
 		if {$right && $inside} {
@@ -559,39 +571,39 @@ proc ::CanvasText::KeySelect {w new} {
 	    } else {
 		set to $new
 	    }
-	    $w select to $id $to
+	    $wcan select to $id $to
 	}
     }
-    $w icursor $id $new
+    $wcan icursor $id $new
 }
 
-proc ::CanvasText::InsertBegin {w} {
+proc ::CanvasText::InsertBegin {wcan} {
     
-    set foc [$w focus]
+    set foc [$wcan focus]
     
     # Find index of new character. Only for left justified text.
-    set ind [expr [$w index $foc insert] - 1]
-    set str [$w itemcget $foc -text]
+    set ind [expr [$wcan index $foc insert] - 1]
+    set str [$wcan itemcget $foc -text]
     set prevNL [expr [string last \n $str $ind] + 1]
     if {$prevNL == -1} {
-	$w icursor $foc 0
+	$wcan icursor $foc 0
     } else {
-	$w icursor $foc $prevNL
+	$wcan icursor $foc $prevNL
     }
 }
 
-proc ::CanvasText::InsertEnd {w} {
+proc ::CanvasText::InsertEnd {wcan} {
     
-    set foc [$w focus]
+    set foc [$wcan focus]
     
     # Find index of new character. Only for left justified text.
-    set ind [$w index $foc insert]
-    set str [$w itemcget $foc -text]
+    set ind [$wcan index $foc insert]
+    set str [$wcan itemcget $foc -text]
     set nextNL [string first \n $str $ind]
     if {$nextNL == -1} {
-	$w icursor $foc end
+	$wcan icursor $foc end
     } else {
-	$w icursor $foc $nextNL
+	$wcan icursor $foc $nextNL
     }
 }
 
@@ -600,21 +612,21 @@ proc ::CanvasText::InsertEnd {w} {
 #       Called when clicking a text item with the text tool selected.
 #
 # Arguments:
-#       w      the canvas widget.
+#       wcan   the canvas widget.
 #       x,y    the mouse coordinates.
 #       select   
 #       
 # Results:
 #       none
 
-proc ::CanvasText::Hit {w x y {select 1}} {
+proc ::CanvasText::Hit {wcan x y {select 1}} {
 
     Debug 2 "::CanvasText::Hit select=$select"
 
-    $w focus current
-    $w icursor current @$x,$y
-    $w select clear
-    $w select from current @$x,$y
+    $wcan focus current
+    $wcan icursor current @$x,$y
+    $wcan select clear
+    $wcan select from current @$x,$y
 }
 
 # ::CanvasText::Drag --
@@ -622,26 +634,22 @@ proc ::CanvasText::Hit {w x y {select 1}} {
 #       Text selection when dragging the mouse over a text item.
 #
 # Arguments:
-#       w      the canvas widget.
+#       wcan   the canvas widget.
 #       x,y    the mouse coordinates.
 #       
 # Results:
 #       none
 
-proc ::CanvasText::Drag {w x y} {
+proc ::CanvasText::Drag {wcan x y} {
     global  this
     
-    set wtop [::UI::GetToplevelNS $w]
-    ::CanvasCmd::DeselectAll $wtop
-    $w select to current @$x,$y
+    ::CanvasCmd::DeselectAll $wcan
+    $wcan select to current @$x,$y
     
     # Mac text bindings.????
     if {[string match "mac*" $this(platform)]} {
-	#$w focus
+	#$wcan focus
     }
-    
-    # menus
-    ::UI::FixMenusWhenSelection $w
 }
 
 # ::CanvasText::SelectWord --
@@ -649,34 +657,30 @@ proc ::CanvasText::Drag {w x y} {
 #       Typically selects wholw word when double clicking it.
 #
 # Arguments:
-#       w      the canvas widget.
+#       wcan   the canvas widget.
 #       x,y    the mouse coordinates.
 #       
 # Results:
 #       none
 
-proc ::CanvasText::SelectWord {w x y} {
+proc ::CanvasText::SelectWord {wcan x y} {
     
-    set wtop [::UI::GetToplevelNS $w]
-    ::CanvasCmd::DeselectAll $wtop
-    $w focus current
+    ::CanvasCmd::DeselectAll $wcan
+    $wcan focus current
     
-    set id [$w find withtag current]
-    if {$id == ""} {
+    set id [$wcan find withtag current]
+    if {$id eq ""} {
 	return
     }
-    if {[$w type $id] != "text"} {
+    if {[$wcan type $id] ne "text"} {
 	return
     }
-    set txt [$w itemcget $id -text]
-    set ind [$w index $id @$x,$y]
+    set txt [$wcan itemcget $id -text]
+    set ind [$wcan index $id @$x,$y]
     
     # Find the boundaries of the word and select word.
-    $w select from   $id [string wordstart $txt $ind]
-    $w select adjust $id [expr [string wordend $txt $ind] - 1]
-    
-    # menus
-    ::UI::FixMenusWhenSelection $w
+    $wcan select from   $id [string wordstart $txt $ind]
+    $wcan select adjust $id [expr [string wordend $txt $ind] - 1]
 }
 
 # ::CanvasText::NewLine --
@@ -685,36 +689,74 @@ proc ::CanvasText::SelectWord {w x y} {
 #       clients; double escaped.
 #
 # Arguments:
-#       w      the canvas widget.
+#       wcan   the canvas widget.
 #       
 # Results:
 #       none
 
-proc ::CanvasText::NewLine {w} {
-    global  prefs
+proc ::CanvasText::NewLine {wcan} {
+    global  prefs fontPoints2Size
     
     set nl_ "\\n"
-    set wtop [::UI::GetToplevelNS $w]
+    set w [winfo toplevel $wcan]
     
     # Find the 'utag'.
-    set utag [::CanvasUtils::GetUtag $w focus]
-    if {$utag == ""}	 {
+    set utag [::CanvasUtils::GetUtag $wcan focus]
+    if {$utag eq ""}	 {
 	return
     }
     
     # If we are buffering text, be sure to send buffer now if any.
     if {$prefs(batchText)} {
-	EvalBufferedText $w
+	EvalBufferedText $wcan
     }
-    set ind [$w index [$w focus] insert]
-    set cmdlocal [list insert $utag $ind \n]
-    set cmdremote [list insert $utag $ind $nl_]
-    set undocmd [list dchars $utag $ind]
-    set redo [list ::CanvasUtils::CommandExList $wtop  \
-      [list [list $cmdlocal local] [list $cmdremote remote]]]
-    set undo [list ::CanvasUtils::Command $wtop $undocmd]
+    set ind [$wcan index [$wcan focus] insert]
+    if {$prefs(wb,nlNewText)} {
+	set str [$wcan itemcget $utag -text]
+	set right [string range $str $ind end]
+	set cmds {}
+	set undocmds {}
+	if {$right ne ""} {
+	    lappend cmds [list [list dchars $utag $ind end] all]
+	    lappend undocmds [list insert $utag end $right]
+	}
+	
+	array set opts [::CanvasUtils::GetItemOpts $wcan $utag]
+	set utagNew [::CanvasUtils::NewUtag]
+	set opts(-tags) [list std text $utagNew]
+	set opts(-text) $right
+	lassign [$wcan bbox $utag] x1 y1 x2 y2
+	array set fmetrics [font metrics $opts(-font)]
+	set y [expr {$y2 + $fmetrics(-descent)}]
+	
+	# Local command.
+	set cmd [concat create text $x1 $y [array get opts]]
+	lappend cmds [list $cmd local]
+	
+	# Remote command.
+	set size [lindex $opts(-font) 1]
+	lset opts(-font) 1 $fontPoints2Size($size)
+	set cmd [concat create text $x1 $y [array get opts]]
+	lappend cmds [list $cmd remote]
+	lappend undocmds [list delete $utagNew]
+	set redo [list ::CanvasUtils::CommandExList $w $cmds]
+	set undo [list ::CanvasUtils::CommandList $w $undocmds]
+      } else {
+	set cmdlocal [list insert $utag $ind \n]
+	set cmdremote [list insert $utag $ind $nl_]
+	set undocmd [list dchars $utag $ind]	
+	set redo [list ::CanvasUtils::CommandExList $w  \
+	  [list [list $cmdlocal local] [list $cmdremote remote]]]
+	set undo [list ::CanvasUtils::Command $w $undocmd]
+    }
+
     eval $redo
-    undo::add [::WB::GetUndoToken $wtop] $undo $redo
+    undo::add [::WB::GetUndoToken $wcan] $undo $redo
+    ::CanvasFile::SetUnsaved $wcan
+
+    if {$prefs(wb,nlNewText)} {
+	$wcan focus $utagNew
+    }
 }
 
 # ::CanvasText::Delete --
@@ -731,43 +773,44 @@ proc ::CanvasText::NewLine {w} {
 # Results:
 #       none
 
-proc ::CanvasText::Delete {w {offset 0}} {
+proc ::CanvasText::Delete {wcan {offset 0}} {
     global  prefs
     variable priv
     
     Debug 2 "::CanvasText::Delete"
 
-    set idfocus [$w focus]
-    set utag [::CanvasUtils::GetUtag $w focus]
-    if {$utag == ""} {
+    set idfocus [$wcan focus]
+    set utag [::CanvasUtils::GetUtag $wcan focus]
+    if {$utag eq ""} {
 	return
     }
-    set wtop [::UI::GetToplevelNS $w]
+    set w [winfo toplevel $wcan]
 	
     # If we have an unsent buffer, be sure to send it first.
     if {$prefs(batchText)} {
-	EvalBufferedText $w
+	EvalBufferedText $wcan
     }
     
-    if {[$w select item] != ""} {
-	set sfirst [$w index $idfocus sel.first]
-	set slast [$w index $idfocus sel.last]
-	set str [$w itemcget $idfocus -text]
+    if {[$wcan select item] ne ""} {
+	set sfirst [$wcan index $idfocus sel.first]
+	set slast [$wcan index $idfocus sel.last]
+	set str [$wcan itemcget $idfocus -text]
 	set str [string range $str $sfirst $slast]
 	set cmd [list dchars $utag $sfirst $slast]
 	set undocmd [list insert $utag $sfirst $str]
-    } elseif {$idfocus != {}} {
-	set ind [expr [$w index $idfocus insert] - 1 + $offset]
-	set str [$w itemcget $idfocus -text]
+    } elseif {$idfocus ne {}} {
+	set ind [expr [$wcan index $idfocus insert] - 1 + $offset]
+	set str [$wcan itemcget $idfocus -text]
 	set str [string index $str $ind]
 	set cmd [list dchars $utag $ind]
 	set undocmd [list insert $utag $ind $str]
     }
     if {[info exists cmd]} {
-	set redo [list ::CanvasUtils::Command $wtop $cmd]
-	set undo [list ::CanvasUtils::Command $wtop $undocmd]    
+	set redo [list ::CanvasUtils::Command $w $cmd]
+	set undo [list ::CanvasUtils::Command $w $undocmd]    
 	eval $redo
-	undo::add [::WB::GetUndoToken $wtop] $undo $redo
+	undo::add [::WB::GetUndoToken $wcan] $undo $redo
+	::CanvasFile::SetUnsaved $wcan
     }
 }
 
@@ -776,13 +819,13 @@ proc ::CanvasText::Delete {w {offset 0}} {
 #       Unix style paste using button 2.
 #       
 # Arguments:
-#       c      the canvas widget.
+#       wcan    the canvas widget.
 #       x,y
 #       
 # Results:
 #       none
 
-proc ::CanvasText::Paste {c {x {}} {y {}}} {
+proc ::CanvasText::Paste {wcan {x {}} {y {}}} {
     
     Debug 2 "::CanvasText::Paste"
     
@@ -795,7 +838,7 @@ proc ::CanvasText::Paste {c {x {}} {y {}}} {
     Debug 2 "\t CanvasTextPaste:: selection=$_s"
     
     # Once the text string is found use...
-    Insert $c $_s
+    Insert $wcan $_s
 }
 
 # ::CanvasText::ScheduleTextBuffer --
@@ -803,37 +846,38 @@ proc ::CanvasText::Paste {c {x {}} {y {}}} {
 #       Schedules a send operation for our text inserts.
 #       
 # Arguments:
-#       w      the canvas widget.
+#       wcan   the canvas widget.
 #       
 # Results:
 #       none.
 
-proc ::CanvasText::ScheduleTextBuffer {w} {
+proc ::CanvasText::ScheduleTextBuffer {wcan} {
     global  prefs
     
-    upvar ::CanvasText::${w}::buffer buffer
+    upvar ::CanvasText::${wcan}::buffer buffer
     
     if {[info exists buffer(afterid)]} {
 	after cancel $buffer(afterid)
     }
     set buffer(afterid) [after [expr $prefs(batchTextms)]   \
-      [list [namespace current]::EvalBufferedText $w]]
+      [list [namespace current]::EvalBufferedText $wcan]]
 }
 
 # ::CanvasText::EvalBufferedText --
 #
 #       This is the proc where buffered text are sent to clients.
 #       Buffer emptied.
+#       Can be called *after* widget is destroyed!
 #       
 # Arguments:
-#       w      the canvas widget.
+#       wcan   the canvas widget.
 #       
 # Results:
 #       any registered send message hook invoked.
 
-proc ::CanvasText::EvalBufferedText {w} {
+proc ::CanvasText::EvalBufferedText {wcan} {
 
-    upvar ::CanvasText::${w}::buffer buffer
+    upvar ::CanvasText::${wcan}::buffer buffer
     
     if {[info exists buffer(afterid)]} {
 	after cancel $buffer(afterid)
@@ -841,14 +885,14 @@ proc ::CanvasText::EvalBufferedText {w} {
     }
     
     # Run all registered hooks like speech.
+    # @@@ This is not good since it repeates the complete string.
     if {[info exists buffer(utag)] && [string length $buffer(str)]} {
-	set str [$w itemcget $buffer(utag) -text]
-	::hooks::run whiteboardTextInsertHook me $str
+	::hooks::run whiteboardTextInsertHook me $buffer(text)
     }
     
     if {[string length $buffer(str)]} {
-	set wtop [::UI::GetToplevelNS $w]
-	::WB::SendMessageList $wtop  \
+	set w ".[lindex [split $wcan .] 1]"
+	::WB::SendMessageList $w  \
 	  [list [list insert $buffer(utag) $buffer(ind) $buffer(str)]]
     }    
     set buffer(str) ""
