@@ -6,9 +6,20 @@
 #      
 #  Copyright (c) 2003  Mats Bengtsson
 #  
-#  See the README file for license, bugs etc.
+#   This program is free software: you can redistribute it and/or modify
+#   it under the terms of the GNU General Public License as published by
+#   the Free Software Foundation, either version 3 of the License, or
+#   (at your option) any later version.
+#   
+#   This program is distributed in the hope that it will be useful,
+#   but WITHOUT ANY WARRANTY; without even the implied warranty of
+#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#   GNU General Public License for more details.
+#   
+#   You should have received a copy of the GNU General Public License
+#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #  
-# $Id: GetFileIface.tcl,v 1.6 2004/12/06 15:27:01 matben Exp $
+# $Id: GetFileIface.tcl,v 1.17 2008-06-11 08:12:05 matben Exp $
 
 package require getfile
 package require uriencode
@@ -19,6 +30,7 @@ namespace eval ::GetFileIface:: {
     
     # Internal vars only.
     variable uid 0
+    variable noErr 0
 }
 
 # GetFileIface::GetFile --
@@ -27,17 +39,18 @@ namespace eval ::GetFileIface:: {
 #       Wrapper around 'getfile::get' for handling UI etc.
 #       
 # Arguments:
-#       wtop       toplevel for whiteboard
+#       w          toplevel widget path
 #       sock
 #       fileName   the file name (tail), uri encoded.
 #       opts       is a list of '-key value' pairs that contains additional 
 #                  information.
 
-proc ::GetFileIface::GetFile {wtop sock fileName opts} {
-    global prefs noErr wDlgs
+proc ::GetFileIface::GetFile {w sock fileName opts} {
+    global prefs wDlgs
     variable uid
+    variable noErr
         
-    ::Debug 2 "::GetFileIface::GetFile wtop=$wtop, fileName=$fileName"
+    ::Debug 2 "::GetFileIface::GetFile w=$w, fileName=$fileName"
     
     array set optArr $opts    
     if {[info exists optArr(-mime)]} {
@@ -53,7 +66,7 @@ proc ::GetFileIface::GetFile {wtop sock fileName opts} {
     
     # Unquote the disallowed characters according to the RFC for URN scheme.
     # ref: RFC2141 sec2.2
-    set fileTail [::uriencode::decodefile $fileName]
+    set fileTail [::uri::urn::unquote $fileName]
 
     # We store file names with cached names to avoid name clashes.
     set dstpath [::FileCache::MakeCacheFileName $fileTail]
@@ -64,8 +77,8 @@ proc ::GetFileIface::GetFile {wtop sock fileName opts} {
     variable $gettoken
     upvar 0 $gettoken getstate
     
-    set getstate(wtop)        $wtop
-    set getstate(can)         [::WB::GetCanvasFromWtop $wtop]
+    set getstate(w)           $w
+    set getstate(can)         [::WB::GetCanvasFromWtop $w]
     set getstate(sock)        $sock
     set getstate(filetail)    $fileTail
     set getstate(dstpath)     $dstpath
@@ -114,7 +127,7 @@ proc ::GetFileIface::GetFile {wtop sock fileName opts} {
     } token]} {
 	::Debug 2 "\t ::getfile::get failed: $token"
 	set str [::GetFileIface::FormatMessage $gettoken $token]
-	::UI::MessageBox -title [mc {File Transfer Error}]  \
+	::UI::MessageBox -title [mc "File Transfer Error"]  \
 	  -type ok -message $str
 	unset getstate
 	return
@@ -136,7 +149,7 @@ proc ::GetFileIface::GetFile {wtop sock fileName opts} {
 #       Open new temporary socket only for this get operation.
 #       
 # Arguments:
-#       wtop       toplevel for whiteboard
+#       w          toplevel widget path
 #       ip         the ip number of the remote server.
 #       port
 #       path       It must be a pathname relative the servers base 
@@ -145,11 +158,12 @@ proc ::GetFileIface::GetFile {wtop sock fileName opts} {
 #       opts       is a list of '-key value' pairs that contains additional 
 #                  information.
 
-proc ::GetFileIface::GetFileFromServer {wtop ip port path opts} {
-    global prefs wDlgs noErr
+proc ::GetFileIface::GetFileFromServer {w ip port path opts} {
+    global prefs wDlgs
     variable uid
+    variable noErr
     
-    ::Debug 2 "::GetFileIface::GetFileFromServer wtop=$wtop, path=$path"
+    ::Debug 2 "::GetFileIface::GetFileFromServer w=$w, path=$path"
     
     array set optArr $opts
     
@@ -166,7 +180,7 @@ proc ::GetFileIface::GetFileFromServer {wtop ip port path opts} {
     
     # Unquote the disallowed characters according to the RFC for URN scheme.
     # ref: RFC2141 sec2.2
-    set fileTail [::uriencode::decodefile [file tail $path]]
+    set fileTail [::uri::urn::unquote [file tail $path]]
 
     # We store file names with cached names to avoid name clashes.
     set dstpath [::FileCache::MakeCacheFileName $fileTail]
@@ -177,8 +191,8 @@ proc ::GetFileIface::GetFileFromServer {wtop ip port path opts} {
     variable $gettoken
     upvar 0 $gettoken getstate
     
-    set getstate(wtop)       $wtop
-    set getstate(can)        [::WB::GetCanvasFromWtop $wtop]
+    set getstate(w)          $w
+    set getstate(can)        [::WB::GetCanvasFromWtop $w]
     set getstate(filetail)   $fileTail
     set getstate(dstpath)    $dstpath
     set getstate(optlist)    $opts
@@ -215,8 +229,8 @@ proc ::GetFileIface::GetFileFromServer {wtop ip port path opts} {
 	  -progress [list [namespace current]::Progress $gettoken] \
 	  -command [list [namespace current]::Command $gettoken]
     } token]} {
-	set msg [mc {File Transfer Error}]
-	::UI::MessageBox -title [mc {File Transfer Error}]  \
+	set msg [mc "File Transfer Error"]
+	::UI::MessageBox -title [mc "File Transfer Error"]  \
 	  -type ok -message $token
 	unset getstate
 	return
@@ -249,14 +263,14 @@ proc ::GetFileIface::GetFileFromServer {wtop ip port path opts} {
 #       May modify 'gettoken(dstpath)'!
 
 proc ::GetFileIface::Prepare {gettoken fileTail mime opts} {
-    global  prefs noErr this
-    
+    global  prefs  this
+    variable noErr
     upvar #0 $gettoken getstate          
 
     array set optArr $opts
    	
     set doWhat [::Plugins::GetDoWhatForMime $mime]
-    if {$doWhat == ""} {
+    if {$doWhat eq ""} {
 	return 321
     }
     
@@ -269,14 +283,14 @@ proc ::GetFileIface::Prepare {gettoken fileTail mime opts} {
 	ask {
 	    
 	    # 2: Ask user what to do with it.
-	    set ans [::UI::MessageBox -title [mc {Request To User}] \
-	      -type yesno -default yes -message [mc messaskreceive $fileTail]]
+	    set ans [::UI::MessageBox -title [mc "Request To User"] \
+	      -type yesno -default yes -message [mc "We are about to receive the file %s. Do you want to receive it?" $fileTail]]
 	    if {[string equal $ans "no"]} {
 		return 321
 	    } else {
 		set ans [tk_chooseDirectory -initialdir $prefs(incomingPath) \
-		  -title [mc {Pick Directory}]]
-		if {$ans == ""} {
+		  -title [mc "Open Folder"]]]
+		if {$ans eq ""} {
 		    return 321
 		} else {
 		    set getstate(dstpath) [file join $ans $fileTail]
@@ -373,20 +387,18 @@ proc ::GetFileIface::Progress {gettoken token total current} {
 #       Command callback for the getfile package.
 
 proc ::GetFileIface::Command {gettoken token what msg} {
-    global  prefs
     upvar #0 $gettoken getstate          
     
     ::Debug 2 "+\t\t::GetFileIface::Command token=$token, what=$what msg=$msg"
 
-    set wtop $getstate(wtop)
+    set w $getstate(w)
     
     set str [::GetFileIface::FormatMessage $gettoken $msg]
     
     if {[string equal $what "error"]} {
-	::WB::SetStatusMessage $wtop $str
-	if {$prefs(talkative) >= 1} {
-	    ::UI::MessageBox -title [mc {Get File Error}] \
-	      -type ok -message $msg
+	::WB::SetStatusMessage $w $str
+	if {$::config(talkative) >= 1} {
+	    ::UI::MessageBox -title [mc "Error"] -type ok -message $msg
 	}
 	
 	# Perhaps we should show a broken image here???
@@ -398,10 +410,10 @@ proc ::GetFileIface::Command {gettoken token what msg} {
 	unset getstate
 	getfile::cleanup $token
     } elseif {[string equal $what "ok"]} {
-	::WB::SetStatusMessage $wtop $str
+	::WB::SetStatusMessage $w $str
 
 	::Debug 3 "+        status=[::getfile::status $token]"
-	if {[::getfile::status $token] == "ok"} {
+	if {[::getfile::status $token] eq "ok"} {
 	    
 	    # Finished and ok!
 	    ::GetFileIface::DoImport $getstate(mime) $getstate(optlist)  \
@@ -444,7 +456,7 @@ proc ::GetFileIface::FormatMessage {gettoken msg} {
 	}
 	[0-9]* {
 	    set codetext [getfile::ncodetotext $msg]
-	    set msg [mc getnot200 $getstate(fromname) $msg $codetext]
+	    set msg [mc "Received response of the server %s.\n Error code %s\nMessage: %s" $getstate(fromname) $msg $codetext]
 	    set doformat 0
 	}
 	default {
@@ -452,7 +464,18 @@ proc ::GetFileIface::FormatMessage {gettoken msg} {
 	}
     }    
     if {$doformat} {
-	set str [eval {mc get${msg}} $pars]
+	switch -- $msg {
+	    contacting	{set str [eval {mc "Contacting %s. Waiting for response"} $pars]...}
+	    eoferr	{set str [eval {mc "Connection to %s failed: received an eof."} $pars]}
+	    connerr	{set str [eval {mc "Connection to %s failed."} $pars]}
+	    negotiate	{set str [eval {mc "Contacted %s. Negotiating"} $pars]...}
+	    readerr	{set str [eval {mc "Cannot read server response from %s."} $pars]}
+	    unknownprot	{set str [eval {mc "The server at %s did not respond with a well formed protocol."} $pars]}
+	    starts	{set str [eval {mc "%s accepted file %s. Initiating file transfer"} $pars]...}
+	    ended	{set str [eval {mc "Transfer from %s ended prematurely."} $pars]}
+	    finished	{set str [eval {mc "Finished downloading file %s from %s."} $pars]...}
+	    neterr	{set str [eval {mc "Cannot download from %s: network error."} $pars]}
+	}
     } else {
 	set str $msg
     }
@@ -480,17 +503,19 @@ proc ::GetFileIface::UpdateProgress {gettoken total current} {
 
 	# Update the progress window.
 	set percent [expr 100.0 * $current/($total + 1.0)]
-	$getstate(wprog) configure -percent $percent   \
+	$getstate(wprog) configuredelayed -percent $percent   \
 	  -text2 $msg2 -text3 $msg3
     } else {
 	
 	# Create the progress window.
 	::Debug 2 "::GetFileIface::UpdateProgress  create ProgWin"
 
-	set str "[mc {Writing file}]: $getstate(filetail)"
-	::ProgressWindow::ProgressWindow $getstate(wprog)  \
-	  -text $str -text2 $msg2 -text3 $msg3 \
-	  -cancelcmd [list [namespace current]::CancelCmd $gettoken]
+	set str [mc "Writing file"]
+	append str ": $getstate(filetail)"
+	ui::progress::toplevel $getstate(wprog)  \
+	  -text $str -text2 $msg2 -text3 $msg3   \
+	  -menu [::JUI::GetMainMenu]              \
+	  -cancelcommand [list [namespace current]::CancelCmd $gettoken]
     }
 }
 
@@ -502,26 +527,7 @@ proc ::GetFileIface::UpdateProgress {gettoken total current} {
 #       
 
 proc ::GetFileIface::DoImport {mime opts args} {
-    global  prefs
-    
-    ::Debug 3 "+        ::GetFileIface::DoImport"
-
-    if {[string equal $prefs(protocol) "jabber"]} {
-	eval {::Jabber::WB::DispatchToImporter $mime $opts} $args
-    } else {
-	if {[::Plugins::HaveImporterForMime $mime]} {
-	    set servCan [::WB::GetServerCanvasFromWtop .]
-	    set errMsg [eval {
-		::Import::DoImport $servCan $opts
-	    } $args]
-	} else {
-	    set errMsg [mc messfailmimeimp $mime]
-	}
-	if {$errMsg != ""} {
-	    ::UI::MessageBox -title [mc Error] -icon error -type ok \
-	      -message "Failed importing: $errMsg"
-	}
-    }
+    eval {::JWB::DispatchToImporter $mime $opts} $args
 }
 
 # GetFileIface::NewBrokenImage --
@@ -535,7 +541,7 @@ proc ::GetFileIface::NewBrokenImage {code gettoken} {
     
     set msg "Failed importing $getstate(filetail): "
     append msg [getfile::ncodetotext $code]
-    ::WB::SetStatusMessage $getstate(wtop) $msg
+    ::WB::SetStatusMessage $getstate(w) $msg
 
     set opts $getstate(optlist)
     array set optArr $getstate(optlist)
@@ -588,9 +594,9 @@ proc ::GetFileIface::CancelAll { } {
     }
 }
 
-proc ::GetFileIface::CancelAllWtop {wtop} {
+proc ::GetFileIface::CancelAllWtop {w} {
 
-    ::Debug 2 "+::GetFileIface::CancelAllWtop wtop=$wtop"
+    ::Debug 2 "+::GetFileIface::CancelAllWtop w=$w"
     
     # Close and clean up.
     set gettokenList [concat  \
@@ -601,8 +607,8 @@ proc ::GetFileIface::CancelAllWtop {wtop} {
     foreach gettoken $gettokenList {
 	upvar #0 $gettoken getstate          
 
-	if {[info exists getstate(wtop)] &&  \
-	  [string equal $getstate(wtop) $wtop]} {
+	if {[info exists getstate(w)] &&  \
+	  [string equal $getstate(w) $w]} {
 	    set tok $getstate(token)
 	    getfile::reset $tok
 	    getfile::cleanup $tok
