@@ -2,14 +2,25 @@
 #      
 #  Copyright (c) 1999-2003  Mats Bengtsson
 #  
-# $Id: Multicast.tcl,v 1.4 2004/12/02 08:22:35 matben Exp $
+#   This program is free software: you can redistribute it and/or modify
+#   it under the terms of the GNU General Public License as published by
+#   the Free Software Foundation, either version 3 of the License, or
+#   (at your option) any later version.
+#   
+#   This program is distributed in the hope that it will be useful,
+#   but WITHOUT ANY WARRANTY; without even the implied warranty of
+#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#   GNU General Public License for more details.
+#   
+#   You should have received a copy of the GNU General Public License
+#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#  
+# $Id: Multicast.tcl,v 1.10 2007-09-14 13:17:09 matben Exp $
 
 package provide Multicast 1.0
 
 namespace eval ::Multicast:: {
     
-    namespace export OpenMulticast
-    
     variable uid 0
     variable txtvarEntMulticast
     variable selMulticastName
@@ -21,13 +32,12 @@ namespace eval ::Multicast:: {
 #       Makes dialog to open streaming audio/video.
 #   
 # Arguments:
-#       wtop        toplevel window. (.) If not "." then ".top."; extra dot!
-#       w           the toplevel dialog.
+#       wcan        canvas widget
 #       
 # Results:
 #       shows dialog.
 
-proc ::Multicast::OpenMulticast {wtop} {
+proc ::Multicast::OpenMulticast {wcan} {
     global  prefs this wDlgs
     
     variable uid
@@ -39,54 +49,58 @@ proc ::Multicast::OpenMulticast {wtop} {
     set w $wDlgs(openMulti)[incr uid]
     ::UI::Toplevel $w -macstyle documentProc -usemacmainmenu 1 \
       -macclass {document closeBox}
-    wm title $w [mc {Open Stream}]
-    set fontSB [option get . fontSmallBold {}]
-    
-    # Global frame.
-    frame $w.frall -borderwidth 1 -relief raised
-    pack  $w.frall -fill both -expand 1
-    
-    # Labelled frame.
-    set wcfr $w.frall.fr
-    labelframe $wcfr -text [mc openquicktime]
-    pack $wcfr -side top -fill both -padx 8 -pady 4 -ipadx 10 -ipady 6 -in $w.frall
-    
-    # Overall frame for whole container.
-    set frtot [frame $wcfr.frin]
-    pack $frtot
-    label $frtot.lbltop -text [mc writeurl] -font $fontSB
+    # TRANSLATORS: old whiteboard feature
+    wm title $w [mc "Open Stream"]
+
     set shorts [lindex $prefs(shortsMulticastQT) 0]
-    set optMenu [eval {tk_optionMenu $frtot.optm  \
-      [namespace current]::selMulticastName} $shorts]
-    $frtot.optm configure -highlightthickness 0 -foreground black
-    #set selMulticastName [lindex [lindex $prefs(shortsMulticastQT) 0] 0]
-    label $frtot.lblhttp -text {http://} -font $fontSB
-    entry $frtot.entip -width 60   \
-      -textvariable [namespace current]::txtvarEntMulticast
-    message $frtot.msg -borderwidth 0 -aspect 500 \
-      -text [mc openquicktimeurlmsg]
-    grid $frtot.lbltop -column 0 -row 0 -sticky sw -padx 0 -pady 2 -columnspan 2
-    grid $frtot.optm -column 2 -row 0 -sticky e -padx 2 -pady 2
-    grid $frtot.lblhttp -column 0 -row 1 -sticky e -padx 0 -pady 6
-    grid $frtot.entip -column 1 -row 1 -columnspan 2 -sticky w -padx 0 -pady 6
-    grid $frtot.msg -column 0 -row 2 -columnspan 3 -padx 4 -pady 2 -sticky news
+
+    # Global frame.
+    ttk::frame $w.frall
+    pack $w.frall -fill both -expand 1
+
+    set wbox $w.frall.f
+    ttk::frame $wbox -padding [option get . dialogPadding {}]
+    pack $wbox -fill both -expand 1
+
+    # Labelled frame.
+    set frtot $wbox.fr
+    ttk::labelframe $frtot -padding [option get . groupSmallPadding {}] \
+      -text [mc "Open QuickTime live stream"]
+    pack $frtot -side top -fill both
     
+    ttk::label $frtot.lbltop -text [mc "Write URL or choose from shortcut"]:
+    eval {ttk::optionmenu $frtot.optm  \
+      [namespace current]::selMulticastName} $shorts
+    ttk::label $frtot.lblhttp -text {http://}
+    ttk::entry $frtot.entip -width 40  \
+      -textvariable [namespace current]::txtvarEntMulticast
+    ttk::label $frtot.msg -style Small.TLabel \
+      -wraplength 400 -justify left -text [mc "Open a URL which contains a SDP file with extension .mov for a QuickTime realtime live streaming sending. Can be audio (radio) or video (TV). Alternatively, use your web browser to find the SDP file for a live transmission, download it on disk and open it is an ordinary movie."]
+
+    grid  $frtot.lbltop   -             $frtot.optm  -padx 2 -pady 2 -sticky w
+    grid  $frtot.lblhttp  $frtot.entip  -            -padx 2 -pady 2 -sticky e
+    grid  $frtot.msg      -             -            -sticky ew
+    
+    grid  $frtot.optm  $frtot.entip  -sticky ew
+    grid columnconfigure $frtot 1 -weight 1
+        
     # Button part.
-    set frbot [frame $w.frall.frbot -borderwidth 0]  
-    pack [button $frbot.btconn -text [mc Open] -default active  \
-      -command [list Multicast::OpenMulticastQTStream $wtop $frtot.entip]]  \
-      -side right -padx 5 -pady 5
-    pack [button $frbot.btcancel -text [mc Cancel]  \
-      -command "set [namespace current]::finished 0"]  \
-      -side right -padx 5 -pady 5
-    pack [button $frbot.btedit -text "[mc Edit]..."   \
-      -command [list Multicast::DoAddOrEditQTMulticastShort edit $frtot.optm]]  \
-      -side right -padx 5 -pady 5
-    pack [button $frbot.btadd -text "[mc Add]..."   \
-      -command [list Multicast::DoAddOrEditQTMulticastShort add $frtot.optm]]  \
-      -side right -padx 5 -pady 5
-    pack $frbot -side top -fill both -expand 1 -in $w.frall  \
-      -padx 8 -pady 6
+    set frbot $wbox.b
+    ttk::frame $frbot -padding [option get . okcancelTopPadding {}]
+    ttk::button $frbot.btconn -text [mc "Open"] -default active  \
+      -command [list Multicast::OpenMulticastQTStream $wcan $frtot.entip]
+    ttk::button $frbot.btcancel -text [mc "Cancel"]  \
+      -command [list set [namespace current]::finished 0]
+    ttk::button $frbot.btedit -text [mc "Edit"]...   \
+      -command [list ::Multicast::DoAddOrEditQTMulticastShort edit $frtot.optm]
+    ttk::button $frbot.btadd -text [mc "Add"]...   \
+      -command [list ::Multicast::DoAddOrEditQTMulticastShort add $frtot.optm]
+    set padx [option get . buttonPadX {}]
+    pack  $frbot.btconn  -side right
+    pack  $frbot.btcancel  -side right -padx $padx
+    pack  $frbot.btedit  -side right
+    pack  $frbot.btadd  -side right -padx $padx
+    pack  $frbot  -side bottom -fill x
     
     wm resizable $w 0 0
     
@@ -159,14 +173,12 @@ proc ::Multicast::TraceSelMulticastName {name junk1 junk2} {
 #       Initiates a separate download of the tiny SDR file with http.
 #   
 # Arguments:
-#       wtop        toplevel window. (.) If not "." then ".top."; extra dot!
+#       wcan        canvas widget
 
-proc ::Multicast::OpenMulticastQTStream {wtop wentry} {
+proc ::Multicast::OpenMulticastQTStream {wcan wentry} {
     global  this prefs
     variable finished
 
-    set wCan [::WB::GetCanvasFromWtop $wtop]
-
     # Patterns.
     set proto_ {[^:]+}
     set domain_ {[A-Za-z0-9\-\_\.]+}
@@ -183,10 +195,10 @@ proc ::Multicast::OpenMulticastQTStream {wtop wentry} {
     unset -nocomplain port
     if {![regexp -nocase "($proto_)://($domain_)(:($port_))?($path_)$"  \
       $url match protocol domain junk port path]} {
-	::UI::MessageBox -message   \
+	::UI::MessageBox -title [mc "Error"] -message   \
 	  "Inconsistent url=$url." -icon error -type ok
 	set finished 0
-	return ""
+	return
     }
     if {[string length $port] == 0} {
 	set port 80
@@ -199,35 +211,32 @@ proc ::Multicast::OpenMulticastQTStream {wtop wentry} {
     if {[string length $fileTail] == 0} {
 	tk_dialog .wrfn "No Path" "No file name in path." \
 	  error 0 Cancel
-	return ""
+	return
     }
     
     # This is opened as an ordinary movie.
-    set anchor [::CanvasUtils::NewImportAnchor $wCan]
-    ::Import::DoImport $wCan $anchor -url $url
+    set anchor [::CanvasUtils::NewImportAnchor $wcan]
+    ::Import::DoImport $wcan $anchor -url $url
 }
 
 proc ::Multicast::CleanupMulticastQTStream {wtop fid fullName token} { 
 
     upvar #0 $token state    
 
-    set wCan [::WB::GetCanvasFromWtop $wtop]
+    set wcan [::WB::GetCanvasFromWtop $wtop]
     set no_ {^2[0-9]+}
     catch {close $fid}
-    
-    # Waiting is over.
-    ::WB::StartStopAnimatedWaveOnMain 0
-    
+        
     # Access state as a Tcl array.
     # Check errors. 
     if {[info exists state(status)] &&  \
       [string equal $state(status) "timeout"]} {
-	::UI::MessageBox -icon error -type ok -message   \
-	  "Timout event for url=$state(url)" 
+	::UI::MessageBox -title [mc "Error"] -icon error -type ok \
+	  -message "Timout event for url=$state(url)" 
 	return
     } elseif {[info exists state(status)] &&  \
       ![string equal $state(status) "ok"]} {
-	::UI::MessageBox -icon error -type ok -message   \
+	::UI::MessageBox -title [mc "Error"] -icon error -type ok -message   \
 	  "Not ok return code from url=$state(url); status=$state(status)"	  
 	return
     }
@@ -235,22 +244,22 @@ proc ::Multicast::CleanupMulticastQTStream {wtop fid fullName token} {
     # The http return status. Must be 2**.
     set httpCode [lindex $state(http) 1]
     if {![regexp "$no_" $httpCode]} {
-	::UI::MessageBox -icon error -type ok \
+	::UI::MessageBox -title [mc "Error"] -icon error -type ok \
 	  -message "Failed open url=$url. Returned with code: $httpCode."
     }
     
     # Check that type of data is the wanted. Check further.
     if {[info exists state(type)] &&  \
       [string equal $state(type) "video/quicktime"]} {
-	::UI::MessageBox -icon error -type ok -message \
+	::UI::MessageBox -title [mc "Error"] -icon error -type ok -message \
 	  "Not correct file type returned from url=$state(url); \
 	  filetype=$state(type); expected video/quicktime."
 	return
     }
     
     # This is opened as an ordinary movie.
-    set anchor [::CanvasUtils::NewImportAnchor $wCan]
-    ::Import::DoImport $wCan "$anchor" -file $fullName  \
+    set anchor [::CanvasUtils::NewImportAnchor $wcan]
+    ::Import::DoImport $wcan "$anchor" -file $fullName  \
       -where "local"
     set fileTail [file tail $fullName]
     ::WB::SetStatusMessage $wtop "Opened streaming live multicast: $fileTail."
